package Modules::Document;

# Document : a simple document manager

=pod

=head1 Modules::Document - document management web applet

The Document Tool provides a simple document manager that easily integrates
into ExSite-driven websites.  It has the following features:

=over 4

=item * paginated reports of documents that are available for download

=item * title, description, filename, type, date, size are reported 

=item * sorted by date

=item * add, update, delete functions for administrators

=item * versioning (using ExSite's CMS tools)

=item * will create the document library if it doesn't exist

=item * can use alternate documemt libraries

=item * webservice-ready

=item * specify documents that we want to open in a new window (newwin=mime-type)

=back

=cut


###  Usage:
###  <!--&Document(<parameters>)-->
###
###  Parameters:
###  lib: name of the library
###  hide_file_info: hide file size and type
###	accept value: 1 or 0
###  hide_file_postdate: hide file post date
###	accept value: 1 or 0
###
###  Examples:
###  Show a list of files with complete file info:
###  <!--&Document(lib=<library_name>)-->
###
###  Show a list of files without file info:
###  <!--&Document(lib=<library_name>&hide_file_info=1)-->
###
###  Show a list of files without file post date:
###  <!--&Document(lib=<library_name>&hide_file_postdate=1)-->
###
###  Sort by content_id, name, description, mtime or ctime
###  <!--&Document(lib=<library_name>&sort=<sort_field>)-->
###
###  Show in ascending order(default is descending)
###  <!--&Document(lib=<library_name>&asc=1)-->


###  TODO:
###  
###  - search
###  - built-in publish function
###  - shortcut to CMS
###  - paginated control panel

use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::HTML;
use ExSite::ML;
use ExSite::Content;
use ExSite::Section;
use Modules::BaseDCD;
use MIME::Base64;

use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD);

sub write {
    my ($this,$options) = @_;
    my $in = $this->{input};  # set by BaseDCD::read
    my %opt = &DecodeString($options);
    # Save input parameters
    $this->{write_options} = \%opt;
    $this->{libname} = $opt{lib} || "docs";
	$this->{newwin} = $opt{newwin};
	$this->{sort} = $opt{sort} || "content_id";
	$this->{asc} = $opt{asc} || 0;
    my $action = $opt{doccmd} || $in->{doccmd} || "list";
    if ($action eq "add") {
	return $this->add;
    }
    elsif ($action eq "update") {
	return $this->update($in->{docid});
    }
    elsif ($action eq "upload") {
	return $this->upload;
    }
    return $this->list;
}

# ioctl : all operations that don't qualify as reads or writes
#         invoked as <!--BaseDCD.ioctl(request)-->

sub ioctl {
    my $this = shift;
    $_ = shift;   # $_ is the ioctl request
    if (/ModuleName/) {
	return "Documents";
    }
    elsif (/ControlPanel/) {
	return \&ctrl_panel;
    }
    elsif (/Category/) {
	return "CMS";
    }
    elsif (/DynContentParam/){
    	return $this->getDynContentParameter(shift);
    }
}

sub getDynContentParameter{
	my ($this,$sectionid) = @_;
	return undef if !$sectionid;
	my $s = new ExSite::Section(id=>$sectionid);
	my @libs = $s->fetch_libraries();
	my @libopt = map { "<option value=\"lib=$_->{filename}\">$_->{title}</option>" } @libs;

	my $ml = new ExSite::ML;
	return $ml->table( [
			    [ 
			      $ml->strong("Select a library:"), 
			      $ml->select(join("\n",@libopt),{name=>"param"}) 
			      ],
			    ] );
}

sub ctrl_panel {
    my $this = shift;
    my $db = $share{DB};
    my $in = $this->{input};
    $this->{admin} = 1;
    my $out = $this->set_section_id;
    $this->{section_id} = $this->get_section_id;

    return $out if (! $this->{section_id});

    my @liblinks;
    my @doclib = $share{DB}->fetch_match("page",{type=>"library",
						 section_id=>$this->{section_id}});

    my ($nlib,$thislib);
    $nlib = 0;
    foreach my $lib (@doclib) {
	push @liblinks, {url=>$this->link(lib=>$lib->{filename},
					  docid=>undef,doccmd=>undef),
			 label=>$lib->{title}};
	if (! $in->{lib} && $lib->{filename} eq "docs") {
	    $thislib = $nlib;
	}
	elsif ($in->{lib} eq $lib->{filename}) {
	    $thislib = $nlib;
	}
	$nlib++;
    }
    my $ml = new ExSite::ML;
    my $newlibicon = $ml->img(undef,{src=>"$config{server}{HTMLpath}/_Modules/Document/update.gif",height=>12,width=>12,alt=>"new library",border=>0});
    push @liblinks, {url=>$this->link(lib=>undef,docid=>undef,
				      doccmd=>"newlib"),
		     label=>"$newlibicon New library"};
    
    my ($thissite,@sitemenu,$nsite,$pane);
    $this->{libname} = $in->{lib} || "docs";
    my $lib = $this->getlib;
    
    if (! $lib) {
	# hmm, no library
	if ($in->{doccmd} eq "makelib") {
	    # create one
	    my $rlib = {section_id=>$this->{section_id},
			title=>"Documents",
			filename=>$this->{libname},
			type=>"library",
			status=>"active",
			access=>"public",};
	    my $lid = $db->insert("page",$rlib);
	    if ($lid) {
		$lib = $this->{lib} = $rlib;
	    }
	    else {
		$pane = $db->error($msg{"failed to create document library"});
	    }
	}
	else {
	    my $ml = new ExSite::ML;
	    my $section = $db->fetch("section",$this->{section_id});
	    $pane .= $ml->h1($msg{"Library Not Found"});
	    $pane .= $ml->p(
			    &substitute($msg{"A document library named \"[[libname]]\" was not found in [[title]]."},
					{
					    libname=>$this->{libname},
					    title=>$section->{title}
					}
					)
			    );
	    if ($this->{libname} eq "docs") {
		# if this is the default library, ask if we should create one
		my $makeurl = $this->link(doccmd=>"makelib");
		$pane .= $ml->p(
				$ml->a($msg{"Click here to make document library"},
				       {href=>$makeurl})
				);
	    }
	}
    }

    # options

    elsif ($in->{doccmd} eq "newlib") {
	$pane = $this->newlib;
    }
    elsif ($in->{doccmd} eq "update") {
	$pane = $this->update($in->{docid});
    }
    elsif ($in->{doccmd} eq "add") {
	$pane = $this->add();
    }
    elsif ($in->{doccmd} eq "edit") {
	$pane = $this->edit();
    }
    elsif ($in->{doccmd} eq "del") {
	$pane = $this->delete($in->{docid});
    }
    elsif ($in->{doccmd} eq "upload") {
	$pane = $this->upload();
    }
    else {
	# show the library contents, and administrator options
	$pane = $this->list;
    }

    $out .= &ExSite::HTML::MenuBox(
				   menu=>\@liblinks,
				   thistab=>$thislib,
				   pane=>$pane,
				   );
    return $out;
}

# get document library

sub getlib {
    my ($this) = @_;
    return $this->{lib} if ($this->{lib} && $this->{lib}{filename} eq $this->{input}{lib}); 
    if (! exists $this->{section_id}) {
	$this->{section_id} = $this->get_section_id;
    }
    return undef if (! $this->{section_id});

    my @doclib;
    my $section_id = $this->{section_id};

    while (scalar @doclib == 0 && $section_id) {
	@doclib = $share{DB}->fetch_match("page",{filename=>$this->{libname},
						  type=>"library",
						  section_id=>$section_id});
	if (scalar @doclib == 0) {
	    my $s = new ExSite::Section(id=>$section_id);
	    $section_id = $s->get_my("parent_id");
	}
    }

    if (@doclib > 0) {
	$this->{lib} = $doclib[0];
    }
    else {
	$this->{lib} = undef;
    }
    return $this->{lib};
}

sub listlib {
    my $this = shift;
    my @liblinks;
    my @doclib = $share{DB}->fetch_match("page",{type=>"library",
						 section_id=>$this->{section_id}});
    my $ml = new ExSite::ML;
    foreach my $lib (@doclib) {
	my $url = $this->link(lib=>$lib->{filename});
	push @liblinks, $ml->a($lib->{title},{href=>$url});
    }
    return join(" - ",@liblinks);
}


# Hide file info(file size and type)?
sub hide_file_name{
	my ($this) = @_;
	# Get settings from the query string
	if (exists($this->{write_options}{hide_file_name})){
		return $this->{write_options}{hide_file_name} || 0;
	}
	# Get settings from the config file
	if (exists($config{Document}{hide_file_name})){
		return $config{Document}{hide_file_name} || 0;
	}
	return 0;
}

# Hide file info(file size and type)?
sub hide_file_info{
	my ($this) = @_;
	# Get settings from the query string
	if (exists($this->{write_options}{hide_file_info})){
		return $this->{write_options}{hide_file_info} || 0;
	}
	# Get settings from the config file
	if (exists($config{Document}{hide_file_info})){
		return $config{Document}{hide_file_info} || 0;
	}
	return 0;
}

# Hide file post date?
sub hide_file_postdate{
	my ($this) = @_;
	# Get settings from the query string
	if (exists($this->{write_options}{hide_file_postdate})){
		return $this->{write_options}{hide_file_postdate} || 0;
	}
	# Get settings from the config file
	if (exists($config{Document}{hide_file_postdate})){
		return $config{Document}{hide_file_postdate} || 0;
	}
	return 0;
}

# list documents by date

sub list {
    my $this = shift;
    my $db = $share{DB};
    my $ml = new ExSite::ML;
    my $in = $this->{input};
    my $lib = $this->getlib;
    return $this->error($msg{"No document library found."}) if (!$lib);

    my $out;
    my @content_records;
    if ($this->{asc}==1) {
	@content_records = $db->fetch_child("content","page",
					    $lib->{page_id}, $this->{sort});		
    }
    else {
	@content_records = reverse $db->fetch_child("content","page",
						    $lib->{page_id}, $this->{sort});		
    }

    my @content;
    my $nrec = scalar @content_records;
    my $start = (defined $in->{st} && $in->{st} < $nrec) ? $in->{st} : 0;
    my $step = $in->{step} || $config{Document}{ndoc} || 10;
    my $end = $start + ($step - 1);
    if ($end > $nrec) { $end = $nrec - 1; }

    my @dlist;
    foreach my $crec (@content_records[$start..$end]) {
	my $listitem;
	my $c = new ExSite::Content(content=>$crec,version=>"newest");
	$c->set_context();
	my $url = $c->get_url;
	my ($file,$size,$mime) = $c->get_fileinfo;
	my $time = $db->show_data_noauth("content_data","ctime",$c->timestamp);
	my $descr = $db->show_data_noauth("content","description",$c->get("content","description"));
	if ($descr) { $descr .= "<br>\n"; }
	my $info;
	my $newwin = $this->{newwin};
	if ($file) {
	    # Save file info to an array
	    my @fileinfo = ($file);
	    # Show file info
	    if (!$this->hide_file_info()){
	    	push(@fileinfo,$ml->span("($mime, $size K)",{class=>"DocumentFileInfo"}));
	    }
	    # Show file post date
	    if (!$this->hide_file_postdate()){
	    	push(@fileinfo,$ml->span($msg{"posted on"}." $time",{class=>"DocumentDate"}));
	    }
	    if (@fileinfo == 0) {
		push @fileinfo, $msg{"(no file available for download)"};
	    }
	    if (!$this->hide_file_name()){
		$info = $ml->em(join(" ",@fileinfo));
	    }
	}

	my ($dt,$dd);
	if ($c->has_content()) {
	    my %a = ( 
		      href => $url,
		      class => &MimeToFile($mime),
		      );

#	    $out .= "<dt><a href=\"$url\"";
	    # check if we need to make this link open in a new window
	    if ($mime =~ /$this->{newwin}/) {
		$a{target} = "_blank";
	    }
	    $dt = $ml->a(
			   $db->show_data_noauth("content","name",$c->get("content","name")),
			   \%a
			 );

		"</a></dt>\n<dd>$descr$info\n";
	}
	else {
	    $dt = $db->show_data_noauth("content","name",$c->get("content","name"));
	    
		"</dt>\n<dd>$descr ".$msg{"(no file available for download)"}."\n";
	}
	$dd = $descr.$info;

	if ($this->{admin} && $db->owns($this->{section_id})) {
	    my $update_url = $this->link(doccmd=>"update",docid=>$crec->{content_id});
	    $dd .= $ml->a(
			  $ml->img(undef,{src=>"$config{server}{HTMLpath}/_Modules/Document/update.gif", height=>12, width=>12, border=>0, alt=>"update", title=>"update"}),
			  { href=>$update_url }
			  );
	    my $edit_url = $this->link(doccmd=>"edit",docid=>$crec->{content_id});
	    $dd .= $ml->a(
			  $ml->img(undef,{src=>"$config{server}{HTMLpath}/_Modules/Document/edit.gif", height=>12, width=>12, border=>0, alt=>"edut", title=>"edit"}),
			  { href=>$edit_url }
			  );
	    my $del_url = $this->link(doccmd=>"del",docid=>$crec->{content_id});
	    $dd .= $ml->a(
			  $ml->img(undef,{src=>"$config{server}{HTMLpath}/_Modules/Document/del.gif", height=>12, width=>12, border=>0, alt=>"delete", title=>"delete"}),
			  { href=>$del_url }
			  );
	}
	push @dlist, { dt=>$dt, dd=>$dd };
    }
    $out .= $ml->dl(\@dlist);

    if ($end - $start < 0) {
	$out .= $ml->p($msg{"No documents found."});
    }
    else {
	if ($end < $nrec - 1) {
	    my $url = $this->link(st=>$end+1);
	    $out .= $ml->p($ml->a($msg{"More documents"},{href=>$url}));
	}
	if ($end < $nrec - 1 || $start > 0) {
	    my $url = $this->link(st=>0,step=>$nrec);
	    $out .= $ml->p($ml->a($msg{"All $nrec documents"},{href=>$url}));
	}
	if ($start > 0) {
	    $out .= $ml->p($ml->a("&gt;&gt; $msg{Back}",{href=>"javascript:history.back()"}));
	}
    }

    # admin links

    if ($this->{admin} && $db->owns($this->{section_id})) {
	# current user owns this site, and is viewing the page dynamically;
	# give them some admin links
	my $url = $this->link(doccmd=>"add");
	my $puburl = "$config{server}{CGIpath}/publish.cgi?page=$this->{lib}{page_id}";
	$out .= $ml->a(
		       $ml->img(undef,
				{src=>"$config{server}{HTMLpath}/_Modules/Document/new.gif", height=>12, width=>12, border=>0, alt=>"new doc", title=>"new doc"}).
		       " ".$msg{"add new document"},
		       {href=>$url});
	$out .= $ml->br();
	$out .= $ml->a(
		       $ml->img(undef,
				{src=>"$config{server}{HTMLpath}/_Modules/Document/publish.gif", height=>12, width=>12, border=>0, alt=>"publish", title=>"publish"}).
		       " ".$msg{"publish all documents"},
		       {href=>$puburl});
    }
    
    return $ml->div($out,{class=>"Document"});
}


sub docform {
    my ($this,%opt) = @_;
    my $url = $opt{doccmd} ? $this->link(doccmd=>$opt{doccmd}) : $this->link();
    my $ml = new ExSite::ML;
    my $fb = new ExSite::FormBuilder(method=>"post",action=>$url);
    my $input;
    if ($opt{upload}) {
	$input = {type=>"file",name=>"upload",prompt=>"Choose document",required=>1};
	if ($opt{upload} != 1) { $input->{value} = $opt{upload}; }
	$fb->input(%$input);
    }
    if ($opt{title}) {
	$input = {type=>"text",name=>"title",prompt=>"Title"};
	if ($opt{title} != 1) { $input->{value} = $opt{title}; }
	$fb->input(%$input);
    }
    if ($opt{descr}) {
	$input = {type=>"textarea",name=>"descr",prompt=>"Description"};
	if ($opt{descr} != 1) { $input->{value} = $opt{descr}; }
	$fb->input(%$input);
    }
#    if ($opt{docid}) {
#	$input = {type=>"hidden",name=>"docid",value=>$opt{docid}};
#	$fb->input(%$input);
#    }
    $fb->buttons( submit=> ($opt{submit} || 1) );
    return $fb->make;
}

# add a new document

sub add {
    my $this = shift;
    my $url = $this->link(doccmd=>"upload");
    my $ml = new ExSite::ML;
    my $out = $ml->h1($msg{"Upload new document"});
    $out .= $this->docform(doccmd=>"upload",
			   upload=>1,title=>1,descr=>1,
			   submit=>"Upload");
    return $out;
}

# add a new document

sub edit {
    my $this = shift;
    my $in = $this->{input};
    my $lib = $this->getlib;
    my $ml = new ExSite::ML;
    my $cid;
    if ($in->{title}) {
	if ($in->{docid}) { 
	    my %c = $share{DB}->fetch("content",$in->{docid});
	    if ($c{page_id} == $lib->{page_id}) {
		$cid = $in->{docid};
	    }
	    else {
		# somebody's trying to fool us
		return $this->error($msg{"Invalid document ID."});
	    }
	}
	else {
	    # should never happen
	    return $this->error($msg{"No document to update!"});
	}
	my $stat = $share{DB}->update("content",{content_id=>$cid,
						 name=>$in->{title},
						 description=>$in->{descr}});
	my $return_url = $this->link(doccmd=>undef,docid=>undef);
	return 
	    $ml->p($stat || $msg{"Document details updated."},{class=>"help"}).
	    $ml->p($ml->a($msg{"Back to document library."},{href=>$return_url}));
    }
    else {
	my $out;
	if ($this->{input}{docid}) {
	    my $doc = $share{DB}->fetch("content",$this->{input}{docid});
	    my $url = $this->link(doccmd=>"edit");
	    $out = $ml->h1("Edit document details");
	    $out .= $this->docform(doccmd=>"edit",
				   title=>$doc->{name},
				   descr=>$doc->{description},
				   submit=>1);
	}
	else {
	    $out = $ml->p($msg{"No such document."},{class=>"error"});
	}
	return $out;
    }
}

# update an existing document

sub update {
    my ($this,$id) = @_;
    my $doc;
    my $db = $share{DB};
    my $ml = new ExSite::ML;
    if ($id) {
	$doc = $db->fetch("content",$id);
	if ($doc) {
	    my $url = $this->link(doccmd=>"upload");
	    my $name = $db->show_data("content","name",$doc->{name});
	    my $out = $ml->h1($msg{"Upload new version of"}." \"$name\"");
	    $out .= $ml->p($msg{"Previous versions are automatically archived,
and can be recovered by deleting the newer versions."},
			   { class=>"help" });
	    $out .= $this->docform(doccmd=>"upload",
				   upload=>1,
				   docid=>$id,
				   submit=>"Upload");
	    return $out;
	}
	else {
	    return $this->error($msg{"Invalid document ID"});
	}
    }
    else {
	return $this->error($msg{"No document to update"});
    }
}

# update an existing document

sub upload {
    my $this = shift;
    my $in = $this->{input};
    my $cid;
    my $lib = $this->getlib;
    if (! $lib) {
	return $this->error($msg{"No document library found."});
    }
    
    if ($in->{upload}) {
	if (exists $in->{title}) {
	    # make content object
	    if (! $in->{title}) {
		my ($file,undef) = split/$config{form}{sepchar}/,$in->{upload};
		my @file = split /[:\\\/]/,$file;
		$in->{title} = $file[-1];
	    }
	    $cid = $share{DB}->insert("content",{page_id=>$lib->{page_id},
						 name=>$in->{title},
						 description=>$in->{descr},
						 type=>"editorial",
					     }
				      );
	}
	else {
	    if ($in->{docid}) { 
		my %c = $share{DB}->fetch("content",$in->{docid});
		if ($c{page_id} == $lib->{page_id}) {
		    $cid = $in->{docid};
		}
		else {
		    # somebody's trying to fool us
		    return $this->error($msg{"Invalid document ID."});
		}
	    }
	    else {
		# should never happen
		return $this->error($msg{"No document to update!"});
	    }
	}
	
	if ($cid) {
	    my $comment = "uploaded on ".(scalar localtime);
	    my $file = $in->{upload};
	    my $fdata = undef;
	    my $size = 0;
	    my $mimedata = "";
	    if ($ENV{CONTENT_TYPE} =~ /multipart\/form-data/) {
		# read file contents
		my $continue = 1;
		while ($continue) {
		    $continue = read($in->{upload},$fdata,1024,$size);
		    $size += $continue;
		}
		# MIME-encode the file contents
		$mimedata = encode_base64($fdata,"");
	    }
	    # strip path from filename
	    my @path = split /[:\\\/]/, $file;
	    my $filename;
	    if ($path[-1]) { 
		$filename = $path[-1];
		$filename =~ s/ /_/g;                # conv space to _
		$filename = &url_escape($filename);  # escape other chars
	    }
	    if ($share{DB}->insert("content_data",{content_id=>$cid,
						   mime_type=>&MimeType($filename),
						   note=>$comment,
						   fdata=>"$filename$config{form}{sepchar}$mimedata",
					       })) {
		my $home = $this->link(doccmd=>undef,docid=>undef);
		my $ml = new ExSite::ML;
		return 
		    $ml->h1("$filename ".$msg{"uploaded successfully"}).
		    $ml->p(
			   $ml->a($msg{"Back to document library"},{href=>$home})
			   );
	    }
	    else {
		return $this->error($msg{"Error saving uploaded file.  It could be larger than the maximum allowed size."});
	    }
	}
	else {
	    return $this->error($msg{"Error configuring uploaded file."});
	}
    }
    else {
	return $this->error($msg{"Nothing to upload!"});
    }
}

# make a new library

sub newlib {
    my $this = shift;
    my $in = $this->{input};
    my $ml = new ExSite::ML;
    my $out;
    
    if ($in->{name}) {
	# create library
	my $libname = &clean_filename($in->{name});
	my %page = (
		    section_id=>$this->get_section_id,
		    type=>"library",
		    title=>$in->{name},
		    filename=>$libname,
		    access=>$in->{access},
		    publish_method=>"static",
			status=>"active",
		    );
	my $lib = new ExSite::Page();
	if ($lib->make(\%page)) {
	    $out .= $ml->h1($msg{"New Library Created"});
	    $out .= $ml->p(
			   $ml->a($msg{"Go to new library"},
				  {href=>$this->link(doccmd=>undef,lib=>$libname),
				   class=>"button"})
			   );
	}
	else {
	    $out .= $ml->h1($msg{"Error Creating Library"});
	    $out .= $lib->show_diagnostics("error","html");
	    $out .= $ml->p(
			   $ml->a("&lt; $msg{Back}",
				  {href=>"javascript:history.back()",
				   class=>"button"})
			   );
	}
    }
    else {
	# show create library form
	my $f = new ExSite::FormBuilder(action=>$this->link());
	$f->input(prompt=>$msg{"Document Library Name"},name=>"name",
		  type=>"text",maxlength=>40,required=>1);
	$f->input(prompt=>$msg{"Access"},name=>"access",type=>"select",
		  options=>["public","members"],required=>1);
	$out = $ml->h1($msg{"Make New Document Library"});
	$out .= $f->make;
    }
    return $out;
}

# update an existing document

sub delete {
    my $this = shift;
    my $db = $share{DB};
    my $ml = new ExSite::ML;
    my $in = $this->{input};
    my $cid;
    my $lib = $this->getlib;
    if (! $lib) {
	return $this->error($msg{"No document library found."});
    }

    if ($in->{confirm}) {
	if ($in->{docid}) {
	    # validate
	    my $doc = $db->fetch("content",$in->{docid});
	    if ($doc->{page_id} eq $lib->{page_id}) {
		# okay to proceed
		# use recursive delete to remove all versions
		
		### FIXME: published version remains on disk
		
		$db->delete_r("content",$in->{docid},1);
		my $home = $this->link(doccmd=>undef,docid=>undef,confirm=>undef);
		return
		    $ml->h1("\"$doc->{name}\" deleted").
		    $ml->p(
			   $ml->a($msg{"Back to documents control panel"},{href=>$home})
			   );
	    }
	    else {
		return $this->error($msg{"Invalid document ID."});
	    }
	}
	else {
	    return $this->error($msg{"No document was specified."});
	}
    }
    else {
	my $url = $this->link(confirm=>1);
	my $doc = $db->fetch("content",$in->{docid});
	return
	    $ml->h1($msg{"Confirm deletion of"}." \"$doc->{name}\"").
	    $ml->ul(
		    $ml->li($ml->a($msg{"Yes, delete this document and all its versions"},{href=>$url})).
		    $ml->li($ml->a($msg{"NO, I changed my mind."},{href=>"javascript:history.back()"}))
		    );
    }
}

1;

