package Modules::ImageRotate;

#----------------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#-----------------------------------------------------------------------
# ImageRotate : tool to animate/rotate through the images in a library
#
# Options:
#
# id=N			library id of photo album, OR
# name=ABC		name of photo album
# interval=x	interval at which Images rotate
# format=onload|fade	onload:only rotate images on page load, not on an interval
#						fade: fade images in
# sort=name		sort images by sort column, random if not d
#
#-----------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Input;
use ExSite::Misc;
use ExSite::Content;
use Modules::BaseDCD;

# inherit basic behaviours from BaseDCD
# some scripts/code not needed if not fading

use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD);

# provide our own write() method

sub write {
	my ($this,$options) = @_;
	my $in = new ExSite::Input;
    my $q = $in->query;
    my %opt = &DecodeString($options);
    my %allopt = (%opt,%$q);
	
	$allopt{interval} = $allopt{interval} || 5000;
	$allopt{speed} = $allopt{speed} || 2500;
	$allopt{random} = 1 if (!$allopt{sort});
	$allopt{sort} = $allopt{sort} || "name";

	my $db = $share{DB};
	my ($album,$album_id);
	if ($allopt{id}) {
    	# album passed by id
		$album_id = $allopt{id};
		$album = $db->fetch("page",$album_id);
	} elsif ($allopt{name}) {
        # album passed by name
        my @lib = $db->fetch_match("page",{section_id=>$share{Page}->id("section"),
                                           filename=>$allopt{name}});
        $album = $lib[0];
        $album_id = $album->{page_id};
    }
    if (!$album) {
        # look for "images" library in current site
        my @lib = $db->fetch_match("page",{section_id=>$share{Page}->id("section"),
                                           filename=>"images"});
        $album = $lib[0];
        $album_id = $album->{page_id};
    }

    if ($album) { 
		my @pics = $db->fetch_child("content", "page", $album_id, $allopt{sort});
		# total number of images
		my $count = scalar @pics;
		my (@images);
		my ($width,$height) = ($allopt{width},$allopt{height});

		foreach my $p (@pics) {
			my $c = new ExSite::Content(content=>$p);
			$c->set_context("child");
			my $link = $c->get("content","description");
			$link = &html_to_text($link);			
			my $url = $c->get_url;
			my $image = {url=>$url,link=>$link};
			# each $image is a hash ref containing information about an image
			push(@images,$image);
		}
		
		# 3 formats for ImageRotate
		my ($is_onload,$is_fade,$is_linked);
		if ( $allopt{format} =~ /onload/ ) { $is_onload = 1; }		
		if ( $allopt{format} =~ /fade/ ) { $is_fade = 1; }		
		if ( $allopt{format} =~ /linked/ ) { $is_linked = 1; }
		
		if (!$height) { 
			my $c = new ExSite::Content(content=>$pics[0]);
			$c->set_context("child");
			my $content_data = $db->fetch("content_data", $c->get("content_data","content_data_id"));
			my $image = new ExSite::Image($content_data->{fdata});
		    ($width,$height) = $image->dim;
		}		
		
		my $out;
		if ($is_fade) { 
			$out = "<div class=\"ImageRotateHolder\" id=\"s1\">";
		} else {
			$out = "<div class=\"ImageRotateHolder\">";
		}
		&fisher_yates_shuffle(\@images) if ($allopt{random});
		for (my $i = 0; $i < $count; $i++) {
			my $style = "style=\"display: none\"" if ($i > 0);
			$out .= "<a href=\"$images[$i]->{link}\">" if $is_linked;
			$out .= "<img src=\"$images[$i]->{url}\" $style border=\"0\">";
			$out .= "</a>" if $is_linked;
			# only load first image if format is_onload
			last if ($is_onload);
		}
		$out .= "</div>\n";
		$out .= "<script type=\"text/javascript\" src=\"$config{server}{HTMLpath}/_Modules/ImageRotate/jquery.js\"></script>"
			if (!$config{ImageRotate}{use_template_jquery});
		$out .= <<"DONE";
<script type="text/javascript" src="$config{server}{HTMLpath}/_Modules/ImageRotate/jquery_cycle.js"></script>
<script type="text/javascript" src="$config{server}{HTMLpath}/_Modules/ImageRotate/jquery_easing.js"></script>
DONE
		$out .= <<"DONE";
<style>
#s1 {
	width: ${width}px; 
	height: ${height}px;
	padding: 0;  
	margin: 0;  
} 
/* menu should appear over rotate */
div.DMenu {
	z-index: 100;
}
 
.ImageRotateHolder img {
	top:  0; 
	left: 0;
}
</style>		
		
<script type="text/javascript">
function get_random()
{
	var today = new Date()
	var seed = today.getSeconds()
    var ranNum= Math.floor(Math.random(seed)*$count)
    return ranNum;
}
</script>

<script type="text/javascript">
\$(document).ready(function() {
\$(function() {
	\$('#s1 img').show();
	\$('#s1').cycle({
    fx:      'fade',
	speed: $allopt{speed},
    timeout:  $allopt{interval},
    easing:  'backinout'
	});
});
});
</script>
DONE
	}
    else {
		my $message = "photo album not found ($options)";
		$this->error($message);
		return "<p class=\"error\">$message</p>\n";
	}	
}

sub ioctl {
    my $this = shift;
    $_ = shift;
    if (/ModuleName/) {
        return "ImageRotate";
    }
    elsif (/ModuleInfo/) {
        return "The ImageRotate tool allows you to 
rotate through your image library.  It also allows you to 
fade images in and load random images on refresh.";
    }
}

sub fisher_yates_shuffle {
    my $array = shift;
    my $i;
    for ($i = @$array; --$i; ) {
        my $j = int rand ($i+1);
        next if $i == $j;
        @$array[$i,$j] = @$array[$j,$i];
    }
}

sub html_to_text {
    $_ = shift;
    s/\r//g;  # remove linebreaks
    s/\n//g;  # remove linebreaks	
    return $_;
}

1;
