package Modules::PluginBuilder;
#----------------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------
#
#   PluginBuilder is a tool that creates plug-in module templates.
#   Developers can fill the templates with their own code.
#
#----------------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::Util;
use ExSite::ML;
use ExSite::FormBuilder;
use ExSite::Input;
use Modules::BaseDCD;
use Modules::CMS;

use vars qw(@ISA $ml);
@ISA = qw(Modules::BaseDCD);

# write : do nothing;

sub write {
    my ($this,$options) = @_;
    return "";
}

sub ioctl {
    my $this = shift;
    $_ = shift;   # $_ is the ioctl request
    if (/ModuleName/) {
	return "Plug-in Builder";
    }
    elsif (/ControlPanel/) {
	return \&ctrl_panel;
    }
    elsif (/isRestricted/) {
	return $config{PluginBuilder}{isRestricted};
    }
    elsif (/ModuleInfo/) {
	return "PluginBuilder is a tool for creating programmers templates for plug-in modules."
    }
    elsif (/Category/) {
	return "System";
    }
    return undef;
}

sub ctrl_panel {
    my $this = shift;
    if (! $share{DB}->is_admin()) {
	return $this->error("Permission denied - PluginBuilder is restricted to system administrators only.");
    }
    $ml = new ExSite::ML;
    my $in = new ExSite::Input;
    my $data = $in->post();
    if (keys %$data > 0) {
	if ($data->{code}) {
	    return $this->publish_module($data);
	}
	else {
	    return $this->build_module($data);
	}
    }
    else {
	return $this->module_questionnaire();
    }
}

sub module_questionnaire {
    my $this = shift;
    my $out = $ml->h1("Plugin Builder");
    $out .= $ml->p("Please answer the following questions, and ExSite will 
automatically configure a code template for the module you have specified.
If you are unsure what answers to give, useful defaults have been 
preselected.  A more detailed explanation of each question is given
in the help.");
    my $f = new ExSite::FormBuilder();
    $f->input(type=>"text",
	      name=>"name",
	      prompt=>"Module Name",
	      required=>1,
	      value=>$config{PluginBuilder}{default}{name},
	      tail=>" (short machine name of the package)");
    $f->input(type=>"text",
	      name=>"title",
	      prompt=>"Module Title",
	      required=>0,
	      size=>60,
	      value=>$config{PluginBuilder}{default}{title},
	      tail=>" (human-friendly name of the package; defaults to the Module Name)");
    $f->input(type=>"text",
	      name=>"info",
	      prompt=>"Module Info",
	      required=>0,
	      size=>60,
	      value=>$config{PluginBuilder}{default}{info},
	      tail=>" (short description of the module and what it does)");
    $f->input(type=>"text",
	      name=>"author",
	      prompt=>"Author",
	      required=>0,
	      value=>$config{PluginBuilder}{default}{author},
	      size=>60,
	      tail=>" (copyright notice will be added for this author)");
    $f->input(type=>"preformatted",
	      name=>"features",
#	      required=>0,
	      prompt=>"<h3>Features</h3>",
	      input=>"<p>Please check off which features you would like to support in this plug-in.</p>");
    $f->input(type=>"checkbox",
	      name=>"read",
	      required=>0,
	      prompt=>" ",
	      tail=>" This plug-in has specialized input management");
    $f->input(type=>"checkbox",
	      name=>"write",
	      required=>0,
	      checked=>1,
	      prompt=>" ",
	      tail=>" This plug-in generates content for normal web pages");
    $f->input(type=>"checkbox",
	      name=>"ctrlpanel",
	      required=>0,
	      checked=>1,
	      prompt=>" ",
	      tail=>" This plug-in needs an administrator control panel");
    $f->input(type=>"checkbox",
	      name=>"isrestricted",
	      required=>0,
	      prompt=>" ",
	      tail=>"This plug-in will be restricted to specific sections only");
    $f->input(type=>"checkbox",
	      name=>"isservice",
	      required=>0,
	      checked=>1,
	      prompt=>" ",
	      tail=>"This plug-in may redirect itself to a particular page when interacted with");
    $f->input(type=>"checkbox",
	      name=>"publish",
	      required=>0,
	      prompt=>" ",
	      tail=>"This plug-in will publish files to disk");
    $f->input(type=>"checkbox",
	      name=>"search",
	      prompt=>" ",
	      required=>0,
	      tail=>"This plug-in will insert its own data into the search index");
    $f->input(type=>"checkbox",
	      name=>"cron",
	      required=>0,
	      prompt=>" ",
	      tail=>"This plug-in can perform scheduled tasks");
    $f->input(type=>"checkbox",
	      name=>"todo",
	      required=>0,
	      prompt=>" ",
	      tail=>"This plug-in can add items to the administrator's to-do list");
    $f->input(type=>"checkbox",
	      name=>"std_license",
	      required=>0,
	      checked=>1,
	      prompt=>" ",
	      tail=>"Use the standard ExSite licensing");

    $out .= &ExSite::HTML::BasicBox(title=>"New Plugin Details",
				    pane=>$f->make(),
				    );
    return $out;
}

sub build_module {
    my ($this,$data) = @_;

    my $out = $this->validate($data);
    return $out if ($out);

    my $code = $this->build_code($data);
    my $conf = $this->build_conf($data);
    my $help = $this->build_help($data);

    if ($this->errorcheck("warn")) {
	$out .= $ml->div(
			 $ml->h2("Warnings:").
			 $this->show_diagnostics("warn","html"),
			 {style=>"border:1px #c00 solid; background-color:#fcc; padding:10px;"}
			 );
    }

    $out .= $ml->h1("Your Stubs");
    $out .= $ml->p("The following stub files can be copied or saved to your 
local computer, and worked on further.");

    $out .= $ml->h2("Plug-in Module Code");
    $out .= $ml->p("This should be saved to the file ".
		   $ml->code("cgi-bin/Modules/$data->{name}.pm").
		   ".");
    $out .= $ml->pre($code,{style=>"height:300px; overflow:auto; border:1px #666 solid; background-color:white; padding:10px;"});

    $out .= $ml->h2("Help File");
    $out .= $ml->p("This should be saved to the file ".
		   $ml->code("html/_Modules/$data->{name}/help.html").
		   ".");
    $out .= $ml->pre($help,{style=>"height:150px; overflow:auto; border:1px #666 solid; background-color:white; padding:10px;"});

    $out .= $ml->h2("Configuration File");
    $out .= $ml->p("This should be saved to the file ".
		   $ml->code("cgi-bin/conf/$data->{name}.conf").
		   ".");
    $out .= $ml->pre($conf,{style=>"height:150px; overflow:auto; border:1px #666 solid; background-color:white; padding:10px;"});
	
    $out .= &ExSite::HTML::Button(url=>"javascript:history.back()",
				  label=>"Revise this plugin");
	
    return $out;
}

sub build_code {
    my ($this,$data) = @_;
    my $out = "package Modules::$data->{name};\n\n";

    if ($data->{std_license}) {
	my $t = new ExSite::Time;
	my $year = $t->write("year");
	$out .= "#----------------------------------------------------------------------------
#
#   Copyright (C) $year - $data->{author}
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------\n";
    }
    if ($data->{info}) {
	$out .= "#\n#  $data->{info}\n#\n#----------------------------------------------------------------------------\n";
    }

    $out .= '
use strict;

# useful kernel libs; you can/should remove any of these that are not needed

use ExSite::Config;          # required
use ExSite::Input;           # optional input manager
use ExSite::Misc;            # optional utils
use ExSite::Util;            # optional utils
use ExSite::ML;              # optional markup-language generation
use ExSite::FormBuilder;     # optional form generation
use ExSite::ReportBuilder;   # optional report generation
use ExSite::Form;            # optional database manager

# recommended base class for plug-in modules

use Modules::BaseDCD;

# declare package globals

use vars qw(@ISA $ml);       # Perl inheritance global & markup language object

# define our class inheritance

@ISA = qw(Modules::BaseDCD); # inherit from this base class

';

    if ($data->{read}) {
	$out .= '# read method (sets up input data)

sub read {
    my ($this,$options) = @_;

    # default behaviour is to fetch overlayed GET and POST data
    # and store them in the object for use by other methods

    my $in = new ExSite::Input;
    $this->{input} = $in->combine;
}';
    }

    if ($data->{write}) {
	$out .= '

# write method (builds output for regular web pages)

sub write {
    my ($this,$options) = @_;

    # declare a markup generator
    $ml = new ExSite::ML;

    # build a block of HTML to return to the CMS for inlining into
    # a template or other content.

    my $out;  # our output buffer

    # based on information in $options or $this->{input}, add HTML to $out.

    $out .= $ml->h1("Hello world");

    return $out;
}';
    }

    my $isRestricted = $data->{isrestricted} ? 1 : 0;
    my $isService = $data->{isservice} ? 1 : 0;
    $out .= "

# ioctl method (used by ExSite to query the module for its functionality)

sub ioctl {
    my \$this = shift;
    \$_ = shift;           # \$_ is the ioctl request

    if (/isRestricted/) {
	return $isRestricted;
    }
    elsif (/isService/) {
	return $isService;
    }
";
    if ($data->{title}) {
        $out .= "    elsif (/ModuleName/) {
	return \"$data->{title}\";
    }
";
    }
    if ($data->{info}) {
        $out .= "    elsif (/ModuleInfo/) {
	return \"$data->{info}\";
    }
";
    }
    if ($data->{ctrlpanel}) {
        $out .= "    elsif (/ControlPanel/) {
	return \\\&ctrl_panel;
    }
";
    }
    if ($data->{publish}) {
        $out .= "    elsif (/Publish/) {
	return \\\&publish;
    }
    elsif (/Unpublish/) {
	return \\\&unpublish;
    }
";
    }
    if ($data->{search}) {
        $out .= "    elsif (/Search/) {
	return \\\&search_index;
    }
";
    }
    if ($data->{cron}) {
        $out .= "    elsif (/Cron/) {
	return \\\&cron;
    }
";
    }
    if ($data->{todo}) {
        $out .= "    elsif (/ToDo/) {
	return \\\&todo;
    }
";
    }
    if ($data->{dynparam}) {
        $out .= '    elsif (/DynContentParam/) {
	# EXAMPLE: returns a select input with two options
	return 
	    $ml->p("Parameter:").
	    $ml->select(
			[
#                          OPTION LABEL    OPTION VALUE
			 [ "1st Option", { value=>"option1" } ],
			 [ "2nd Option", { value=>"option2" } ],
			],
		        { name=>"param" }
			);
';
    }
    $out .= "}\n";

    $out .= "\n#----------------------------------------------------------------------------
# Everything after this point consists of private methods.
";

    if ($data->{ctrlpanel}) {
        $out .= '
# ctrl_panel() generates the contents of the administrator control panel

sub ctrl_panel {
    my $this = shift;

    # declare a markup generator
    $ml = new ExSite::ML;

    # build a block of HTML to display the control panel

    my $out;  # our output buffer
    $out .= $ml->h1("This is your control panel");
    return $out;
}
';
    }
    if ($data->{publish}) {
        $out .= '
# publish() can be used to write out any files that will benefit from 
# being available statically

sub publish {
    my $this = shift;

    # this method will be called via publish.cgi, which has permission to
    # write files into the webserver directories.

    # It can optionally return some status messages.
    my $out;

    # your publishing code goes here.

    return $out;
}

sub unpublish {
    my $this = shift;

    # This does the opposite of publish() and removes files that this 
    # module has previously published.

    # It can optionally return some status messages.
    my $out;

    # your un-publishing code goes here.

    return $out;
}
';
    }
    if ($data->{search}) {
        if ($data->{isservice}) {
            $out .= '
# search_index() allows this plug-in to manipulate the system search index.

sub search_index {
    my ($this,$search,$site,$page) = @_;

    # $search is an ExSite::Search object.  Use this to modify the
    # search index.

    # $site is an ExSite::Section object for the current section.  It is
    # normally used to identify information that should be indexed in
    # this call.

    # $page is an ExSite::Page object for the service page that 
    # all search hits will be directed to.

    # You can optionally return some status messages
    my $out;

    # manipulate the search index here.

    return $out;
}
';
        }
        else {
            $this->warn("To manipulate search indexes, you must have a service page to redirect to.");
        }
    }
    if ($data->{cron}) {
        $out .= '
# cron() allows this plug-in to respond to scheduling events.

sub cron {
    my ($this,$command,$type,$id) = @_;

    # $command, $type, $id are message parameters passed to us by the
    # scheduler.  They have no predefined meaning, so you can do as you
    # please with them.

    # You can optionally return some status messages
    my $out;

    # perform your scheduled tasks here.

    return $out;
}
';
    }
    if ($data->{todo}) {
        $out .= '
# todo() allows this plug-in to advise the administrator of work that needs
#        attention.

sub todo {
    my $this = shift;

    # the todo list is analogous to an RSS feed of tasks
    my @todo;

    # determine your list of tasks, and add them to the list, eg.
    #push @todo, { link=> $url, title=> $title, description=> $description },

    return @todo;
}
';
    }

    $out .= "\n1;\n";

    return $out;
}

sub build_conf {
    my ($this,$data) = @_;
    my $out = "# $data->{name} configuration file

# settings in this file like X.Y = Z will be available to the code
# as \$config\{$data->{name}\}{X}{Y} == \"Z\"

example = foo

another.example = bar

";
    return $out;
}

sub build_help {
    my ($this,$data) = @_;
    my $title = $data->{title} || $data->{name};
    my $out = "&lt;h1&gt;$title&lt;/h1&gt;

&lt;p&gt;This is your help file for site administrators.  Use it to 
explain how to configure this plug-in, interact with your control panel, 
and style their sites/templates to display your output in the best way.&lt;/p&gt;\n";
    return $out;
}

sub validate {
    return;
}

1;
