#!/usr/bin/perl
#-----------------------------------------------------------------------
#
#   Copyright 2001-2009 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

package Modules::QA::Form;

use strict;
use ExSite::Config;
use ExSite::Base;
use ExSite::Form;
use ExSite::Captcha;
use ExSite::Misc;
use ExSite::Mail;
use ExSite::Object;
use ExSite::ObjectList;
use Modules::QA::Question;
use Modules::QA::Response;

use vars qw(@ISA);
@ISA = qw(ExSite::Object Modules::BaseDCD);

sub my_type { return "qa_form"; }

sub name {
    my $this = shift;
    $this->load() if (!defined $this->{data});
    return $this->getdata("title");
}

sub flag {
    my ($this,$flag) = @_;
    my $flags = $this->getdata("flags");
    if ($flag) {
	return $flags =~ /$flag/;
    }
    else {
	my @flag = split /; /, $flags;
	return wantarray ? @flag : \@flag;
    }
}

#=== DISPLAY

sub show {
    my ($this,%opt) = @_;
    if ($this->ok) {
	if (! $this->{data}) { $this->load(); }
	my $name = ucfirst $this->name;
	if ($opt{version}) {
	    $this->set("version",$opt{version});
	}
	elsif ($share{Page} && $share{Page}->get_my("version")) {
	    $this->set("version",$share{Page}->get_my("version"));
	}
	if (! $this->{data}) {
	    return $this->error("$name ".$msg{"does not exist."});
	}
	elsif (! $this->{list}) {
	    return $this->error("$name ".$msg{"has no questions."});
	}
	else {
	    my $in = new ExSite::Input;
	    my $indata = $in->post_or_query();
	    if ($indata->{_qakey}) {
		# looks like we have form inputs
		return $this->do_form($indata);
	    }
	    else {
		# display ordered list of questions
		return $this->show_form(%opt);
	    }
	}
    }
    return $this->error($this->{status});
}

sub show_form {
    my ($this,%opt) = @_;
    if (! $this->{data}) { $this->load(); }
    if ($this->flag("members-only") && ! $share{DB}->level) {
	return $this->warn("You must be logged in to view this form.");
    }
    my $l = $this->{list};
    my $v = $this->get("version");
    my $out;
    my $num = 0;
    my %fkey = ( form=>$this->id );
    my $fb = new ExSite::FormBuilder(%opt);
#    $fb->set("highlight_rows",1);
    my $template = $opt{template} || $this->getdata("template");
    if ($template =~ /^\w+$/) {
	# single-word template = content object; search in context of current
	# page if possible, our section if not
	my $obj = $share{Page} ? 
	    $share{Page} : 
	    new ExSite::Section(id=>$this->getdata("section_id"));
	my $c = $obj->find($template);
	$template = $c->get_html() || undef;
    }
    if ($template) {
	$fb->template($template);
    }

    $opt{data} = $this->run_handler("QA_Prepop_data", $this->name);
 
    # any data to prepop the form with?
    my $response_data;
    if ($opt{response} =~ /^\d+$/) {
	my $r = new Modules::QA::Response(id=>$opt{response});
	$response_data = $r->get_response_data();
    }
    elsif (ref($opt{data}) eq "HASH" || $opt{data} eq "HASH") {
	$response_data = $opt{data};
    }
    my $in = new ExSite::Input();
    my $query_data = $in->query();
    
    my %names;
    while (my $q = $l->next()) {
	$num++;
	if ($v) { $q->set("version",$v); }
	my $qid = $q->qname;
	$names{$qid} = 1;
	my %qopt = (
		    FormBuilder=>$fb,
		    id=>$qid,
		    name=>$q->getdata("name"),
		    );
	if (ref $response_data eq "HASH" && defined $response_data->{$qid}) {
	    # use data from response
	    $qopt{value} = $response_data->{$qid};
	}
	elsif (exists $query_data->{$qid}) {
	    # query string has suggested value
	    $qopt{value} = $query_data->{$qid};
	}
	$q->show(%qopt);
    }
    my $c = new ExSite::Crypt();
    my $confirm = $this->flag("confirmation screen") ? 0 : 1;
    $fb->input(type=>"hidden",
	       name=>"_qakey",
	       value=>$c->encrypt($this->id.";".$confirm));

    # captcha - should captchas be conditional on login status?
    if ($this->flag("captcha")) {
	my $input = new ExSite::Input;
	my $in = $input->combine;
	my $c = new ExSite::Captcha(mode=>$in->{captchamode});
	$fb->input(name=>"captcha", prompt=>" ", input=>$c->make());
    }
    elsif ($this->flag("robot counter-measures")) {
	my $name = "comment";
	if ($names{$name}) {
	    # default honeypot field collides with our form
	    my $n = 0;
	    do { $name = "comment$n"; $n++ } until (! $names{$name});
	}
	my $c = new ExSite::Captcha(mode=>"passive",name=>$name);
	$fb->input(name=>"captcha", prompt=>" ", input=>$c->make($this->id));
    }

    return $fb->make();
}

sub do_form {
    my ($this,$data) = @_;
    my @err = $this->validate_form($data);
    my $ml = new ExSite::ML;
    my $out;
    if (@err > 0) {
	$out = $this->error(@err);
	$out .= $ml->p(&ExSite::HTML::Button(url=>"javascript:history.back()",
					     label=>"&lt; ".$msg{"Back to form"}));
    }
    elsif ($this->{confirmed}) {
	my $r = new Modules::QA::Response();
	my $uid = $share{DB}->my_uid || 0;
	my $rid = $r->make_response(
				    { 
					qa_form_id => $this->id ,
					member_id => $uid,
				    },
				    $this->{response_data}
				    );
	if ($rid) {
	    # response was logged successfully
	    if ($r->defined()) {
		# email notification
		$out .= $this->notify($r);
		# receipt callback for custom post-processing
		$out .= $this->run_handler("qa_receipt",$r);
	    }
	    if (! $r->errorcheck()) {
		# No errors were logged in post-processing...
		# we can show completion message
		my $message = 
		    $msg{$this->getdata("message")} || 
		    $ml->h1($msg{"Thank you"});
		return $message . $out;
	    }
	}
	else {
	    $out .= $r->show_diagnostics("error","html");
	}
    }
    else {
	return $this->confirmation_page($data);
    }
    return $out;
}

sub validate_form {
    my ($this,$data) = @_;
    my @err;
    if ($this->getdata("status") eq "inactive" && ! $share{DB}->is_admin()) {
	push @err, $msg{"Sorry, this form has been disabled and is not accepting input at this time."};
    }
    else {
	my $c = new ExSite::Crypt();
	my ($fkey,$confirm) = split /;/,$c->decrypt($data->{_qakey});
	$this->{confirmed} = $confirm;
	if ($fkey !~ /^\d+$/ || $fkey != $this->id) {
	    push @err, $msg{"Invalid form ID."};
	}
	else {
	    # looks like valid input

	    if (my $deny = $this->deny) {
		# failed to pass security check
		return ($deny);
		# don't bother with other validation
	    }

	    # create a question index
	    $this->{qobj} = {};  # index of question objects, by ID
	    $this->{qname} = {}; # index of question IDs, by name
	    while (my $q = $this->{list}->next()) {
		my $id = $q->id;
		$this->{qobj}{$id} = $q;
		my $name = $q->qname();
		if (exists $this->{qname}{$name}) {
		    push @err, $msg{"Duplicate questions under name"}." '$name'";
		    # dupes will overwrite each other in the response data
		}
		else {
		    $this->{qname}{$name} = $id;
		}
	    }

	    # get our list of questions and input names
	    $this->{response_data} = {};
	    my %legal_questions = &keywise($this->{list}->key,
					   scalar $this->{list}->getlist());

	    # map the response as qid => answer value
	    # spurious inputs (eg. captcha solutions) should be ignored
	    foreach my $key (%$data) {
		if (exists $this->{qname}{$key}) {
		    $this->{response_data}{$this->{qname}{$key}} = $data->{$key};
		}
	    }

	    # are all required answers present?
	    # do selected answers conform to allowed values?
	    foreach my $qid (keys %legal_questions) {
		my $q = $this->{qobj}{$qid};
		my $valerr = $q->validate_answer($this->{response_data}{$qid});
		if ($valerr) { push @err, $valerr; }
	    }
	}
    }
    return @err;
}

# deny : test if the form input should be rejected

sub deny {
    my $this = shift;
    if ($this->flag("captcha")) {
	# interactive captcha -> user must solve a puzzle
	my $input = new ExSite::Input;
	my $in = $input->combine;
	my $c = new ExSite::Captcha(mode=>$in->{captchamode});
	if (!$c->pass()) {
	    return $msg{"Incorrect answer to the anti-spam question."};
	}
    }
    if ($this->flag("robot counter-measures")) {
	# passive captcha
	my $c = new ExSite::Captcha(mode=>"passive");
	if (!$c->pass($this->id)) {
	    return $msg{"Failed to process form.  Please contact the webmaster."};
	}
    }
    if ($this->flag("members-only")) {
	if ($share{DB}->level < 1) {
	    return $msg{"Sorry, you must be logged in to use this form."};
	}
    }
    return undef;
}

sub confirmation_page {
    my ($this,$data) = @_;
    my $ml = new ExSite::ML();
    my $out = $ml->h1($msg{"Please Confirm"});
    my $report = new ExSite::ReportBuilder(
					   title=>$msg{"Your Response"},
					   headers=>[$msg{Question},$msg{Answer}],
					   );
    my $l = $this->{list};
    while (my $q = $l->next()) {
	next if ($q->format() =~ /^(hidden|label)$/);
        $q->set_version($this->{version}) if ($this->{version});
	# this is a dummy answer
	my $a = new Modules::QA::Answer(data=>{qa_answer_id=>-1, # kludge
					       qa_question_id=>$q->id,
					       value=>$data->{$q->qname()}});
	$report->push($q->label(),$a->show());
    }

    my $formdata;
    if ($config{QA}{top_confirm}) {
	$formdata .= 
	    $ml->p(
		   $ml->input(undef,{type=>"button",value=>"&lt; $msg{Back}",onclick=>"javascript:history.back()"}).
		   "&nbsp;".
		   $ml->input(undef,{type=>"submit",value=>$msg{Confirm}})
		   );
    }
    $formdata .= $report->make();
    foreach my $key (keys %$data) {
	if ($key eq "_qakey") {
	    # rewrite with a confirmation flag
	    my $c = new ExSite::Crypt();
	    my ($fkey,$confirm) = split /;/,$c->decrypt($data->{_qakey});
	    $data->{$key} = $c->encrypt("$fkey;1");
	}
	my $val = $data->{$key};
	$val =~ s/\"/&quot;/g;
	$formdata .= $ml->input(undef,{type=>"hidden",name=>$key,value=>$val});
	$formdata .= "\n";
    }
    $formdata .= $ml->p(
			$ml->input(undef,{type=>"button",value=>"&lt; $msg{Back}",onclick=>"javascript:history.back()"}).
			"&nbsp;".
			$ml->input(undef,{type=>"submit",value=>$msg{Confirm}})
			);

    $out .= $ml->p( $ml->form($formdata,{method=>"post"}) );
    return $out;
}

sub notify {
    my $this = shift;
    my $response = shift;
    my $stat = $this->run_handler("qa_notify",$response);
    return $stat if ($stat);

    my $notify = $this->getdata("notify");
    return if (! $notify);
    
    my $formtitle = $this->getdata("title");
    my $subject = "Response received from '$formtitle'";
    my $message = $subject . "\non $config{site}{name} website:\n\n";
    
    while (my $q = $response->{questions}->next()) {
	next if ($q->format() =~ /^(hidden|label)$/);
	$q->set("version",$this->{version});
	my $question = $q->label();
	my $answer = $q->answer($response->id)->value();
	$message .= "$question: $answer\n";
    }
    $message .= "\n";
    $message .= "Response ID: ".$response->id()."\n";

    &SendMail($notify,$config{site}{administrator},$subject,$message);
    return;
}

# list of fields or questions for this form

sub question_names {
    my $this = shift;
    my @fname;
    if ($this->defined) {
	$this->{list}->reset();
	while (my $q = $this->{list}->next()) {
	    push @fname, $q->qname();
	}
    }
    return @fname;
}

sub question_labels {
    my $this = shift;
    my @label;
    if ($this->defined) {
	$this->{list}->reset();
	while (my $q = $this->{list}->next()) {
	    push @label, $q->label();
	}
    }
    return @label;
}

#=== DB

# load record : can pass an ID or a datahash

sub load {
    my ($this,$data) = @_;
    $this->{list} = undef;
    if ($this->SUPER::load($data)) {
	my $list =  new ExSite::ObjectList(type=>"qa_question");
	$list->load({qa_form_id=>$this->id});
	$list->sort("sortkey","qa_question_id");
	$this->{list} = $list;
    }
}

sub count_responses {
    my $this = shift;
    if ($this->defined) {
	return $share{DB}->count("qa_response",{qa_form_id=>$this->id});
    }
    return undef;
}

sub count_questions {
    my $this = shift;
    if ($this->defined) {
	return $this->{list}->count();
    }
    return undef;
}

sub copy {
    my $this = shift;
    # only copy forms, not libraries?
    if ($this->SUPER::copy()) {
	# also copy questions

    }
}

1;

