#!/usr/bin/perl
#-----------------------------------------------------------------------
#
#   Copyright 2001-2009 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

package Modules::QA::Response;

use strict;
use ExSite::Config;
use ExSite::Base;
use ExSite::Form;
use ExSite::Misc;
use ExSite::Object;
use ExSite::ObjectList;
use Modules::QA::Answer;
use Modules::QA::Form;

use vars qw(@ISA);
@ISA = qw(ExSite::Object);

sub my_type { return "qa_response"; }

sub name {
    my $this = shift;
    $this->load() if (!defined $this->{data});
    return "Response ".$this->id;
}

sub form {
    my $this = shift;
    $this->load() if (!defined $this->{data});
    return new Modules::QA::Form(id=>$this->getdata("qa_form_id"));
}

sub get_response_data {
    my $this = shift;
    my %data;
    if ($this->loaded()) {
	my $ans = $this->{answers};
	$ans->reset();
	while (my $answer = $ans->next()) {
	    my $question = $answer->question();
	    $data{$question->qname()} = $answer->value();
	}
    }
    return wantarray ? %data : \%data;
}

sub answer {
    my ($this,$qname) = @_;
    if ($this->loaded()) {
	my $ans = $this->{answers};
	$ans->reset();
	while (my $answer = $ans->next()) {
	    return $answer if ($qname eq $answer->question->qname);
	}
    }

    ### FIXME: instantiate an empty answer, in case we want to define it

    return undef;
}

sub question {
    my ($this,$qname) = @_;
    if ($this->loaded()) {
	my $q = $this->{questions};
	$q->reset();
	while (my $question = $q->next()) {
	    return $question if ($qname eq $question->qname);
	}
    }
    return undef;
}

sub show {
    my ($this,%opt) = @_;
    if ($this->ok) {
	if ($this->loaded()) {
	    my $name = ucfirst $this->name;
	    if (! $this->{data}) {
		return $this->error($msg{"Sorry, the requested response does not exist."});
	    }
	    elsif (! $this->{answers}) {
		return $this->error(&substitute($msg{"[[response]] has no answers.",{response=>$name}}));
	    }
	    else {
		my $r = new ExSite::ReportBuilder(
						  title=>$this->form->name." (".$this->name.")",
						  headers=>["#",$msg{Question},$msg{Answer}],
						  );
		my $num = 0;
		while (my $q = $this->{questions}->next()) {
		    if ($opt{version}) {
			$q->set("version",$opt{version});
		    }
		    next if ($q->format eq "label");  # not really a question
		    $num++;
		    $r->push($num,$q->label(),$q->answer($this->id)->show());
		}
		$r->foot(&substitute($msg{"Response submitted at [[time]]."},{time=>$this->showdata("ctime")}));
		return $r->make();
	    }
	}
    }
    return $this->error($this->{status});
}

#=== DB

# load record : can pass an ID or a datahash

sub defined {
    my $this = shift;
    if (! $this->{data} || ! $this->{questions}) { $this->load(@_); }
    return ($this->{data} && $this->{questions});
}

sub load {
    my ($this,$data) = @_;
    $this->{list} = undef;
    if ($this->SUPER::load($data)) {
	# load questions
	my $q =  new ExSite::ObjectList(type=>"qa_question");
	$q->load({qa_form_id=>$this->{data}{qa_form_id}});
	$q->sort("sortkey","qa_question_id");
	$this->{questions} = $q;

	# load answers, but first sort them into the question order
	my %qkey = keywise("qa_question_id",scalar $q->getlist());
	my @ans = sort { $qkey{$a->{qa_question_id}}{sortkey} <=> $qkey{$b->{qa_question_id}}{sortkey} } $this->DB->fetch_match("qa_answer",{qa_response_id=>$this->id});
	my $a = new ExSite::ObjectList(type=>"qa_answer");
	$a->push(@ans);
	$this->{answers} = $a;
    }
}

sub copy {
    my $this = shift;



}

sub make_response {
    my ($this,$response_data,$answer_data) = @_;
    # data is question_id => answer value
    
    ### validation is done in form, but we should also validate in case
    ### a response is inserted indirectly

    # We do not add a note value by default;  the response must be updated
    # with a note and resaved if there is anything special to note about it.

    $this->{data} = $response_data;
    my $rid = $this->insert();
    if ($rid) {
	foreach my $qid (keys %$answer_data) {
	    next if (! $answer_data->{$qid});   # no answer given
	    # trim leading/trailing whitespace from answer
	    my $value = $answer_data->{$qid};
	    $value =~ s/\s+$//;
	    $value =~ s/^\s+//;
	    my $a = new Modules::QA::Answer(data=>{
		qa_response_id=>$this->id,
		qa_question_id=>$qid,
		value=>$value
	    });
	    my $aid = $a->insert();
	    if (! $aid) {
		$this->warn("Failed to insert answer to question $qid: $answer_data->{$qid}");
	    }
	}
    }
    else {
	$this->error("Failed to insert response.");
    }
    return $rid;
}

1;
