package Modules::SimpleMenu;

#----------------------------------------------------------------------------
#
#   Copyright 2001-2009 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#-----------------------------------------------------------------------

=pod

=head1 Modules::SimpleMenu - generic menu-builder for ExSite

SimpleMenu is a multi-purpose menu-builder for ExSite websites.  It
will create context-specific menus on any web page in a site.
Context-specific means that the site navigation links will reflect the
current location in the site.

SimpleMenu can also serve as a code template for a customized menu
builder, since it contains a general purpose menu-construnction
algorithm based on the ExSite page placement parameters.

SimpleMenu can build four types of menus:

=over 4

=item vertical 

navigation links are stacked in a vertical list, with submenus
exploded within this list.  This is the default menu type.

=item horizontal 

navigation links are listed left-to-right, menubar style.  Only the
top-level links in the site are shown, and no submenus are displayed.

=item path 

the path from the site home page to the current page
is displayed in the "cookie crumb" format, ie. page1 > page2 > page3

=item sitemap 

the full site structure is shown, with child
pages indented beneath their parents.

=back

When inserting a web application tag into a block of HTML, any of the
above types may be provided as a parameter.

When manually coding a SimpleMenu dynamic content tag, the format is:

C<E<lt>!--&SimpleMenu(format)--E<gt>>

where "format" is one of the above menu types.

=head3 Sub-menus

SimpleMenu builds global menus for the whole website by default.  That
is, the "top" level pages displayed in the menu are those that have no
parent page defined.  Sub-menus show pages that have a common parent page.

SimpleMenu automatically inserts sub-menus into vertically-formatted menus
if the viewer is viewing a top-level parent page that has child pages, or 
is viewing any of those child pages.  It does descend any further 
into the menu hierarchy than this.

If you want to build a menu starting from one of the submenus, instead
of from the top-level, then add a "submenu" keyword to the parameters:

C<E<lt>!--&SimpleMenu(vertical,submenu)--E<gt>>

C<E<lt>!--&SimpleMenu(horizontal,submenu)--E<gt>>

In these cases, the menu shows the pages that are children of the
currently-viewed page.  If the currently-viewed page has no children,
then it shows the submenu to which the current page belongs.  This is
the same as the top-level menu if the viewer is currently on a top-level
page.  If you never want to show ONLY submenu pages, and never top-level 
pages, then change <tt>submenu</tt> to <tt>submenu-only</tt> in the above
parameters.

=head2 Customizing Menu Appearance

=head3 CSS

The following CSS classes are used, and may be customized in any
given site's stylesheets:

=over 4

=item div.vmenu 

All vertically-formatted menus are wrapped in a div of the "vmenu" class.

=item span.hmenu 

All horizontally-formatted menus are wrapped in a span of the "hmenu" class.

=item span.pmenu 

All path menus are wrapped in a span of the "pmenu" class.

=item ul.sitemap 

All sitemaps are contained in an unnumbered list of the "sitemap" class.

=item a.menu

All menu links in vertical, horizontal, and path menus are of the "menu" class.

=item a.submenu

All submenu links in vertically-formatted menus are of the "submenu" class.

=item a#toppage

If a submenu is open in a vertically-formatted menu, the parent page has the
id "toppage".

=item a#thispage

The page currently being viewed has the id of "thispage" (unless it has 
already been tagged as "toppage", above.

=back

=head3 Menu Templates

Vertically-formatted menus will attempt to inline special content objects
to spice up the menu presentation:

=over 4

=item SimpleMenuTop

This content will be placed before all menu links.

=item SimpleMenuMid

This content will be placed between all menu links.

=item SimpleMenuBot

This content will be placed after all menu links.

=back

Using these content objects, a template can incorporate custom table layouts,
border graphics, and other visual tricks to lay out the menu.

You can do the same thing with horizontally-formatted menus, using
content objects named B<SimpleMenuTopH>, B<SimpleMenuMidH>, and
B<SimpleMenuBotH>.

=head3 Menu Spacer Strings

If the content objects noted above do not exist, then SimpleMenu will
used some simple spacer strings in their place.  These are defined
in the SimpleMenu configuration file, which should be located in 
C<cgi/conf/SimpleMenu.conf>:

=over 4

=item hmenu_spacer 

inserted between menu links in a horizontal menu.  Default: C<&nbsp;|&nbsp;>

=item pmenu_spacer

inserted between menu links in a path menu.  Default: C<&nbsp;&gt;&nbsp;>

=item vmenu_spacer

inserted between menu links in a vertical menu.  Default:
C<E<lt>br<gt>> Note that if you use display:block in your CSS for
menu links, the vmenu_spacer should be left blank to prevent extra
blank lines from being added to your menu layout.

=item submenu_prefix 

inserted in front of submenu links.  Default: C<&nbsp;&nbsp;&nbsp;&middot;&nbsp;>

=back

=head2 Sections

By default, SimpleMenu displays a menu for the current section.

To generate a menu for the parent section instead of the current
section, add the "parent" parameter:

    <!--&SimpleMenu(horizontal,parent)-->

To generate a menu for a child section instead of the current section, 
specify the child's section ID:

    <!--&SimpleMenu(horizontal,section=NNN)-->

=head3 Subsections in Menus

By default, SimpleMenu displays only the pages within a section.  If
you add the following configuration setting in its config file:

    show_subsections = 1

Then it will also include subsections under the current section.
Subsections are listed after all pages, and in the order the
subsections were created.  These cannot be reordered.

=head1 Configuring Menus

Page order and nesting in menus is determined by the following page
attributes:

=over 4

=item parent_id

Sets the page that this pages nests under.  Set to 0 (or unselect the
page) to make the page a top-level page.

=item rank

This is a number (typically 1-9, but you can use any numbers) used to
sort pages at a particular level in the menus.  Rank 1 pages are
placed before rank 2 pages, etc.  Pages of the same rank have no
particular order with respect to each other.

=back

You can set these values manually in the page configuration forms of most
of the CMS tools.

The easiest way to reorganize your menus, however, is to use the
SimpleMenu control panel which allows you to configure these settings
using a simple drag-and-drop interface.  Click and drag any page to
any other position in the site map.  The new position will be shown
with a dashed line so that you can tell what level of the menus you
are dropping it into.  Once you have your sitemap organized the way
you want, you must click the "Save" button to have SimpleMenu rewrite
the above menu configuration parameters for all of the affected pages.

Note that you can only reorder pages in the menu;  subsections always
appear at the end of the menu in a fixed order.

=cut

use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::Util;
use ExSite::Page;
use ExSite::Input;
use ExSite::Tree;
# inherit from BaseDCD class
use Modules::BaseDCD;
use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD);

sub write {
    my ($this, $opt) = @_;

    # setup 

    my $Page = $share{Page};
    return if (!$Page); 
    my $sid = $Page->id("section");
    if ($opt =~ /parent/) {
	my $parent_id = $Page->get("section")->{parent_id};
	if ($parent_id) {
	    $sid = $parent_id;
	}
	else {
	    return "";
	}
    }
    if ($opt =~ /section=(\d+)/) {
	# show menu for a subsection
	my $sid2 = $1;
	my $s2 = new ExSite::Section(id=>$sid2);
	if ($s2->get_my("parent_id") == $sid) {
	    my $version = $Page->get("page","version");
	    $this->{map} = $s2->get_mapped_pages(0,{version=>$version});

	    if ($config{SimpleMenu}{show_subsections}) {
		my @site = $share{DB}->fetch_match("section",{parent_id=>$sid2});
		foreach my $s (@site) {
		    $s->{index} = "section ".$s->{section_id};
		    $s->{pindex} = "section ".$s->{parent_id};
		    $this->{map}->addtopnode($s,"index","pindex");
		}
	    }
	}
	else {
	    # you cannot view a menu of an unrelated section
	    return $this->error("Section $sid: permission denied");
	}
    }
    elsif ($Page->is_page()) {
	# show menu for current section
	my $version = $Page->get("page","version");
	my $s = new ExSite::Section(id=>$sid);
	$this->{map} = $s->get_mapped_pages(0,{version=>$version});

	if ($config{SimpleMenu}{show_subsections}) {
	    my @site = $share{DB}->fetch_match("section",{parent_id=>$sid});
	    foreach my $s (@site) {
		$s->{index} = "section ".$s->{section_id};
		$s->{pindex} = "section ".$s->{parent_id};
		$this->{map}->addtopnode($s,"index","pindex");
	    }
	}
    }
    else {
	# probably a template; make a fake map
	$this->{map} = new ExSite::Tree("page_id","parent_id",
	      ({page_id=>1,parent_id=>0,label=>"Home",access=>0,url=>"#",section_id=>$sid},
	       {page_id=>2,parent_id=>0,label=>"About",access=>0,url=>"#",section_id=>$sid},
	       {page_id=>3,parent_id=>0,label=>"Contact",access=>0,url=>"#",section_id=>$sid},
	       {page_id=>4,parent_id=>0,label=>"Links",access=>0,url=>"#",section_id=>$sid},
	       ));
    }

    # generate a menu

    $this->{menu_mode} = $this->menu_mode();
    if ($opt =~ /horizontal/) { return $this->hmenu($opt); }
    elsif ($opt =~ /path/) { return $this->pmenu($opt); }
    elsif ($opt =~ /sitemap/) { return $this->sitemap($opt); }
    elsif ($opt =~ /next/) { return $this->nextpage($opt); }
    else { return $this->vmenu($opt); }
}

sub ioctl {
    my $this = shift;
    $_ = shift;
    if (/ModuleName/) {
	return "Menus";
    }
    elsif (/ModuleInfo/) {
	return "SimpleMenu is a multi-purpose menu-building system.
It automatically assembles site menus and site maps by inspecting the
relationships between your webpages.  It can build vertically-formatted
menus with exploded sub-menus, horizontally-formatted menus, and 
path menus (also known as \"cookie-crumb\" menus) that show you where
you are in the site.  It has options for customizing the formatting and
layout of menus so that they can be incorporated into most graphic designs.
SimpleMenu does not do dynamic pop-up menus, or menus with graphical 
buttons.  These options will require specialized menu plug-ins.";
    }
    elsif (/DynContentParameter/) {
	return "<p><b>Menu Type:</b>
<select name=\"param\">
<option value=\"\">(default)</option>
<option value=\"vertical\">Vertical (top-level pages)</option>
<option value=\"vertical,submenu\">Vertical (submenu or main menu pages)</option>
<option value=\"vertical,submenu-only\">Vertical (only submenu pages)</option>
<option value=\"vertical,all\">Vertical (all submenus shown)</option>
<option value=\"horizontal\">Horizontal (top-level pages)</option>
<option value=\"horizontal,submenu\">Horizontal (submenu or main menu pages)</option>
<option value=\"horizontal,submenu-only\">Horizontal (only submenu pages)</option>
<option value=\"path\">Path to current page</option>
<option value=\"next\">Link to next page</option>
<option value=\"sitemap\">Site Map</option>
</select></p>\n";
    }
    elsif (/ControlPanel/) {
	return \&ctrl_panel;
    }
    elsif (/Category/) {
	return "CMS";
    }
    elsif (/isStatic/) {
	return 0;
    }
    return undef;
}

# hmenu - return a horizontal menu (global or child pages)

sub hmenu {
    my $this = shift;
    my $opt = shift;

    my $CGIpath = $config{server}->{CGIpath};
    my $HTMLpath = $config{server}->{HTMLpath};
    my $db = $share{DB};
    my $Page = $share{Page};
    my $this_page = $Page->id;
    my $version = $Page->get("page","version");
    my $wraptag = $config{SimpleMenu}{wrap} || "span";

    my ($menutop, $menumid, $menubot);
    $menutop = $Page->find("SimpleMenuTopH")->get_html() ||
	"<!-- begin menu -->\n<$wraptag class=\"hmenu\">\n";
    $menumid = $Page->find("SimpleMenuMidH")->get_html() ||
	$config{SimpleMenu}{hmenu_spacer};
    $menubot = $Page->find("SimpleMenuBotH")->get_html() || 
	"</$wraptag>\n<!-- end menu -->\n";

    # fetch all menu-able pages from this section
    my @pages = $this->get_top_pages($opt);
    my $access = $db->authorize;
    my @menu;
    foreach my $page (@pages) {
	next if ($page->{visibility} =~ /^(hidden|sitemap-only)$/);
	next if ($config{SimpleMenu}{no_default_version} &&
		 $version ne $page->{version});
	my $url = $page->{url};
	if (!$url) {
#	    my $p = new ExSite::Page(page=>$page);
	    $url = $this->get_url($page);
	}
	my $id;
	if ($Page->id() == $page->{page_id}) { 
	    $id = " id=\"thispage\""; 
	}
	elsif ($Page->get_my("parent_id") == $page->{page_id}) { 
	    $id = " id=\"toppage\""; 
	}
	my $label = $page->{label};
	if ($config{SimpleMenu}{nobreak}) {
	    $label =~ s/\s+/&nbsp;/g;
	}
	push @menu, "<a href=\"$url\" class=\"menu\"$id>$label</a>";
    }

    # optional links for members only
    if (! $config{SimpleMenu}{nologout}   # include logout links
	&& $access                        # if user is logged in
	&& $opt !~ /child|submenu|parent/ # on the main menu only
	&& ! $Page->{publish}             # if this is a dynamic page view
	&& ! $Page->is_template) {        # of a real page
	push @menu, "<a href=\"$CGIpath/logout.cgi\" class=\"menu\">$msg{Logout}</a>";
    }

    return $menutop.join($menumid, @menu).$menubot;
}

# pmenu - return a path menu
#
# Note that for pmenu we do not put the content into the menu class,
# because most commonly a pmenu is placed into the page body, not the
# menu bar.

sub pmenu {
    my $this = shift;
    my $opt = shift;

    my $CGIpath = $config{server}->{CGIpath};
    my $HTMLpath = $config{server}->{HTMLpath};
    my $db = $share{DB};
    my $Page = $share{Page};
    my $id = my $this_page = $Page->id;
    my $version = $Page->get("page","version");
    my $pdata = $Page->get;
    my $purl = $this->get_url($Page);
    my $label = $pdata->{label};
    if ($config{SimpleMenu}{nobreak}) {
	$label =~ s/\s+/&nbsp;/g;
    }
    my @path = "<a href=\"$purl\">$label</a>";
    my $wraptag = $config{SimpleMenu}{wrap} || "span";
    my $out = "<!-- begin menu --><$wraptag class=\"pmenu\">\n";

    my $toppage;
    while (my $page = $this->{map}->get_parent_data($id)) {
	$toppage = $page;
#	$Page = new ExSite::Page(page=>$page);
	$purl = $this->get_url($page);
	$label = $page->{label};
	if ($config{SimpleMenu}{nobreak}) {
	    $label =~ s/\s+/&nbsp;/g;
	}
	unshift @path, "<a href=\"$purl\">$label</a>";
	$id = $page->{parent_id};
    }
    if ($toppage->{filename} !~ /^index\./) {
	# also include the index page in the path
	my $db = $share{DB};
	my @p = $db->fetch_match("page",
				 {section_id=>$pdata->{section_id},
				  filename=>"index.%",
				  status=>"active",
				  visibility=>"visible"},
				 "page_id");
	if ($version) { 
	    foreach my $page (@p) {
		my $pid = $page->{page_id};
		my @alt = $db->fetch_match("page",{parent_id=>$pid,
						   type=>"alternate",
						   version=>$version,
					       },"page_id");
		if (@alt > 0) {
		    # alternate version of this page was found
		    # swap it in place
		    $page = $alt[-1];
		}
	    }
	}
	if (@p > 0) {
	    $Page = new ExSite::Page(page=>$p[-1]);
	    $purl = $this->get_url($Page);
	    $pdata = $Page->get;
	    $label = $pdata->{label};
	    if ($config{SimpleMenu}{nobreak}) {
		$label =~ s/\s+/&nbsp;/g;
	    }
	    my $link = "<a href=\"$purl\">$label</a>";
	    if ($link ne $path[0]) { 
		unshift @path, $link;
	    }
	}
    }

    $out .= join $config{SimpleMenu}{pmenu_spacer},@path;
    $out .= "</$wraptag><!-- end menu -->\n";

    return $out;
}

sub vmenu {
    my $this = shift;
    my $opt = shift;

    my $CGIpath = $config{server}->{CGIpath};
    my $HTMLpath = $config{server}->{HTMLpath};
    my $db = $share{DB};
    my $Page = $share{Page};
    my $this_page = $Page->id;
    my $pdata = $Page->get;
    my $wraptag = $config{SimpleMenu}{wrap} || "span";
    my $version = $Page->get("page","version");

    my $out;
    
    # get menu template HTML
    my ($menutop, $menumid, $menubot);
    $menutop = $Page->find("SimpleMenuTop")->get_html();
    if ($menutop) {
	$menumid = $Page->find("SimpleMenuMid")->get_html();
	$menubot = $Page->find("SimpleMenuBot")->get_html();
    }
    else {
	$menutop = "<!-- begin menu -->\n<$wraptag class=\"vmenu\">\n";
	$menumid = $config{SimpleMenu}{vmenu_spacer};
	$menubot = "</$wraptag>\n<!-- end menu -->\n";
    }

    my @items;

    my $access = $db->authorize;
    my $parent = $Page->parent;
    my $ppid = $parent->id();
    
    my $separate_submenu;

    # get starting pages of the menu
    my @map = $this->get_top_pages($opt);
    while (@map > 0) {
	my $p = shift @map;

	# skip hidden pages
	next if ($p->{visibility} =~ /^(hidden|sitemap-only)$/);

	# skip pages if they are not the same version
	next if ($config{SimpleMenu}{no_default_version} &&
		 $version ne $p->{version});

	my $url = $this->get_url($p);
	my $id = (exists $p->{page_id}) ? $p->{page_id} : 0;

	# show submenu if we are on this top page, or any descendant page of it
	my @submap = $this->{map}->get_child_data($id);
	my $this_submenu = ($id == $this_page);
	my $need_submenu = ($id == $this_page) || ($opt =~ /all/ || ($p->{page_id} == $ppid && $config{SimpleMenu}{enable_multilevel_submenu}));
	my $ancestor = $this->{map}->get_ancestor_data($this_page);
	if (! $this_submenu) {
	    $this_submenu = $ancestor ? ($ancestor->{page_id} == $id) : 0;
	}
	my $toppage = $ancestor ? ($ancestor->{page_id} == $id) : 0;
	if (! $need_submenu) {
	    $need_submenu = $toppage;
	}

	my (@submenu,$aid);
	if ($need_submenu) {
		push(@submenu, $this->build_sub_menus($opt, \@submap));
	}

	#my (@submenu,$aid);
	#if ($need_submenu) {
	#    while (@submap > 0) {
	#	my $p = shift @submap;
	#	next if ($p->{visibility} =~ /^(hidden|sitemap-only)$/);
	#	next if ($config{SimpleMenu}{no_default_version} &&
	#		 $version ne $p->{version});
	#
	#	my $url = $this->get_url($p);
	#	$aid = (exists $p->{page_id} && ($p->{page_id} == $this_page || $p->{page_id} == $ppid )) ? 
	#	    " id=\"thispage\"" : "";
	#	my $label = $p->{label};
	#	if ($config{SimpleMenu}{nobreak}) {
	#	    $label =~ s/\s+/&nbsp;/g;
	#	}
	#	push @submenu,"$config{SimpleMenu}{submenu_prefix}<a href=\"$url\" class=\"submenu\"$aid>$label</a>";
	#   }
	#}

	my $tagtype = $config{SimpleMenu}{tag} || "toppage";
	my $thispage;
	if ($id == $this_page) { $thispage = 1; }
	if ($thispage && ! ($toppage && $tagtype eq "toppage")) {
	    $aid = " id=\"thispage\"";
	}
	elsif ($toppage) {
	    $aid = " id=\"toppage\"";
	}
	my $label = $p->{label};
	if ($config{SimpleMenu}{nobreak}) {
	    $label =~ s/\s+/&nbsp;/g;
	}
	
	if (@submenu > 0 && $opt =~ /full-tree/) {
	    my $submenu_special;
	    my $count = 0;
	    
	    if ( $config{SimpleMenu}{last_menu_level_css}{enable} ) {
		    while (@submenu) {
		        my $submenu = shift(@submenu);
		        
		        if (!@submenu) {
		        	$submenu_special = " id=\"thissubmenulast\"";
		        }
			elsif ($this_submenu) {
				$submenu_special = " id=\"thissubmenu$count\"";
			}

			$separate_submenu .= "\n<$wraptag class=\"submenu\"$submenu_special>\n$submenu\n</$wraptag>\n";
			$count++;
		    }
	    }
	    else {
		    foreach my $submenu (@submenu) {
			if ($this_submenu) {
				$submenu_special = " id=\"thissubmenu$count\"";
			}

			$separate_submenu .= "\n<$wraptag class=\"submenu\"$submenu_special>\n$submenu\n</$wraptag>\n";
			$count++;
		    }
	    }
	    
	    push @items, "<a href=\"$url\" class=\"menu\"$aid>$label</a>";
	}
	elsif (@submenu > 0) {
	    my $submenu_special;
	    if ($this_submenu) {
		$submenu_special = " id=\"thissubmenu\"";
	    }
	    push @items, "<a href=\"$url\" class=\"menu\"$aid>$label</a>$config{SimpleMenu}{vmenu_spacer}\n<$wraptag class=\"submenu\"$submenu_special>\n".join("$config{SimpleMenu}{vmenu_spacer}\n",@submenu)."\n</$wraptag>\n";
	}
	else {
	    my $menu_class = "menu";
	    $menu_class = "menulast" if ($config{SimpleMenu}{last_menu_level_css}{enable});
	    push @items, "<a href=\"$url\" class=\"$menu_class\"$aid>$label</a>";
	}
    }	

    # optional links for members only
    if ($opt !~ /submenu-only|child|parent/) {   # only show on main menus
	if ($db && $Page) {
	    if (! $Page->{publish}) {
		my $level = $db->authorize;
		if ($level) {
		    push @items, "<a href=\"$CGIpath/logout.cgi\" class=\"menu\">$msg{Logout}</A>";
		}
	    }
	}
    }

    my $menu = "$menutop\n".
	join("$menumid\n",@items).
	"\n$menubot\n";

    if ($config{SimpleMenu}{separate_submenu}) {
    	$menu = "<div id=\"" . $config{SimpleMenu}{main_menu_id} . "\">$menu</div> $separate_submenu";
    }

    return $menu;
}

sub nextpage {
    my $this = shift;
    my $Page = $share{Page};
    my $next_page = $this->{map}->get_next_data($Page->id);
    if ($next_page) {
	my $url = $this->get_url($next_page);
	return "<div class=\"nextPage\"><a class=\"menu\" href=\"$url\">$next_page->{label}</a></div?>\n";
    }
    return undef;
}

sub build_sub_menus {
	my ($this, $opt, $submap) = @_;

	my $Page = $share{Page};
	my $parent = $Page->parent;
	my $this_page = $Page->id;
	my $version = $Page->get("page","version");	
	my $ppid = $parent->id();
	
	my (@submenu,$submenu,$aid,@subsubmaps,$need_submenu);
	
	foreach my $p (@$submap) {
		next if ($p->{visibility} =~ /^(hidden|sitemap-only)$/);
		next if ($config{SimpleMenu}{no_default_version} &&
			 $version ne $p->{version});

		my $url = $this->get_url($p);
		$aid = (exists $p->{page_id} && ($p->{page_id} == $this_page || $p->{page_id} == $ppid )) ? 
		    " id=\"thispage\"" : "";
		my $label = $p->{label};
		if ($config{SimpleMenu}{nobreak}) {
		    $label =~ s/\s+/&nbsp;/g;
		}
		
		$submenu .= "$config{SimpleMenu}{submenu_prefix}<a href=\"$url\" class=\"submenu\"$aid>$label</a>\n";
		
		if ($opt =~ /full-tree/) {
			my $subsubmap = $this->{map}->get_child_data($p->{page_id});
			$need_submenu = ($p->{page_id} == $this_page || $p->{page_id} == $ppid);
			push(@subsubmaps, $subsubmap) if ($need_submenu);
		}
	}
	
	push(@submenu, $submenu) if ($submenu);
	
	if ($opt =~ /full-tree/) {
		foreach my $subsubmap (@subsubmaps) {
			push(@submenu, $this->build_sub_menus($opt, $subsubmap));
		}
	}
	
	return @submenu;
}

sub menu_mode {
    my $this = shift;
    # always do static menus when publishing
    return 1 if ($share{Page}->is_publishing());
    # otherwise, use static menus only when not in CMS mode
    return 0;
    #return &get_cookie("ExSiteCMS") ? 0 : 1;
}

sub get_url {
    my ($this,$p) = @_;

    if ((ref $p) =~ /Page/) {
	if ($this->{menu_mode}) {
	    # link to public view
	    return $p->get_url_canonical();
	}
	else {
	    # link to preview if page is not yet published
	    return $p->get_url();
	}
    }
    elsif ((ref $p) eq "HASH") {
	if (exists $p->{index}) {
	    # subsection hash
	    my $s = new ExSite::Section(section=>$p);
	    return $s->get_url();
	}
	elsif ($p->{url}) {
	    # dummy page hash
	    return $p->{url}
	}
	else {
	    # page hash
	    my $page = new ExSite::Page(page=>$p);
	    if ($this->{menu_mode}) {
		# link to public view
		return $page->get_url_canonical();
	    }
	    else {
		# link to preview if page is not yet published
		return $page->get_url();
	    }
	}
    }
}

sub sitemap {
    my $this = shift;
    my $out;
    my $db = $share{DB};
#    my $page = $share{Page}->get;
#    my @pages = $db->fetch_match("page",{section_id=>$page->{section_id},
#					 type=>"page",
#					 parent_id=>0,
#					 visibility=>"visible"},"rank");
    my @pages = $this->get_top_pages();
    my $wraptag = $config{SimpleMenu}{wrap} || "span";
    $out .= "<$wraptag class=\"sitemap\">\n<UL CLASS=\"sitemap\">\n";
    foreach my $p (@pages) {
	next if ($p->{visibility} =~ /^(hidden|menu-only)$/);
	$out .= $this->sitemap_r($p);
    }
    $out .= "</UL></$wraptag>\n";
    return $out;
}

sub sitemap_r {
    # recursive mapping routine
    my ($this,$page) = @_;
    my $db = $share{DB};

    # NB: will also work if $page is actually a subsection
    my $pid = $page->{page_id};
    my $url = $this->get_url($page);
    my $label = $page->{label};
    my $access = $page->{access} || "public";

    my $out = "<li><a href=\"$url\">$label</a>";
    if ($access ne "public") { $out .= " (".$msg{"restricted access"}.")"; }
    $out .= "</li>\n";
    if ($access ne "public" && $db->level() == 0) { return $out; }

    my @submap = $this->{map}->get_child_data($pid);
    my $submenu;
    foreach my $page (@submap) { 
	next if ($page->{visibility} =~ /^(hidden|menu-only)$/);
	$submenu .= $this->sitemap_r($page); 
    }
    if ($submenu) {
	$out .= "<ul class=\"sitemap\">\n$submenu\n</ul>\n";
    }
    return $out;
}

sub get_top_pages {
    my ($this,$opt) = @_;
    my $Page = $share{Page};
    my $this_page = $Page->id;
    my @pages;
    if ($opt =~ /child|submenu/) {
	# see if there are any submenu pages
	@pages = $this->{map}->get_child_data($this_page);
	if (@pages == 0 || $config{SimpleMenu}{enable_multilevel_submenu}) {
	    # no submenu;  rebuild the submenu we're already on
	    my $parent = $Page->get("page","parent_id");
	    if ($parent) {
	    	if ($config{SimpleMenu}{enable_multilevel_submenu}) {
	    		my $p = new ExSite::Page(id=>$parent);
	    		my $page_id = $p->get("page","parent_id") || $p->id();
	    		@pages = $this->{map}->get_child_data($page_id);
	    	}
	    	else {
			@pages = $this->{map}->get_child_data($parent);
		}
	    }
	}
    }
    if (@pages == 0 && $opt !~ /submenu-only/) {
	@pages = $this->{map}->get_topnodes_data();
    }
    return @pages;
}

# drag-drop control panel

# sort function for sorting the serialized post data
sub hashDepthSort {
	my @a_matches = ($a =~ /\[(\d+)\]/g);
	my @b_matches = ($b =~ /\[(\d+)\]/g);	
	my $a_count = scalar @a_matches;
	my $b_count = scalar @b_matches;	
	return $a_count cmp $b_count;
}

sub ctrl_update_hierarchy {
	my ($this) = @_;
	my $in = $this->{input};
	if ($in->{serialized}) {
		my %hash = &DecodeString($in->{serialized});
		
		my %tree;
		foreach my $key (sort hashDepthSort (keys %hash)) {
			my $page_id = $hash{$key};
			# match the hierarchial rank of the page
			my @matches = ($key =~ /\[(\d+)\]/g);
			# index the page by hierarchy so we can retrieve page_id of parent
			my $match_string = join(";",@matches);
			$tree{$match_string} = $page_id;
			my $depth = scalar @matches - 1;
			# the match string (index) for the parent is the match string of the page minus the final rank of the page
			my $parent_match_string = join(";",@matches[0..$depth-1]);
			my $rank = $matches[$depth];
			my $parent = $tree{$parent_match_string} || 0;
			$share{DB}->update("page",{rank=>$rank,parent_id=>$parent},{page_id=>$page_id});
		}
		my $out .= <<END;
<div class="bold">
SiteMap saved successfully!  Republish the website for changes to take effect on static pages.
</div>
END
		return $out;
	}
}	

sub ctrl_panel {
	my $this = shift;
	# set ctrl panel flag
	$this->{admin} = 1;
	my $file = "$config{server}->{HTMLpath}/_Modules/SimpleMenu/file.gif";
	my $folder = "$config{server}->{HTMLpath}/_Modules/SimpleMenu/folder.gif";

	my $out .= $this->set_section_id;
	# update hierarchy if there was a serialized post
	$out .= $this->ctrl_update_hierarchy;
	if ($this->{input}{section_id} && $share{DB}->owns($this->{input}{section_id})) {
		$this->{sid} = $this->{input}{section_id};
	} else {
		$this->{sid} = $this->get_section_id;
	}
	my $s = new ExSite::Section(id=>$this->{sid});
	my $site = $share{DB}->fetch("section",$this->{sid});

	# check for permission to reconfigure pages
	if ($this->{sid} && ! $s->allow_update("page",{section_id=>$this->{sid}})) {
	    return $this->error("Permission denied - you need administrator privileges to configure menus.");
	}

	$this->{map} = $s->get_mapped_pages(0);
	$out .= <<END;
<script type="text/javascript" src="$config{server}->{HTMLpath}/_Modules/SimpleMenu/jquery-1.1.4.js"></script>
<script type="text/javascript" src="$config{server}->{HTMLpath}/_Modules/SimpleMenu/interface.js"></script>
<script type="text/javascript" src="$config{server}->{HTMLpath}/_Modules/SimpleMenu/inestedsortable-1.0.1.pack.js"></script>
<link rel="stylesheet" type="text/css" href="$config{server}{HTMLpath}/_Modules/SimpleMenu/SimpleMenu.css">
END
	$out .= "<h2>$site->{title}</h2>";
	$out .= $this->sortable;
	$out .= <<END;
<script type="text/javascript">
\$(document).ready(function() {
\$('#left-to-right').NestedSortable(
{
	accept: 'page-item1',
	noNestingClass: "no-nesting",
	opacity: 0.8,
	helperclass: 'helper',
	activeclass: 'active',
	ghosting: false,
	fx: 400,
	revert: true,
	currentNestingClass: 'current',
	onChange: function(serialized) {
		\$('#left-to-right-ser').html("<input type=hidden name=serialized value=" + serialized[0].hash + ">");
	},
	autoScroll: true,
	handle: '.sort-handle'
});
});
</script>		
END
	return $out;
}

sub sitemap_ctrl {
    my $this = shift;
    my $out;
    my $db = $share{DB};
    my @pages = $this->get_top_pages_ctrl;
    $out .= "<div class=\"wrap\">\n<ul id=\"left-to-right\" class=\"page-list\">\n";
    foreach my $p (@pages) {
	
#	next if ($p->{visibility} =~ /^(hidden|menu-only)$/);
	$out .= $this->sitemap_r_ctrl($p);
	}
    $out .= "</ul></div>\n";
    return $out;
}

sub sitemap_r_ctrl {
    # recursive mapping routine
    my ($this,$page) = @_;
    my $db = $share{DB};
    my $pid = $page->{page_id};
    my $p = new ExSite::Page(page=>$page);
    my $url = $p->get_url;
	my $file = "$config{server}->{HTMLpath}/_Modules/SimpleMenu/file.gif";	
	if (($page->{visibility} !~ /hidden|sitemap-only/ && $page->{access} eq "public")) {
		$file = "$config{server}->{HTMLpath}/_Modules/SimpleMenu/page_white_text.png";
	} elsif ($page->{access} eq "members") {
		$file = "$config{server}->{HTMLpath}/_Modules/SimpleMenu/page_white_link.png";
	} elsif ($page->{visibility} =~ /hidden|sitemap-only/) {
		$file = "$config{server}->{HTMLpath}/_Modules/SimpleMenu/page_white_zip.png";
	}
	# fall back on title if page doesn't have menu label (hidden pages sometimes don't)
	my $label = $page->{label} || $page->{title};
	# background is a IE fix for their opacity problem
    my $out = "<li style='background-color:#e0ecff' id=\"ele-$pid\" class=\"clear-element page-item1 left\">\n<div class=\"sort-handle\"><img src=\"$file\" align=\"left\">$label";
	if (($page->{publish_method} eq "static" && $page->{access} eq "public")) {
		$out .= " ($page->{filename})";
	} elsif ($page->{access} eq "members") {
		$out .= " <i>[members-only page]</i>";
	} else {
		$out .= " <i>[dynamic page]</i>";
	}
	if ($page->{access} ne "public") {
		$out .= " <i>[RESTRICTED]</i>";
	}
	if ($page->{visibility} =~ /hidden|sitemap-only/) {
		$out .= " <i>[hidden page]</i>";
	}
	$out .= "\n</div>\n";
    my @submap = $this->{map}->get_child_data($pid);
    my $submenu;
    foreach my $page (@submap) { 
	$submenu .= $this->sitemap_r_ctrl($page); 
    }
    if ($submenu) {
	$out .= "<ul class=\"page-list\">\n$submenu</ul>\n";
    }
    $out .= "</li>\n";
    return $out;
}

sub get_top_pages_ctrl {
    my ($this,$opt) = @_;
    my $db = $share{DB};
	my $section = $this->{sid} || $this->get_section_id;
	my %match = (section_id=>$section,
		type=>"page",
		status=>"active",
		parent_id=>0,);
	my @pages = $db->fetch_match("page",\%match,["rank","page_id"]);
	return @pages;
}

sub sortable {
	my ($this) = @_;
	my $out;
	my $db = $share{DB};
	$out .= <<END;
<h3>Drag & Drop Site Map Menu Manager</h3>
<div class="wrap">
	Change the order of your website's pages by dragging and dropping them into place.
	Once you are happy with your "sitemap", click on the "save menu rank" button to save your changes.
	Keep in mind you still need to publish your website for the changes to take effect on any of your static pages. 
</div>
<br>
END
	my $url = $this->link;
	$out .= <<END;
<form class="sitemap" action="$url" method="post">
<p><input type="submit" value="Save Menu Rank"></p>
<div id='left-to-right-ser'></div>
</form>
END
	$out .= $this->sitemap_ctrl;
	return $out;
}

### ctrl_panel end

1;
