package Modules::ToDo;
#----------------------------------------------------------------------------
#
#   Copyright 2001-2008 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#-----------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Module qw(&module_list &get_module);
use Modules::BaseDCD;
use vars qw(@ISA $ml);
@ISA = qw(Modules::BaseDCD);

sub write {
    my ($this,$opt);
    $ml = new ExSite::ML;
    if ($this->{input}{mod} && $share{DB}->is_manager()) {
	# probably an AJAX call to update the control panel
	return $this->get_todo_list($this->{input}{mod});
    }
    return undef;
}

sub ioctl {
    my $this = shift;
    $_ = shift;
    if (/ModuleName/) {
	return "To-Do List";
    }
    elsif (/ModuleInfo/) {
	return "The To-Do plugin finds all tasks requiring administrator 
attention in all available plug-in modules, and provides links for the
admin to attend to those tasks.";
    }
    elsif (/ControlPanel/) {
	return \&ctrl_panel;
    }
    elsif (/Category/) {
	return "System";
    }
    return undef;
}

sub ctrl_panel {
    my $this = shift;
    my $out;
    $ml = new ExSite::ML;
    my @modules = sort &module_list;
    foreach my $modname (@modules) {
	$out .= $this->get_todo_list($modname);
    }
    if ($out) {
	return $out;
    }
    else {
	return $ml->p("There does not appear to be anything for you to do!");
    }
}

sub get_todo_list {
    my ($this,$modname) = @_;
    my $out;
    my $mod = &get_module($modname);
    my $br = $ml->br;
    if ($mod) {
	my $todo = $mod->ioctl('ToDo');
	if (ref $todo eq "CODE") {
	    my @todo_list = $mod->$todo();
	    if (@todo_list > 0) {
		my $name = $mod->ioctl("ModuleName") || $modname;
		$out .= $ml->div(
				 $ml->h2($name),
				 { id=>'ToDo_$modname' }
				 );
		my $list;
		foreach my $item (@todo_list) {

		    my $li = $ml->a(
				    $ml->strong($item->{title}),
				    { href=>"javascript:popup_large('$item->{link}')" }
				    );
		    if ($item->{description}) { 
			$li .= $br.$item->{description};
		    }
		    $list .= $ml->li($li);
		}
		$out .= $ml->ul($list);
#		my $link = $this->link(mod=>$modname);
#		$out .= "<p>".&ExSite::HTML::Button(url=>"javascript:subRequestData('ToDo_$modname','$link')",label=>"update $name to-do list")."</p></div>";
	    }
	}
    }
    return $out;
}

1;

=pod

=head1 ToDo

This module manages tasks that require the attention of an administrator.

Any plug-in module can create its own to-do lists of tasks.  The ToDo
module aggregates all of these to-do lists into a single master list
so that you can see at a glance what items require your attention
immediately.

To identify whether a module supports ToDo lists, we instantiate the
module, and then query it as follows:

    my $todofunc = $module->ioctl('ToDo');

This returns C<undef> if ToDo lists are not supported, and a code
reference if they are.  We can then call this code reference to obtain
the ToDo list:

    my @todo_list = $module->$todofunc();

The returned list is an array of hashes; each hash is a ToDo task
item, with three attributes/keys:

=over 4

=item url

This is the link the admin should follow to attend to the task.  It may
execute the task directly, or it may go to a preview screen from which the
task can be performed.

=item title

This is used as the anchor text for the above link.

=item description

This is additional descriptive text to describe the task, and is optional.

=back

The ToDo module simply displays this information for the convenience
of the administrator.  No other information or functionality is provided.

The links in the ToDo list are offered up by their respective
applications, and the ToDo plugin does not process these links.  These
links may execute the task directly, or they may take the
administrator to a screen at which the task may be attended to.  There
is no requirement that these links behave one way or the other.  It
may be helpful to indicate in the description what the links do, or
what the administrator must do after following the link.

Simply following a link does not guarantee that a task is
satisfactorily completed.  Even if a task is completed, that may in
turn cause new tasks to appear.  The ToDo control panel therefore does
not clear or update the ToDo list automatically.  You can use the
"update" links in the ToDo list to update the list for a particular
module, if you want to see how it has changed after attending to some
tasks.  (You can also refresh the whole control panel, but that may
take longer.

=cut
