#!/usr/bin/perl

=pod

=head1 Modules::Zine

This is a multi-purpose plug-in for the ExSite article management subsystem.
It provides end-user and administrator tools for reading, browsing, posting,
and managing various types of e-zines, articles, and comments.

The article management subsystem is contained in the Zine subdirectory.
This includes the following packages:

    Modules::Zine::Config  - defines basic rules and behaviours
    Modules::Zine::Base    - data and security management
    Modules::Zine::Article - formatting and output of articles
    Modules::Zine::Form    - handling all forms, input, and updates
    Modules::Zine::Index   - generic indexing engine
    Modules::Zine::Index*  - alternate indexing engines

Any plug-in can make use of this framework.  This package, the generic
Zine plug-in, provides a simple, flexible interface to all of the
features in the framework.  It also adds some high-level functions
that do not pertain to specific articles or e-zines, such as:

=over 4

=item directories of zines

=item highlight recent articles or comments

=item highlight a random article

=back

=cut

package Modules::Zine;

use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::Tree;
use ExSite::Time;
use ExSite::HTML;
use Modules::Zine::Article;
use Modules::Zine::Form;
use Modules::Zine::Digest;
use Modules::BaseDCD;

use vars qw(@ISA);
@ISA = qw(Modules::Zine::Article Modules::BaseDCD);

sub read {
    my ($this,$opt) = @_;
    # fetch overlayed query and form input
    my $in = new ExSite::Input;
    $this->{input} = $in->combine;
    # add path info to input hash as "path"
    if (! $this->{input}{aid}) {
	$in->path_info();
	$this->{input}{path} = $in->path();
    }
}

sub write {
    my ($this,$options) = @_;
    my (%opt,$in,$id,$cmd,$n);
    
    # allow bare ID, bare zine type, or bare article name to be passed

    %opt = &DecodeString($options);
    $this->{opt} = \%opt;
    $in = $this->{input};
    $this->clear;  # in case this isn't our first call to Zine
    $this->load_article($in->{aid});
    if (! $this->{article_id}) { 
	$this->load_article($opt{aid}); 
	if ($this->{article_id}) { $cmd = $opt{zine} || "show"; }
    }
    $id = $this->{article_id};
    if (! $cmd) {
	$cmd = $in->{zine} ||          # explicit cmd
	    ($id ? "show" : undef) ||  # implicit show
	    $opt{zine};                # hard-coded default
    }
    # variable n = number of articles to show for new articles, latest comments
    $n = $in->{n} || $opt{n};
    
    my $out;
    # modified to pass id and n / moved to top of write function

    # process any hard-coded command requests

    if ($opt{zine} eq "newarticles") {
	$out = $this->index_latest_articles($opt{aid},$opt{new_articles});
    }
    elsif ($opt{zine} eq "newcomments") {
	$out = $this->index_latest_comments($opt{aid},$opt{new_comments});
    }
    elsif ($cmd eq "rss") {
	my $p = $share{Page} || new ExSite::Page();
	$p->cancel; 
	print "Content-type: text/xml\n\n";
	print $this->rss();
	# tell dcd.cgi not to put out a text/html default content type header
	$share{noContentHeader} = 1;
	return undef;
    }
    elsif ($cmd eq "toggle") {
	$out = $this->index_toggle("article",$id,$n);
    }	
    elsif ($cmd eq "random") {
	return $this->index_random($id);
    }	
    elsif ($cmd =~ /zine?dex/) {
	return $this->zinedex($in->{type} || $opt{type}, undef, $opt{aid});
    }	
    elsif ($cmd =~ /feature/) {
	return $this->feature($opt{aid});
    }
    return $out if ($out);
    
    # process Zine forms

    if ($cmd =~ /^(submit|new|reply|post|newarticle|edit|approve|reject)$/) {
	my $formout .= $this->form($cmd);
	if ($formout) {
	    $out .= $formout;
	}
	elsif ($id) {
	    $out .= $this->show;
	}
	else {
	    $out = $this->zinedex($in->{type},$this->{input}{section_id});
	}
    }

    # user indexes

    elsif ($cmd =~ /^history$/ && $this->{config}{history}) {
	$out = $this->index_user_history($in->{uid},$id,$in->{limit});
    }

    # display or update articles

    elsif ($this->{article_id}) {
	if ($cmd eq "index") {
	    $out = $this->index(%opt,archive=>$in->{archive});
	}
	elsif ($cmd eq "del") {
	    $out = $this->delete();
	}
	else {
	    # show an article/zine
	    if ($this->{config}{use_metadata} &&    # zine controls metadata
		! $opt{nometa} &&                   # not disabled in this case
		ref $this->{article} eq "HASH" &&   # real article is loaded
		$share{Page}) {                     # on a public page
		# change our metadata to reflect the current article
		$share{Page}->set_metadata("title",$this->{article}{title});
	    }
	    $out = $this->show(archive=>$in->{archive});
	}
    }
    elsif (! $out) {
	$out = $this->topview($options);
    }
    return $this->wrap("Zine",$out);
}

sub ioctl {
    my ($this) = shift;
    $_ = shift;

    if (/ControlPanel/) {
	return \&ctrl_panel;
    }
    elsif (/ModuleName/) {
	return "e-Zines";
    }
    elsif (/isRestricted/) {
	return $this->{config}{isRestricted};
    }
    elsif (/isService/) {
	return $this->{config}{isService};
    }
    elsif (/Publish/) {
	return \&Modules::Zine::Base::publish;
    }
    elsif (/Search/) {
	return \&Modules::Zine::Base::search_index;
    }
    elsif (/Cron/) {
	return \&cron;
    }
    elsif (/ToDo/) {
	return \&todo;
    }
    elsif (/DynContentParam/) {
	my $section_id = shift;
	my $ml = &get_obj("ML");
	my $opt = 
	    $ml->option("&lt;default&gt;",{value=>""}).
	    $ml->option("show all e-zines",{value=>""});
	my @zine = $this->our_zines($section_id);
	foreach my $zine (@zine) {
	    $opt .= $ml->option($zine->{title},{value=>"aid=$zine->{article_id}&zine=show"});
	}
	return "E-Zines: " . $ml->select($opt,{name=>"param"});
    }
}

sub ctrl_panel {
    my $this = shift;
    my $out;
    my $ml = &get_obj("ML");
    my $sid = $this->get_section_id();
    my $in = $this->{input};
    $_ = $in->{zadm};
    $this->{admin} = 1;  # set admin flag to add admin tools

    if ($in->{zine} =~ /^(submit|new|reply|post|newarticle|edit|conf|newzine)$/) {
	my $formout .= $this->form($in->{zine});
	if ($formout) {
	    $out .= $formout;
	}
	elsif ($in->{aid}) {
	    $this->load($in->{aid});
	    $out .= $this->show;
	}
	else {
	    $out = $this->topview_admin;
	}
    }
    elsif ($in->{zine} || $in->{aid}) { 
	$out .= $this->write; 
    }
    elsif (/list/) {
	/^list(.*)$/;
	my $type = $1;
	$out .= $this->topview_admin($type);
	$out .= $ml->br;
	my $url = $this->link(zadm=>"new$type");
	$out .= &ExSite::HTML::Button(url=>$url,label=>"Create a new $type");
    }
    elsif (/new/) {
	/^new(.*)$/;
	my $type = $1;
	if ($type eq "raw") { $type = "article"; }
	$out .= $this->makeform({type=>$type});
    }
    elsif (/publish/) {
	$out .= $ml->h3("Publishing Images").
	    $ml->iframe(
			$ml->a("Click here to publish images.",
			       {href=>"$config{server}{CGIpath}/publish.cgi/Zine?module=Zine", target=>"publish"}),
			{ height=>300, width=>600, 
			  src=>"$config{server}{CGIpath}/publish.cgi/Zine?module=Zine" }
			);
    }
    elsif (/search/) {
	$out .= $share{DB}->make(table=>"article",search=>1);
    }
    else {
	if ($share{DB}->is_admin()) {

	    my $typeform = $ml->form("Show: ".
				     $ml->select(
						 $ml->option("== select ==",{value=>""}).
						 $ml->option("All zines",{value=>""}).
						 $ml->option("e-zine",{value=>"e-zine"}).
						 $ml->option("blog",{value=>"blog"}).
						 $ml->option("forum",{value=>"forum"}).
						 $ml->option("book",{value=>"book"}).
						 $ml->option("wiki",{value=>"wiki"}).
						 $ml->option("photoforum",{value=>"photoforum"}).
						 $ml->option("photoblog",{value=>"photoblog"}),
						 { name=>"type", onchange=>"document.forms[1].submit()" }
						 ).
				     $ml->input(undef,{type=>"hidden", name=>"section_id", value=>$sid})
				     );
				     
	    $out .= $ml->table(
			       $ml->tr(
				       $ml->td($this->set_section_id()).
				       $ml->td($typeform)
				       )
			       );
	    $out .= $ml->br;
	}
        $out .= $this->topview_admin;
	$out .= "<br>\n";
	my $url = $this->link(zine=>"newzine");
	$out .= &ExSite::HTML::Button(url=>$url,label=>"Create a new Zine");
    }

    return $out;
}

# load_article : fetch the base article
#
# The parameter $id is the default article to load, but may be over ridden 
# by the aid parameter or the path info.

sub load_article {
    my ($this,$id) = @_;
    my $in = $this->{input};
    if ($id) { 
	$this->load($id); 
    }
    elsif ($in->{path} && ! $this->{admin}) { 
	# we don't use path method of referencing articles in admin mode
	$this->find($in->{path}); 
    }

# moved to show()
#    # change our metadata to reflect the current article
#    if ($this->{config}{use_metadata} && 
#	ref $this->{article} eq "HASH" &&
#	$share{Page}) {
#	$share{Page}->set_metadata("title",$this->{article}{title});
#    }
    return;
}

# form : all posts go through here

sub form {
    my ($this,$cmd) = @_;
    my $out;
    my $ml = &get_obj("ML");
    
    my $art = $this->{article} || $this->{input}{aid};
    my $f = new Modules::Zine::Form($art,{admin=>$this->{admin}});
    
    $this->zine_handler();
    
    if ($cmd eq "submit") {
	my @err = $f->doform();
	if (@err) {
	    $out = $ml->h3($msg{"Problem:"}).
		$this->error(@err).
		$ml->p("[".
		       $ml->a($msg{"Back"},{href=>"javascript:history.back()"}).
		       "]");
	    return $out;
	}
	if ($f->{moderation_requested}) {
	    $out .= $ml->p($msg{"Your post has been received, but will not appear until it has been approved by the moderator."},
			   {class=>"ZineWarning"});
	    $this->load($art);
	    $out .= $this->show;
	    return $out;
	}
	return undef;
    }
    elsif ($cmd eq "new") {
	$out = $f->newmsg;
    }
    elsif ($cmd eq "reply") {
	$out = $f->reply;
    }
    elsif ($cmd eq "post") {
	$out = $f->post;
    }
    elsif ($cmd eq "newarticle") {
	$this->load($art);
	$out = $f->makeform({type=>"article"},$this->{article});
    }
    elsif ($cmd eq "edit") {
	$this->load($art);
	$out = $f->makeform($this->{article});
    }
    elsif ($cmd eq "conf") {
	$this->load($art);
	$out = $f->confform();
    }
    elsif ($cmd eq "newzine") {
	$out = $f->newzine();
    }
    elsif ($cmd =~ /approve|reject/) {
	my $stat = $f->status($cmd);
	if ($stat) {
	    $out .= $ml->p($stat . " " .
			   &substitute($msg{"This action can be un-done using the [[conf]] function, by selecting a different status."},{conf=>$this->{config}{tool}{configure}}),
			   {class=>"ZineWarning"});
	}
	$out = $this->show(archive=>$this->{input}{archive});
    }
    return $out;
}

# zine handler
sub zine_handler {
	my $this = shift;
	my $db = $share{DB};
	$db->handler("select_foreign_key",\&handler_select_foreign_key);
	return;
}

# select foreign key handler
sub handler_select_foreign_key{
	my $this = shift;
	my $table = shift;
	my $rec = shift;
	my $col = shift;
	my $ftable = shift;
	my $ok = $this->authorize();
	
	if ($ok > 1 && $col eq "owner"){
		return $this->custom_query("SELECT * FROM member WHERE access > 1");
	}
	
	return undef;
}

#--------------------------------------------------------------------------

# high-level indexes

# index of available zines (site-specific)

sub topview {
    my ($this,$opt) = @_;
    my $stat = $this->run_handler("ZineTopView",$opt);
    return $stat if ($stat);
    my $ml = &get_obj("ML");
    my %match = (status=>"active", section_id=>0, reference=>0);
    my (@art,$type);
    if ($opt && $opt !~ /=/) { 
	# request to view a particular class of top-level articles
	$type = $opt; 
    }

    # get all zines for this site
    @art = $this->our_zines();

    my $out;
    if (@art == 1) {
	$this->save();
	$this->set($art[0]);
	$out = $this->show();
	$this->restore();
    }
    elsif (@art > 0) {
	foreach my $art (@art) {
	    next if ($type && $art->{type} ne $type);
	    $this->set($art);
	    my $url = $this->link(zine=>"show",aid=>$art->{article_id});
	    my $title = $this->title;
	    my $author = $this->author;
	    my $summary = $this->show_data("summary");
	    my $dt = 
		$ml->a($title,{href=>$url}) . 
		"&nbsp;&nbsp;&nbsp;";
	    if ($author && $art->{type} !~ /forum|wiki/) {
		$dt .= "by $author";
	    }
	    $out .= $ml->dt($dt,{class=>"ZineItem"});
	    if ($summary) {
		$out .= $ml->dd($ml->em($summary));
	    }
	}
	$out .= $ml->dl($out,{class=>"ZineIndex"});
    }
    return $out;
}

sub topview_admin {
    my ($this,$opt) = @_;
    if (! $share{DB}->is_manager) {
	return $this->error("Permission denied.");
    }

    return $this->zinedex($this->{input}{type},$this->{input}{section_id});

    ### rest of this routine is deprecated

    my (@art,$match);
    if ($share{DB}->is_admin) {
	$match = {reference=>0};
    }
    else {
	$match = {reference=>0,owner=>$share{DB}->my_uid()};
    }
    if ($opt) { $match->{type} = $opt; }
    @art = $share{DB}->fetch_match("article",$match);

    my $out;
    my @adata;
    if (@art > 0) {
	foreach my $art (@art) {
	    $this->set($art);
	    my $url = $this->link(zine=>"show",zadm=>undef,aid=>$art->{article_id});
	    my $title = "<a href=\"$url\">".$this->title,"</a>";
	    my $summary = $this->show_data("summary");
	    my $site;
	    if ($art->{section_id}) {
		my $s = $share{DB}->fetch("section",$art->{section_id});
		$site = $s->{title};
	    }
	    else {
		$site = "(shared)";
	    }
	    my $type = $this->type;
	    my $tools = $this->tools($art);
	    push @adata, [$title,$type,$site,$tools];
	}
    }
    my @head = ("Zine","Type","Section","Options");
    my $zrpt = new ExSite::ReportBuilder(
					 title=>"Zines",
					 headers=>\@head,
					 data=>\@adata,
					 );
    return $zrpt->make;
}

# zinedex - tabular index of zines, with activity summaries

sub zinedex {
    my ($this,$type,$section_id,$zine_id) = @_;
    my $stat = $this->run_handler("ZineDex",$type);
    return $stat if ($stat);

    my $ml = &get_obj("ML");
    my $db = $share{DB};
    my ($sid,$see_general_zines);
    $this->{zinedex_count} = 0;
    if ($db->is_admin()) {
	$sid = $section_id || ($share{Page} ? $share{Page}->id("section") : 0); 
	$see_general_zines = 1;
    }
    else {
	$sid = $share{Page} ? 
	    $share{Page}->id("section") : 
	    $db->my_user_record()->{section_id};
	$see_general_zines = 0;
    }
    my @allzine = $this->all_zines();
    my $tree = new ExSite::Tree("article_id","parent_id",@allzine);

    # walk our tree, and disable zines that we should not be seeing

    my @z = $zine_id ? 
	$tree->get_child_data($zine_id) : 
	$tree->get_topnodes_data();
    while (my $z = shift @z) {
	if ($sid && $z->{section_id} && $z->{section_id} != $sid) {
	    # not ours; prune this branch
	    $tree->delnode($z->{article_id});
	    next;
	}
	elsif (! $z->{section_id} && ! $see_general_zines) {
	    # we cannot view global zines
	    $tree->delnode($z->{article_id});
	    next;
	}
	elsif ($type && ! $tree->count({type=>$type},$z->{article_id})) {
	    # none of the desired type in this branch
	    $tree->delnode($z->{article_id});
	    next;
	}
	my @child = $tree->get_child_data($z->{article_id});
	foreach my $c (@child) {
	    if ($type && ! $tree->count({type=>$type},$c->{article_id})) {
		# none of the desired type in this sub-branch
		$tree->delnode($c->{article_id});
		next;
	    }
	}
    }
    
    # display the zines in a tabular format

    my $title;
    if ($type) {
	$title = $msg{$this->confrule("whatami",undef,$type)."s"};
    }
    else {
	$title = $msg{"Zines"};
    }
    my $atype = (! $type || $this->confrule("allow","article",$type)) ?
	$msg{"Articles"} : $msg{"Threads"};

    my $out = 
	$ml->caption($title).
	$ml->tr(
		$ml->th($msg{$this->{config}{article_type}{$type}}).
		$ml->th($msg{$atype}).
		($this->{config}{hide_comments_column}{$type} ? "" : $ml->th($msg{"Comments"})).
		$ml->th($msg{"Most Recent Post"})
		);

    my @zine = $tree->get_topnodes_data();
    foreach my $z (@zine) {
	$out .= $this->zinedex_r($type,$z,$tree);
    }
    return $ml->table($out,{ class=> "Report" } );
}

sub zinedex_r {
    my ($this,$type,$article,$tree) = @_;
    my ($out,$db,$article_count,$comment_count);
    $db = $share{DB};
    $this->set($article);
    my $ml = &get_obj("ML");

    # count methods create their own trees as needed
    $comment_count = $this->count($article->{article_id},"comment");
    if (! $type || $this->confrule("allow","article",$type)) {
	$article_count = $this->count($article->{article_id},"article");
    }
    else {
	# should be no articles; count threads instead
	my $tree = $this->fetch_tree($this->{article_id});
	my $match = {type=>"comment",thread=>0};
	if (! $this->admin_mode) { $match->{status} = "active"; }
	$article_count = $tree->count($match,$this->{article_id});
    }

    if ($type && $article->{type} ne $type) {
	# wrong type; assume it is an oganizational category
	$out .= $ml->tr($ml->th($article->{title},{ class=>"ReportTitle", colspan=>4 }));
    }
    else {
	my $time;
	my @articles = $this->fetch_latest("article",$this->{article_id},1);
	my @comments = $this->fetch_latest("comment",$this->{article_id},1);

	if (@articles > 0) { $time = $articles[-1]{mtime}; }
	if (@comments > 0 && $comments[-1]{mtime} > $time) { $time = $comments[-1]{mtime}; }
	my $date = "n/a";
	if ($time) {
	    my $t = new ExSite::Time($time,"sql_timestamp");
	    $date = $t->write("datetime");
	}
	my $title;
	if ($article->{section_id} && $this->{admin}) {
	    my $s = $db->fetch("section",$article->{section_id});
	    $title = $s->{title}.": ";
	}

	# the zinedex link has a few features for styling "new" links
	# The "ZineNew" class is added if is_new() indicates that the zine
	# holds new content.  This is customizable with a handler.
	# We also add dummy ac and cc parameters to hold the article count
	# and comment count; these will cause links to the zine to appear
	# unvisited if these counts change since the last time you were there.

	my $linkclass = $this->is_new ? "ZineNew" : undef;
	$title .= $this->show_link(
				   undef,
				   $linkclass,
				   ac=>$article_count,
				   cc=>$comment_count,
				   );

	if (! $type) { $title .= " ".$ml->em("(".ucfirst($article->{type}).")"); }
	my $description = $article->{summary} ? 
	    $ml->br.$ml->span($article->{summary},{class=>"ZineReportAsbstract"}) : "";
	$this->{zinedex_count} ++;
	
	my $comments_column = $ml->td($comment_count) unless ($this->{config}{hide_comments_column}{$type});
	
	my $class = ($this->{zinedex_count} % 2) ? "A" : "B";
	$out .= $ml->tr(
			$ml->td(
				$ml->span($title,{class=>"ZineReportTitle"}).
				$description
				).
			$ml->td($article_count).
			$comments_column.
			$ml->td($date),
			{ class=>$class }
			);
    }

    # recurse into child zines
    foreach my $z ($tree->get_child_data($article->{article_id})) {
	next if (! $z->{section_id} && ! $db->is_admin());  # global zine
	$out .= $this->zinedex_r($type,$z,$tree);
    }
    return $out;
}

# index_latest: summarize recently added articles

sub index_latest {
    my ($this,$type,$id,$number) = @_;
    return undef if ($type !~ /^(article|comment)$/);
    my $out;
    my $ml = &get_obj("ML");
    my @latest = $this->fetch_latest($type,$id,$number);
    if (@latest > 0) {
	# conf file option to turn off header
	if ($this->{config}{"new_${type}s_heading"}) {
	    $out = $ml->h1($msg{$this->{config}{"new_${type}s_heading"}});
	}
	foreach my $a (@latest) {
	    $this->set($a);
	    my $title = $this->title();
	    my $auth = $this->author();
	    my $author = $auth ? &substitute($msg{"by [[author]]"},{author=>$auth}) : "";
	    my $reftitle = $this->reftitle();
	    my $date = $this->date;
	    my $url = $this->link(zine=>"show",aid=>$this->{article}{article_id});
	    my $rurl = $this->link(zine=>"show",aid=>$this->{reference}{article_id});
	    $out .= $ml->p(
			   $ml->a($title,{href=>$url, class=>"ZineTitle"}).
			   " $author $date ".
			   &substitute("in [[zine]]",{zine=>$ml->a($reftitle,{href=>$rurl, class=>"ZineRef"})}),
			   { class=>"ZineTeaser" }
			   );
	}
	$out = $ml->div($out,{class=>"Zine"});
    }
    return $out;
}

sub index_latest_articles {
    my ($this,$id,$number) = @_;
    # return a dummy string in case there are no articles
    return $this->index_latest("article",$id,$number) || " ";
}

sub index_latest_comments {
    my ($this,$id,$number) = @_;
    # return a dummy string in case there are no comments
    return $this->index_latest("comment",$id,$number) || " ";
}

# index_user_posts: show postings made by a particular user

sub index_user_history {
    my ($this,$uid,$zine_id,$limit) = @_;
    my $out;
    my $ml = &get_obj("ML");
    defined $limit or $limit = $this->{config}{history_limit};
    $uid or $uid = $share{DB}->my_uid;
    my @art = $share{DB}->custom_query("select a1.*,(select count(*) from article a2 where a2.type='comment' and a2.status='active' and a2.parent_id=a1.article_id) replies from article a1 where owner=? and status='active' order by article_id desc",$uid);
    my $count = 0;
    foreach my $a (@art) {
	$this->set($a);
	next if ($this->type !~ /article|comment/);
	my $title = $this->title;
	my $date = $this->date;
	my $url = $this->link(zine=>"show",aid=>$this->{article}{article_id});
	my $reply = $a->{replies} == 1 ? $msg{"reply"} : $msg{"replies"}; 
	$out .= $ml->p(
		       $ml->a($title,{href=>$url, class=>"ZineTitle"}).
		       " $date (".$a->{replies}." ".$msg{$reply}.")",
		       { class=>"ZineTeaser" }
		       );
	$count++;
	last if ($limit && $count == $limit);
    }
    if ($out) {
	$out = $ml->h1(&substitute($msg{"Recent Postings from [[user]]"},{user=>$share{DB}->user_name($uid)})).$out;
    }
    return $out;
}

# feature article is a trivial "index" of one fixed article

sub feature {
    my ($this, $article_id) = shift;
    $this->fetch($article_id);
    return $this->show_item($this->{config}{feature_format},1);
}

#--------------------------------------------------------------------------
# TODO - reminder of tasks

sub todo {
    my $this = shift;
    my @todo = $share{DB}->custom_query("select a.*,b.article_id as aid2,b.type as type2,c.article_id as aid3,c.type as type3 from article as a, article as b left join article as c on b.reference=c.article_id where a.type='comment' and a.status='pending' and a.reference=b.article_id");
    my @task;
    foreach my $job (@todo) {
	$this->set($job);
	my $zid = $job->{aid3} || $job->{aid2};
	my $cid = $job->{article_id};
	if ($this->owns($zid)) {
	    my $link = "$config{server}{server}$config{server}{CGIpath}/$config{prog}{ctrlpanel}/Zine?zine=show&amp;aid=$cid";
	    my $abstract = $this->abstract();
	    push @task, {link=>$link,
			 title=>"Moderation required: ".$this->title(),
			 description=>$this->date()."<br>\n".
			     "From: ".$this->author()."<br>\n".
			     $abstract,
		     };
	}
    }
    return @task;
}

#--------------------------------------------------------------------------
# CRON - scheduled tasks
#
# allowed commands
#
# daily|weekly digest     article|section
# approve                 article
# archive                 article
# publish                 zine|article|rss

sub cron {
    my ($this,$command,$type,$id) = @_;
    my $out;
    if ($command =~ /digest/) {

	my %opt;
	if ($type eq "section") {
	    $opt{section} = $id;
	}
	my $digest_type = $command;
	$digest_type =~ s/ digest//;
	$opt{type} = $digest_type;

	my $aid = $type eq "article" ? $id : 0;

	my $d = new Modules::Zine::Digest($aid,\%opt);
	my $nsent = $d->send_digest();
	$out = "$nsent $command messages sent";
    }
    elsif ($command =~ /^(approve|archive)$/) {
	if ($type eq "article" && $id =~ /^\d+$/) {
	    my $art = $this->load($id);
	    if ($art) {
		my $status = $command eq "archive" ? "archived" : "active";
		$art->status($status);
	    }
	    else {
		$out = "No such article ($id)";
	    }
	}
	else {
	    $out = "Cannot $command $type $id";
	}
    }
    elsif ($command =~ /^publish$/) {
	$this->load($id) if ($id);
	if ($type eq "rss") {
	    $this->publish_rss($this->{article});
	    return "Published RSS for zine $id";
	}
	elsif ($type eq "article") {
	    $this->publish_article($this->{article});
	    return "Published article $id";
	}
	elsif ($type eq "zine") {
	    if ($id) {
		$this->publish_zine($this->{article});
		return "Published zine $id";
	    }
	    else {
		$this->publish();
		return "Published zines";
	    }
	}
    }
    else {
	$out = "Unknown command: $command";
    }
    return $out;
}

1;
