#!/usr/bin/perl

package Modules::Zine::Article;

#==========================================================================
# This is the viewer component for individual articles.
# Aggregate article views are handled by the Index class.
#==========================================================================

use strict;
use ExSite::Config;
use ExSite::ReportBuilder;
use ExSite::Misc;
use ExSite::Util;
use ExSite::HTML;
use ExSite::Image;
use ExSite::Time;
use ExSite::RSS;
use ExSite::Tree;
use Modules::Zine::Base;

use vars qw(@ISA);
@ISA = qw(Modules::Zine::Base);

### markup

sub css {
    my $this = shift;
    my $ml = &get_obj("ML");
    return $ml->link(undef,{rel=>"stylesheet", type=>"text/css", href=>"$config{server}{HTMLpath}/_Modules/Zine/zine.css"});
}

sub wrap {
    my ($this,$class,$contents,$tag,$id) = @_;
    return undef if (!$contents);
    my $ml = &get_obj("ML");
    $tag or $tag = "div";
    my $c = $this->{debug} ?
	$ml->div($class,{class=>"ZineDebug"}) . $contents : 
	$contents;
    my $attr = $id ? {class=>$class, id=>$id} : {class=>$class};
    return $ml->Element($tag,$c,$attr);
}

### show parts of articles

sub header {
    my $this = shift;
    my $stat = $this->run_handler("ZineHeader");
    return $stat if ($stat);
    my $ml = &get_obj("ML");
    my $icon = $this->moderator ? $this->status_icon() : undef;
    my $title = $icon.$this->clean_text($this->title);
    my $class = $this->class;
    my $n = $config{$this->module_name}{heading_level}{$class};
    if ($n =~ /^[123456]$/) {
	my $tag = "h$n";
	$title = $ml->$tag($title);
    }
    my $subtitle = $this->clean_text($this->show_data("subtitle"));
    if ($subtitle) { $subtitle = $this->wrap("ZineSubtitle",$ml->em($subtitle)); }
    my $bookmark = $ml->a(undef,{name=>'Article$this->{article_id}'});
    return $this->wrap("ZineHeader",$bookmark.$this->wrap("ZineTitle",$title).$subtitle);
}

sub about {
    my ($this,$datetype) = @_;
    my $stat = $this->run_handler("ZineAbout");
    return $stat if ($stat);

    my ($show_auth, $show_date, $about, $author, $date, $summary);
    $show_auth = $this->rule("showauthor");
    $show_date = $this->rule("showdate");
    
    if ($show_auth) {
	$author = $this->author;
	if ($author) { 
	    if ($this->{config}{history}      # history enabled
		&& $this->type eq "comment"   # on comments only
		&& $this->{article}{owner}    # comment is tied to a user
		&& $this->{article}{author})  # user didn't anonymize it
	    {
		my $hurl = $this->link(aid=>undef,zine=>"history",uid=>$this->{article}{owner});
		my $ml = &get_obj("ML");
		$author = $ml->a($author,{href=>$hurl});
	    }
	    $about = $this->wrap("ZineAuthor",&substitute($msg{"by [[author]]"},{author=>$author}),"span"); 
	}
    }
#    if ($this->class =~ /article|comment/) {
    if ($show_date) {
	$date = $this->date($show_date);
	$about .= ($author ? ", " : "").$this->wrap("ZineDate",$date,"span");
    }
    # this line escapes all HTML codes:
    #$summary = $this->show_data("summary");
    # this line permits HTML:
    $summary = $this->clean_text($this->{article}{summary},1);
    if ($summary && $this->{config}{abstract_full}) {
	$about .= $this->wrap("ZineSummary",$summary);
    }

    return $about;
}

sub footer {
    my $this = shift;
    my $stat = $this->run_handler("ZineFooter");
    return $stat if ($stat);
    my $ml = &get_obj("ML");
    my ($footnotes, $tags, $tools);
    $footnotes = $this->clean_text($this->{article}{about},1); # allow html
    if ($this->{config}{tags}) {
	# call the Tags module for our keyword associations
	$tags = "<!--&Tags(tags=article&id=$this->{article_id})-->\n";
    }
    $tools = $this->tools;
    # disable conditional wrapping - we need the wrapper in some cases,
    # even if there is no content.
    #return $this->wrap("ZineFooter",$footnotes.$tools);
    return $ml->div($footnotes.$tags.$tools,{class=>"ZineFooter"});
}

# should we show photos for some article types??
sub abstract {
    my $this = shift;
    my $stat = $this->run_handler("ZineAbstract");
    return $stat if ($stat);

    if ($this->{article}{summary}) {
	# this escapes HTML codes:
	#return $this->show_data("summary");
	# this allows HTML codes:
	return $this->{article}{summary};
    }
    my $out = $this->body;
    # remove HTML 
    $out =~ s/<.+?>//g;
    my $teaserlen = $config{$this->module_name}{teaser_length} || 200;
    if (length($out) > $teaserlen) {
	my $pos = $teaserlen; 
	my $char;
	while (($pos < length($out)) && ($char = substr($out,$pos,1))) {
	    last if ($char =~ /\s/);
	    $pos++;
	}
	substr($out,$pos) = "...";
    }
    return $out;
}

sub body {
    my $this = shift;
    my $stat = $this->run_handler("ZineBody");
    return $stat if ($stat);
    my $body = $this->get("body");
    my $ml = &get_obj("ML");

    # convert to legal HTML
    $body = $this->preprocess_html($body);

    if ($this->class eq "article") {

	# cross-link to other articles

	$body =~ s/\(\((.+?)(\|(.+?))?\)\)/$this->link_to_name($1,$3)/egs;
    }
    elsif ($this->class eq "comment") {

	# try to impose some style rules on the quoted sections, in case
	# the designer wants to highlight them

	my @text = split /\n/,$body;
	$body = undef;
	my $quote = 0;
	my $quoted;
	foreach my $line (@text) {
	    if ($line =~ /^(>|&gt;)/) {
		if ($this->{config}{remove_quote_char}) {
		    $line =~ s/^(>|&gt;)//;
		}
		$quoted .= $line;
	    }
	    else {
		if ($quoted) {
		    $body .= $ml->div($quoted,{class=>"ZineQuote"});
		    $quoted = undef;
		}
		$body .= $line;
	    }
	}
	#$body = join("\n",@text)."\n";

	# emoticons

	if ($this->{config}{emoticons}) {
	    foreach my $key (@{$this->{config}{emoticon_re}}) {
		$body =~ s/$key/<img src="$config{server}{HTMLpath}\/_Modules\/Zine\/$this->{config}{emoticon_img}{$key}" alt="emoticon">/g;
	    }
	}

	if ($this->{config}{body_strut_size} || $this->{admin}) {
	    # This is a kludge for pseudo-table layouts, where the body
	    # column may end up being shorter than the about column.  You
	    # can add a body strut to force a minimum body height.  
	    # Ironically, in order to make the wrapper DIVs look like they
	    # are really a table, we have to use a real table.  
	    # Gotta love CSS...
	    my $strut_height = $this->{config}{body_strut_size} || 75;
	    $body = $ml->table(
			       $ml->tr(
				       $ml->td($body,{valign=>"top"}).
				       $ml->td(
					       $ml->img(undef,{src=>"$config{server}{HTMLpath}/_ExSite/images/null.gif", width=>1, height=>$strut_height, align=>"right", border=>0, alt=>""})
					       )
				       )
			       );
	}

    }

    # attachments?

    return $body;
}

# preprocess a piece of text to make it HTML-friendly
# Article elements are often entered as plain text or partial-html, so
# we need to prep it for insertion into a full-html document.

sub preprocess_html {
    my ($this,$text) = @_;
    my $ml = &get_obj("ML");

    # kill/neuter any hidden CMS tags
    if (! $this->{config}{allow_cms_tags}) {
	$text =~ s/<!--/<!-- /g;
    }
    if (! $this->{config}{allow_cms_links}) {
	$text =~ s/\{\{(?![^<>]*>)/\{&\#00;\{/g;
	$text =~ s/\}\}(?![^<>]*>)/\}&\#00;\}/g;
	$text =~ s/\[\[(?![^<>]*>)/\[&\#00;\[/g;
	$text =~ s/\]\](?![^<>]*>)/\]&\#00;\]/g;
    }

    # do we trust any embedded html?
    if (! $this->is_trusted()) {
	$text = &safehtml($text);
    }

    # process linebreaks
    my $br = $ml->br;
    if ($this->class eq "comment") {
	$text =~ s/\r?\n/$br/g;       # convert NL to <br>
    }
    elsif ($this->{article}{content_type} !~ /html/i) {
	# insert paragraph tags at any double-linebreaks
	### not correct XHTML
	$text =~ s/\r\n\r\n/<p>\n/g;  # handles CR-LF linebreaks
	$text =~ s/\n\n/<p>\n/g;      # handles LF linebreaks
    }
    return $text;
}

sub size {
    my $this = shift;
    return length $this->{article}{body};
}

######################### show complete articles

# display a complete article and its associated indexes

sub show {
    my ($this,%opt) = @_;
    if (! $this->approve) {
	my $type = $this->type;
	return $this->privacy =~ /public/ ?
	    $this->error(&substitute($msg{"Sorry, this [[type]] is not available for viewing."},{type=>$type})) :
	    $this->error(&substitute($msg{"Sorry, this [[type]] has restricted access.  Please log in."},{type=>$type}));
    }
    my $stat = $this->run_handler("ZineShow");
    return $stat if ($stat);
    my $out = $this->show_path(%opt);
    $out .= $this->show_article(%opt);

    my $class = $this->class();
    my $refclass = $this->refclass();
    my ($zindex,$aindex,$cindex);

    # zine index

#    if ($class eq "zine") {
#	$opt{mode} = "zine";    # use a type, not a class
#	$aindex = $this->index(%opt);
#    }

    # article index

    if ((($class eq "zine") && $this->rule("allow","article")) ||
	(($class eq "article") && $this->rule("nest","article"))) {
	$opt{mode} = "article";
	$aindex = $this->index(%opt);
    }

    # comment index
    
    if (($class eq "article") || 
	($class eq "zine" && $this->rule("allow","comment")) ||
	($class eq "comment" && $this->rule("nest","comment"))) {
	$opt{mode} = "comment";
	$cindex = $this->index(%opt);
    }

    my $ml = &get_obj("ML");
    if ($aindex) {
	if ($cindex) { 
	    my $type = ucfirst $this->rule("whatami","article");
	    $out .= $ml->h3($msg{"${type}s"});
	}
	$out .= $aindex;
    }
    if ($cindex) {
	if ($aindex || $class ne "comment") { 
	    my $type = ucfirst $this->rule("whatami","comment");
	    $out .= $ml->h3($msg{"${type}s"});
	}
	$out .= $cindex;
    }
    return $out;
}

# display a complete article

sub show_article {
    my $this = shift;
    if (! $this->approve) {
	my $type = $this->type;
	return $this->error(&substitute($msg{"Sorry, [[article]] is not available for viewing."},{article=>"$type $this->{article_id}"}));
    }
    my $stat = $this->run_handler("ZineShowArticle");
    return $stat if ($stat);
    # build the article structure up from component elements
    #
    ### should we conditionally float the picture, depending on its size?
    ### Or, depending on whether there is any body to float around?
    #
    my $picture = $this->picture();
    my $attachment = $this->wrap("ZineAttachment",$this->attachment);
    my $body = $this->wrap("ZineBody",$picture.$this->body.$attachment);
    my $about = $this->wrap("ZineAbout",$this->about);
    my $about_body = $this->wrap("ZineAboutBody",$about.$body);
    my $head = $this->header;
    my $foot = $this->footer;  # includes tools
    if ($this->{article}{type} eq "comment") {
	return $this->wrap("ZineComment",$head.$about_body.$foot)."\n";
    }
    else {
	return $this->wrap("ZineArticle",$head.$about_body.$foot)."\n";
    }
}

# show_item - show an article as an item in an index
# shows summary indexes under some items

sub show_item {
    my ($this,$format,$subindex) = @_;
    # local article override of index format
    my $aformat = $this->{article}{indexformat} || $format;
    my $out;
    if ($aformat eq "title") {
	$out = $this->show_link;
    }
    elsif ($aformat eq "summary") {
	$out = $this->show_teaser(defined $subindex ? $subindex : 1);
    }
    elsif ($aformat eq "full") {
	$out = $this->show_full(defined $subindex ? $subindex : 1);
    }
    else { 
	$out = $this->show_info;
    }
    return $this->wrap("ZineItem",$out,undef,"Article$this->{article_id}");
}

sub show_teaser {
    my $this = shift;
    my $subindex = shift;
    my $stat = $this->run_handler("ZineShowTeaser");
    return $stat if ($stat);
    my $ml = &get_obj("ML");
    my $out = $this->show_link . " ";
    my $author = $this->author;
    if ($author) { 
	my $show_auth = $this->rule("showauthor");
	if ($show_auth) { $out .= "by $author" . $ml->br; }
    }
    if ($this->rule("showdate")) {
	$out .= $this->wrap("ZineDate",$this->date(),"span");
    }
    $out .= $this->wrap("ZineAbstract",$this->abstract);
    if ($subindex) { $out .= $this->summary_index(); }
    return $out;
}

sub show_link {
    my ($this,$anchor,$class,%opt) = @_;
    my $ml = &get_obj("ML");
    my $url = $this->link(zine=>"show",aid=>$this->{article_id},type=>undef,%opt);
    my $label = $anchor ? $anchor : $this->title;
    my $aclass = $class ? "ZineLink $class" : "ZineLink";
    return $this->status_icon() . $ml->a($label,{href=>$url, class=>$aclass});
}

sub show_info {
    my $this = shift;
    my $stat = $this->run_handler("ZineShowInfo");
    return $stat if ($stat);
    my $link = $this->show_link;
    my $author = $this->author;
    my $date = $this->date;
    return $author ? 
	"$link by $author;  $date" :
	"$link &mdash; $date";
}

sub show_row {
    my ($this,$data,@col) = @_;
    my $stat = $this->run_handler("ZineShowRow",$data,@col);
    return $stat if ($stat);
    if (! defined $data) { $data = 1; }
    if (@col == 0) { @col = qw(title author ctime); }
    my ($celltype,@colout);
    if ($data) {
	foreach my $c (@col) {
	    push @colout, $c eq "title" ? 
		$this->show_link() :
		$this->show_data($c);
	}
	$celltype = "td";
    }
    else {
	foreach my $c (@col) {
	    my %cmap = $share{DB}->{map}->get_column("article",$c);
	    push @colout, $this->label(\%cmap);
	}
	$celltype = "th";
    }
    my $ml = &get_obj("ML");
    my @out = map { $ml->Element($celltype,$_) } @colout;
    return join("",@out);
}

# show an item in full, with a summary index

sub show_full {
    my ($this,$subindex) = @_;
    my $stat = $this->run_handler("ZineShowFull");
    return $stat if ($stat);
    if (! $this->approve) {
	my $type = $this->type;
	return $this->error("Sorry, you don't have permission to view this $type.");
    }
    my $ml = &get_obj("ML");
    my $out = $this->show_article;
    if ($subindex) { 
	my $id = $this->{article_id};
	if ($this->count($id,"article") + $this->count($id,"comment")) {
	    # only show summary index if it contains something
	    $out .= $this->summary_index(); 
	}
    }
    return $out;
}

# a summary index is a brief indicator of sub-articles and comments,
# with a link to view them.

sub summary_index {
    my $this = shift;
    my $out;
    my $ml = &get_obj("ML");
    my $article_count = $this->count($this->{article_id},"article");
    my $comment_count = $this->count($this->{article_id},"comment");
    my $url = $this->link(zine=>"show",aid=>$this->{article_id});
    if ($article_count || $comment_count) {
	if ($article_count) {
	    $out .= "$article_count more articles; ";
	}
	if ($comment_count) {
	    my $comment_type;
	    if ($comment_count > 1) { 
		$comment_type = ($this->class() eq "article") ? 
		    "comments" : "replies";
	    }
	    else {
		$comment_type = ($this->class() eq "article") ? 
		    "comment" : "reply";
	    }
	    $out .= "$comment_count $comment_type; ";
	}
    }
    $out .= $ml->a($msg{"Read more..."},{href=>$url});
    $out = $this->wrap("ZineSummaryIndex","($out)");
    return $out;
}

sub status_icon {
    my $this = shift;
    # return undef if ($this->class eq "zine");
    return undef if (! $this->admin_mode);
    return undef if (! $this->moderator);
    my $ml = &get_obj("ML");
    return $ml->img(undef,{ src=>"$config{server}{HTMLpath}/_Modules/Zine/$this->{config}{status_icon}{$this->{article}{status}}", height=>16, width=>16, alt=>"($this->{article}{status})", title=>"($this->{article}{status})", class=>"ZineStatusIcon" });
}

##################### show images and attachments

sub attachment {
    my ($this,@att) = @_;
    my $stat = $this->run_handler("ZineAttachment",@att);
    return $stat if ($stat);
    my $ml = &get_obj("ML");
    if (@att == 0) {
	@att = $share{DB}->fetch_match("article",{
	    parent_id=>$this->get("article_id"),
	    type=>"attachment",
	    status=>"active",
	});
	return "" if (@att == 0);
    }
    my @attout;
    foreach my $attachment (@att) {
	my $img = new ExSite::Image($attachment->{picture});
	my $name = $img->name;
	my $type = &MimeType($name);
	my $att;
	if ($type =~ /^image/) {
	    my $url = $this->link(zine=>"show",aid=>$attachment->{article_id});
	    $att = $ml->a($this->thumb($attachment),{href=>$url});
	}
	else {
	    $att = $share{DB}->show_data_noauth("article","picture",$attachment->{picture},$attachment->{article_id});
	}
	push @attout, $ml->table(
				 $ml->tr(
					 $ml->td($att)
					 ).
				 $ml->tr(
					 $ml->td(
						 $ml->div($attachment->{caption},{class=>"ZineCaption"})
						 )
					 )
				 );
    }
    return join("",@attout);
}

sub picture {
    my ($this,$img) = @_;
    my $stat = $this->run_handler("ZinePicture",$img);
    return $stat if ($stat);
    my $out;
    my $ml = &get_obj("ML");
    if (! $img) {
	return undef if (! $this->{article}{picture});
	$img = new ExSite::Image($this->{article}{picture});
    }
    my ($width,$height) = $img->dim;
    my $filename = "$this->{article_id}_".&clean_filename($img->{filename});
    if (! -e $this->diskpath."/".$filename) {
	# not published; go to the database
	$filename = "$config{server}{CGIpath}/getdata.cgi?tab=article&col=picture&id=$this->{article}{article_id}";
    }
    else {
	$filename = $this->htmlpath."/".$filename;
    }
    my $pictag;
    if (&MimeType($img->{filename}) =~ /^image/) {
	$pictag = $ml->img(undef,{src=>$filename, height=>$height, width=>$width, alt=>$img->{filename}});
    }
    elsif ($this->{config}{allow_documents}) {
	# not an image!  Perhaps it's a download.
	$pictag = $this->show_data("picture");
    }
    else {
	# invalid content
	$pictag = $this->error($msg{"Image processing error"});
    }

    my $table = $ml->tr(
			$ml->td($pictag)
			);
    my $cap = $this->show_data("caption");
    if ($cap) {
	$table .= $ml->tr(
			  $ml->td(
				  $ml->div($cap,{class=>"ZineCaption"})
				  )
			  );
    }
    $out .= $ml->table($table,{class=>"ZineImage",width=>$width});
    my $img_wrap_size = $config{$this->module_name}{img_max_wrap_size} || 400;
    my $body_wrap_size = $config{$this->module_name}{body_min_wrap_size} || 1000;
    if ($width < $img_wrap_size && $this->size > $body_wrap_size) {
	# we already have table.ZineImage to apply special styles to;
	# conditional wrapping with div.ZineImage allows us to conditionally
	# float the image depending on its size.

	### FIXME: could also float if the image has a small height

	return $this->wrap("ZineImage",$out);
    }
    return $out;
}

sub thumb {
    my ($this,$art) = @_;
    my $stat = $this->run_handler("ZineThumb");
    return $stat if ($stat);
    my $out;
    my $ml = &get_obj("ML");
    $art = $art || $this->{article};
    if ($art->{thumb}) {	
	my $img = new ExSite::Image($art->{thumb});
	my $filename = "$art->{article_id}T_".&clean_filename($img->{filename});
	if (! -e $this->diskpath."/".$filename) {
	    # not published; go to the database
	    $filename = "$config{server}{CGIpath}/getdata.cgi?tab=article&col=thumb&id=$art->{article_id}";
	}
	else {
	    $filename = $this->htmlpath."/".$filename;
	}
	$out = $ml->img(undef,{class=>"ZineThumb", src=>$filename, border=>0, alt=>"thumbnail"});
    }
    elsif ($art->{picture}) {
	my $img = new ExSite::Image($art->{picture});
	my $filename = "$art->{article_id}_".&clean_filename($img->{filename});
	if (! -e $this->diskpath."/".$filename) {
	    # not published; go to the database
	    $filename = "$config{server}{CGIpath}/getdata.cgi?tab=article&col=picture&id=$art->{article_id}";
	}
	else {
	    $filename = $this->htmlpath."/".$filename;
	}
	my $thumbdim = $config{$this->module_name}{thumbdim} || 100;
	$out .= $ml->img(undef,{class=>"ZineThumb", src=>$filename, width=>$thumbdim, border=>0, alt=>"thumbnail"});
    }
    return $out;
}

# tools -
# new article
# comment
# reply
# configure
# delete
# approve
# reject
# current/archives
# rss

sub tools {
    my ($this,$labels,$orientation,%conf) = @_;
    my ($out,$url,$ntool,$article,$aid);
    $article = $this->{article};
    $aid = $article->{article_id};

    if (scalar keys %conf == 0) {
	my $allow_comment = $this->allow_comments();
	my $allow_article = $this->allow_articles();

	%conf = exists $this->{config}{tools}{$aid} ? 
	    %{$this->{config}{tools}{$aid}} : ();

	# show default tools
	exists $conf{reply} or $conf{reply} = 
	    $article->{type} eq "comment" && 
	    $allow_comment;
	exists $conf{comment} or $conf{comment} = 
	    $this->class() =~ /zine|article/ && 
	    $allow_comment;
	exists $conf{link} or $conf{link} = 
	    # permalink in article indexes
	    $article->{type} =~ /article/ && $aid != $this->{input}{aid};
	exists $conf{new} or $conf{new} = 
	    $allow_article && 
	    ($this->{admin} || ($share{Page} && ! $share{Page}->publishing));
	exists $conf{edit} or $conf{edit} = 
	    $this->{admin} ||
	    # zine owners can edit their own articles from a page view
	    ( $article->{type} eq "article" && 
	      $share{Page} && ! $share{Page}->publishing &&
	      $article->{owner} == $share{DB}->my_uid ) ||
	    # commenters can optionally edit their own comments
	    ( $article->{type} eq "comment" && 
	      $share{Page} && ! $share{Page}->publishing &&
	      $allow_comment );
	exists $conf{configure} or $conf{configure} = 
	    $this->{admin};
	exists $conf{tags} or $conf{tags} = 
	    $this->{config}{tags} && 
	    $this->{admin} && 
	    $this->class =~ /zine|article/;
	exists $conf{delete} or $conf{delete} = 
	    $this->{admin};
	exists $conf{approve} or $conf{approve} = 
	    $article->{status} =~ /inactive|pending|rejected/ &&
	    $this->moderator;
	# $conf{rss} and $conf{archive} are only set manually
    }

    if (! defined $labels) { 
	$labels = $this->{config}{icon_labels} || "1";
    }
    if (! defined $orientation) { 
	$orientation = $this->{config}{tool_orientation} || "horizontal";
    }
    if (! exists $this->{tools}) { $this->{tools} = {}; }

    my (@toolicon,@toollabel,@toolurl);
    if ($conf{reply}) {
	push @toolurl, $this->link(zine=>"reply",aid=>$aid);
	push @toolicon, $this->tool_icon("reply","reply to this");
	push @toollabel, $this->tool_label("reply");
	$ntool++;
    }
    elsif ($conf{comment}) {
	my $lbl = $this->{config}{comment}{$article->{type}}{label} || "comment";
	push @toolurl, $this->link(zine=>"new",aid=>$aid);
	push @toolicon, $this->tool_icon("comment","new comment");
	push @toollabel, $this->tool_label($lbl);
	$ntool++;
    }

    if ($conf{link}) {
	### FIXME: use a regular link, even in AJAX mode
	push @toolurl, $this->link(zine=>"show",aid=>$aid);
	push @toolicon, $this->tool_icon("link","permalink");
	push @toollabel, $this->tool_label("link");
	$ntool++;
    }

    if ($conf{new}) {
	push @toolurl, $this->link(zine=>"newarticle",aid=>$article->{article_id});
	push @toolicon, $this->tool_icon("new","new article");
	push @toollabel, $this->tool_label("new");
	$ntool++;
    }

    if ($conf{edit}) {
	my $edit = 1;
	if (! $this->{admin} && $article->{type} eq "comment") {
	    # we can only edit our comments within a few minutes of posting
	    my $t = new ExSite::Time;
	    my $t1 = $t->write("unix_timestamp");
	    $t->set($article->{ctime},"sql_timestamp");
	    my $t2 = $t->write("unix_timestamp");
	    $edit = ($t1 - $t2 < $config{$this->module_name}{edit_my_comment_timeout});
	}
	if ($edit) {
	    push @toolurl, $this->link(zine=>"edit",aid=>$article->{article_id});
	    push @toolicon, $this->tool_icon("edit","edit");
	    push @toollabel, $this->tool_label("edit");
	    $ntool++;
	}
    }

    if ($conf{configure}) {
	push @toolurl, $this->link(zine=>"conf",aid=>$article->{article_id});
	push @toolicon, $this->tool_icon("tool","configure");
	push @toollabel, $this->tool_label("configure");
	$ntool++;
    }

    if ($conf{tags}) {
	my $url = "$config{server}{CGIpath}/$config{prog}{ctrlpanel}/Tags?tagcmd=addtags&article_id=$aid&_bare=1";
	push @toolurl, "javascript:popup_small('$url')";
	push @toolicon, $this->tool_icon("tag","tag");
	push @toollabel, $this->tool_label("tag");
	$ntool++;
    }

    if ($conf{delete}) {
	my $type = $this->type;
	push @toolurl, "javascript:confirm_custom('Delete $type $this->{article_id} and all of its related articles and comments?','".$this->link(zine=>"del",aid=>$article->{article_id})."')";
	push @toolicon, $this->tool_icon("delete","delete"); 
	push @toollabel, $this->tool_label("delete");
	$ntool++;
    }

    if ($conf{approve}) {
	push @toolurl, $this->link(zine=>"approve",aid=>$article->{article_id})."#Article$article->{article_id}";
	push @toolicon, $this->tool_icon("approve","approve");
	push @toollabel, $this->tool_label("approve");
	$ntool++;
	if ($article->{status} ne "rejected") {
	    push @toolurl, $this->link(zine=>"reject",aid=>$article->{article_id})."#Article$article->{article_id}";
	    push @toolicon,  $this->tool_icon("reject","reject");
	    push @toollabel, $this->tool_label("reject");
	    $ntool++;
	}
    }

    if ($conf{back}) {
	push @toolurl, $this->link(zine=>"show",aid=>$article->{reference});
	push @toolicon, $this->tool_icon("back","back");
	push @toollabel, $this->tool_label("back");
	$ntool++;
    }

    if ($conf{archive}) {
	if ($this->{input}{archive}) {
	    push @toolurl, $this->link(zine=>undef,aid=>$this->{article_id},archive=>undef);
	    push @toollabel, $conf{archive} ne "1" ? $conf{archive} : $this->tool_label("current");
	}
	else {
	    push @toolurl, $this->link(zine=>undef,aid=>$this->{article_id},archive=>1);
	    push @toollabel, $conf{archive} ne "1" ? $conf{archive} : $this->tool_label("archives");
	}
	push @toolicon, $this->tool_icon("archive","view archives");
	$ntool++;
    }

    if ($conf{rss} && $share{Page} && $this->rss_page()) {
	my $rssfile = $this->diskpath."/index.rss";
	my $rsslink;
	if (-e $rssfile) {
	    $rsslink = $this->htmlpath."/index.rss";
	}
	else {
	    my $uri = new ExSite::URI;
	    $uri->query(zine=>"rss",aid=>$this->{article_id});
	    $uri->service_page("Zine");
	    if ($article->{privacy} ne "public" && ! $share{Page}->is_publishing()) {
		$rsslink = $uri->authorize_url($share{DB}->my_login(),0);
	    }
	    else {
		$rsslink = $uri->write_full();
	    }
	}
	    
	push @toolurl, (-e $rssfile ? 
			$this->htmlpath."/index.rss" : 
			$rsslink);
	                #$this->link(zine=>"rss",aid=>$this->{article_id}));
	push @toolicon, $this->tool_icon("rss","RSS");
	push @toollabel, $this->tool_label("rss");
	$ntool++;
    }

    my $ml = &get_obj("ML");
    if ($ntool) {
	my @toolhtml;
	for (my $itool = 0; $itool < $ntool; $itool++) {
	    my $url = shift @toolurl;
	    my $icon = shift @toolicon;
	    my $label = shift @toollabel;
	    if (! $labels) {
		push @toolhtml, $ml->a($icon,{href=>$url});
	    }
	    elsif ($orientation eq "horizontal") {
		push @toolhtml, $ml->a($icon.$ml->br.$label,{href=>$url});
	    }
	    else {
		push @toolhtml, $ml->td($ml->a($icon,{href=>$url})).
		    $ml->td($ml->a($label,{href=>$url}));
	    }
	}
	my $table = ($orientation eq "horizontal") ?
	    $ml->tr(join ("", map { $ml->td($_) } @toolhtml)) :
	    join ("", map { $ml->tr($_) } @toolhtml);
	$out .= $ml->table($table,{class=>"ZineTools"});
    }

    return $out;
}

sub tool_icon {
    my ($this,$iconname,$alt) = @_;
    my $style = $config{$this->module_name}{icon_style} || "gif";
    my $filename = "$iconname.$style";
    my $ml = &get_obj("ML");
    return $ml->img(undef,{src=>"$config{server}{HTMLpath}/_Modules/Zine/$filename", height=>16, width=>16, border=>0, alt=>$alt, title=>$alt});
}

sub tool_label {
    my ($this,$label) = @_;
    if (! $this->{toollabels}) {
	$this->{toollabels} = $config{$this->module_name}{tool} || {};
    }
    my $uselabel = $this->{toollabels}{$label} || $label;
    return $msg{$uselabel};
}

##################### show indexes

sub index {
    my ($this,%opt) = @_;

    # try a handler first
    my $stat = $this->run_handler("ZineIndex",%opt);
    return $stat if ($stat);

    my $out;
    my $ml = &get_obj("ML");
    my ($nart,$narch);
    my $class = $this->class();
    my $type = $this->type();

    # save current article
    $this->save;

    # get index display parameters
    $opt{mode} = $opt{mode} || "article";
    $opt{sort} = $opt{sort} || $this->indexsort($opt{mode});
    $opt{format} = $opt{format} || $this->indexformat($opt{mode});
    $opt{limit} = $opt{limit} || $this->indexlimit($opt{mode});
    $opt{minimum} = $opt{minimum} || $this->rule("indexmin",$opt{mode});
    $opt{age} = $opt{age} || $this->indexage($opt{mode});
    if ($opt{mode} eq "comment" and $type ne "forum") {
	# do not index comments in archive mode
	$opt{archive} = 0;
    }

    # layout
    if ($opt{archive}) {
	$opt{layout} = "archive";
    }
    else {
	if (! $opt{layout}) { $opt{layout} = $this->indextype($opt{mode}); }
	if ($opt{layout} eq "basic") { $opt{layout} = ""; }
    }

    # run the indexing engine
    my $indexengine = ucfirst($opt{layout});
    $indexengine =~ s/[^\w]//g;
    my $indexmod = "Modules::Zine::Index$indexengine";
    #-- make safe for publishing
    $indexmod =~ /^(Modules::Zine::Index\w*)$/;
    my $safe_indexmod = $1;
    #--
    if ($safe_indexmod) {
	if (eval "require $safe_indexmod") {
	    # preserve admin state in the index object
	    my $index = $indexmod->new($this->{article},{admin=>$this->{admin}});
#	    my $index = $indexmod->new($this->{article});
	    if ($index) {
		$out = $index->index(%opt);
		$nart =  $index->article_count();
		$narch = $index->archive_count();
	    }
	    else {
		$out = $this->error("$indexmod: $@");
	    }
	}
	else {
	    $out = $this->error("$indexmod: $@");
	}
    }
    else {
	$out = $this->error("$indexmod: invalid index module");
    }
    $out = $this->wrap("ZineIndex",$out);

    # reset article
    $this->restore;

    # show public zine tools (archive/rss)
    my (%tools);
    if ($opt{archive}) {
	my $url = $this->link(archive=>undef);
	if ($out) {
	    $tools{archive} = $msg{"Current $opt{mode}s"};
	}
	elsif ($opt{mode} eq "article" && $class eq "zine") {
	    $tools{archive} = $msg{"Back to current articles"};
	}
    }
    elsif ($narch > 0) {
	if (! exists $this->{config}{tools}{$this->{article_id}}{archive} ||
	    $this->{config}{tools}{$this->{article_id}}{archive}) {
	    # archives have not been explicitly disabled for this zine
	    $tools{archive} = "Archives";
	}
    }
    if ($out && $class eq "zine") {
	# this provides RSS links for article indexes or comment index,
	# but not both.  If the zine allows articles, you get RSS for the
	# articles (eg. a blog).  If it does not, you get RSS for the comments
	# (eg. a forum).
	if (exists $this->{config}{tools}{$this->{article_id}}{rss}) {
	    # preconfigured for this zine
	    $tools{rss} = $this->{config}{tools}{$this->{article_id}}{rss};
	}
	else {
	    $tools{rss} = 
		$this->rule("allow","article") ? 
		($opt{mode} eq "article") : ($opt{mode} eq "comment");
	}
    }
    if ($this->{article}{reference} && $out) {
	$tools{back} = 1;
    }

    # In certain cases, it's worth noting that the index is empty.
    if (! $nart &&
	$opt{mode} eq "article" &&
	$class eq "zine" && $this->rule("allow","article") && 
	! $opt{archive}) { 
	# only show the "no articles" message if we're indexing a Zine
	# that allows articles.  ie. don't show it if we're attempting to 
	# show a sub-index under a simple article, or a forum (which only
	# accepts comments, not articles).
	my $article_name = $this->confrule("whatami","article",$this->zinetype);
	$out = $ml->p($msg{"No current ${article_name}s found."});
    }

    if (keys %tools > 0) {
	$out .= $this->tools(1,"vertical",%tools);
    }
    return $out;
}

# RSS feeds

sub rss {
    my ($this, @art) = @_;
    my $stat = $this->run_handler("ZineRSS");
    return $stat if ($stat);
    return undef if (! $this->is_zine);

    my ($title,$description,$currart,$url,@art);

    # only publish feeds if there is a page to service the links
    my $page = $this->rss_page();
    return undef if (! $page);

    if (@art == 0) {
	if ($this->privacy eq "public" || $share{DB}->is_member) {
	    my $limit = $this->{config}{rss_limit} || 10;
	    my $rsstype = $this->confrule("allow","article",$this->{article}{type}) ?
		"article" : "comment";
	    $title = $this->title;
	    $description = $this->abstract;
	    @art = $this->fetch_latest($rsstype,$this->{article_id},$limit);
	    $url = $this->link(__plaintext=>1,
			       zine=>"show",
			       aid=>$this->{article_id});
	}
	else {
	    # permission denied - use a dummy article
	    @art = (
		    { title=>"Permission denied",
		      summary=>"The $this->{article}{type} you are attempting to view has restricted access, and you do not have permission to view it.",
		      article_id=>$this->{article_id},
		  } );
	}
    }
    else {
	$title = "Articles";
	$description = "from $config{site}{name}";
	$url = $config{server}{server};
    }
    return undef if (@art == 0);

    my $rss = new ExSite::RSS;
    $rss->title($this->rss_clean($title));
    $rss->description($this->rss_clean($description));
    $rss->link($url);

    $this->save;
    foreach my $a (@art) {
	$this->set($a);
	my $t = new ExSite::Time($this->{article}{ctime},"sql_timestamp");
	my $pubdate = $t->write("arpa");
	my $item = {
	    title=>$this->rss_clean($this->title),
	    description=>$this->rss_clean($this->abstract),
	    link=>$this->link(__plaintext=>1,zine=>"show",aid=>$this->{article_id}),
	    pubDate=>$pubdate,
	};
	if ($a->{type} eq "comment") {
	    $item->{author} = $a->{author};
	}
	$rss->item($item);
    }

    $this->restore;
    return $rss->write;
}

# the RSS page is the page that RSS links point to

sub rss_page {
    my ($this) = @_;
    my $section_id = $this->{article}{section_id};
    if (! $section_id && $share{Page}) {
	$section_id = $share{Page}->id("section");
    }
    return undef if (! $section_id);
    my $section = new ExSite::Section(id=>$section_id);
    my $page = $section->service_page("Zine");
    return $page;
}

sub rss_clean {
    my ($this,$text) = @_;
    $text = &html_to_plaintext($text);
    # above will actually unescape escaped html, so strip all tags
    $text =~ s/<.+?>/ /g;
    return $text;
}

1;



