#!/usr/bin/perl

package Modules::Zine::Base;

#==========================================================================
# Zine base class - defines package globals, and general-purpose accessors
#==========================================================================

use strict;
use ExSite::Config;
use ExSite::Report;
use ExSite::Misc;
use ExSite::Util;
use ExSite::Mail;
use ExSite::HTML;
use ExSite::Crypt;
use ExSite::Image;
use ExSite::Time;
use ExSite::RSS;
use ExSite::Tree;
use Modules::Zine::Config;

use vars qw(@ISA);

@ISA = qw(Modules::Zine::Config);

=pod

=head1 Queries & Finding Articles

=cut

# setup - initialize object

sub setup {
    my ($this,$article,$opt) = @_;

    # load the article
    if ($article) { $this->load($article); }

    # set up query lib

    my $db = $share{DB};
    $db->set_query("articles-direct relations",
		   (sql=>"select * from article where article_id=? or thread=? or parent_id=? or reference=? order by article_id",
		    nparam=>4,
		    mode=>"r",
		    keys=>"article")
		   );
    $db->set_query("articles-indirect relations",
		   (sql=>"select a.* from article as a, article as b where (b.article_id=? or b.thread=? or b.parent_id=? or b.reference=?) and a.reference=b.article_id and a.reference!=? order by a.article_id",
		    nparam=>5,
		    mode=>"r",
		    keys=>"article")
		    );
    $db->set_query("articles-direct relations of indirect relations",
		   (sql=>"select a.* from article as a, article as b, article as c where (c.article_id=? or c.thread=? or c.parent_id=? or c.reference=?) and a.reference=b.article_id and b.reference=c.article_id and b.reference!=? order by a.article_id",
		    nparam=>5,
		    mode=>"r",
		    keys=>"article")
		    );

    return;
}

# clear - un-initialize object

sub clear {
    my $this = shift;
    delete $this->{article};
    delete $this->{article_id};
    delete $this->{reference};
}

sub is_active {
    my ($this) = @_;
    return 0 if (ref $this->{article} ne "HASH");
    return 1 if ($this->{article}{status} eq "active");
    return 0;
}

# fetch - retrieve an article by its ID or name
#         Try to match to any cached articles, if we can.

sub fetch ($$) {
    my ($this,$id) = @_;
    my $article;
    if ($id =~ /^\d+$/) {
	if ($id == $this->{article_id}) {
	    $article = $this->{article};
	}
	elsif ($id == $this->{reference}{article_id}) {
	    $article = $this->{reference};
	}
	elsif ($this->{tree} && $this->{tree}->exists($id)) {
	    $article = $this->{tree}->getnode_data($id);
	    if (keys %$article <= 1) { $article = undef; }  # dummy node
	}
	if (! $article) {
	    $article = $share{DB}->fetch("article",$id);
	}
    }
    elsif (ref $id eq "HASH") {
	# looks like we were passed the explicit record
	$article = $id;
    }
    elsif ($id) {
	# non-numeric article ID; try matching to "name"
	if ($id eq $this->{article}{name}) {
	    $article = $this->{article};
	}
	else {
	    my $ref_id = $this->class eq "zine" ?
		$this->{article_id} : $this->{reference}{article_id};
	    my $name = $id;
	    $name =~ s/_/%/s;
	    my @art = $share{DB}->fetch_match("article",{name=>$id,
							 reference=>$ref_id,
						     },"article_id");
	    if (@art > 0) {
		$article = $art[-1];
	    }
	}
    }
    return $article;
}

# load : fetch an article and configure the object with it

sub load ($$) {
    my ($this,$id) = @_;
    if ($id =~ /^\d+$/) {
	if ($id != $this->{article_id}) {
	    $this->{article} = $this->fetch($id);
	    if (ref $this->{article} eq "HASH") {
		if ($this->{article}{type} eq "alias") {
		    # pointer to another article
		    $id = $this->{article}{reference};
		    $this->{article} = $this->fetch($id);
		}
		$this->{article_id} = $id;
		if ($this->{article}{reference}) {
		    $this->{reference} =
			$this->fetch($this->{article}{reference});
		}
		else {
		    $this->{reference} = undef;
		}
	    }
	}
    }
    elsif (ref $id eq "HASH") {
	# looks like we were passed the explicit record
	$this->{article} = $id;
	$this->{article_id} = $id->{article_id};
	$this->{reference} = $this->{article}{reference} ?
	    $this->fetch($this->{article}{reference}) : undef;
    }
    elsif ($id) {
	# non-numeric article ID; try matching to "name"
	if ($id != $this->{article}{name}) {
	    my $ref_id = $this->class eq "zine" ?
		$this->{article_id} : $this->{reference}{article_id};
	    my @art = $share{DB}->fetch_match("article",{name=>$id,
							 reference=>$ref_id,
						     },"article_id");
	    if (@art > 0) {
		$this->{article} = $art[-1];
		if ($ref_id != $this->{reference}{article_id}) {
		    $this->{reference} =
			$this->fetch($ref_id);
		}
	    }
	    else {
		$this->{article} = undef;
		$this->{reference} = undef;
	    }
	}
    }
    return $this->{article};
}

# find - retrieve an article by its path

sub find {
    my ($this,$path) = @_;
    my @path = split /\//,$path;
    my ($article,$ref,@zpath);

    shift @path;  #dummy
    my $name = shift @path;
    
    # the first element of the path could be a public or private zine

    # make a list of section IDs to search for the matching zine
    my @sids;
    if ($share{Page}) {
	push @sids, $share{Page}->id("section");
    }
    else {
	@sids = map { $_->{section_id} } $share{DB}->my_real_sites;
    }
    push @sids,0;

    # find the base zine
    my $query = "select * from article where name=? and parent_id=0 and section_id in (".join(",",@sids).")";
    $article = $share{DB}->custom_query($query,$name)->[0];
    if (! $article->{article_id} && $name =~ /^\d+$/) {
	# not found, but could be an ID
	$query = "select * from article where article_id=? and parent_id=0 and section_id in (".join(",",@sids).")";
	$article = $share{DB}->custom_query($query,$name)->[0];
    }

    # abort if no match
    return undef if (! $article->{article_id});
    
    # continue the search
    push @zpath,$name;
    while (@path) {
	$name = shift @path;
	next if ($name =~ /^\s*$/);
	if ($name !~ /^[\w-\.]+$/) {
	    # $this->warn("'$name' is an invalid article name.");
	    last;
	}
	$ref = $article;

	$article = $this->fetch_match({name=>$name,reference=>$ref->{article_id}})->[0];
	if (! $article->{article_id} && $name =~ /^\d+$/) {
	    # not found, but could be an ID
	    $article = $this->fetch_match({article_id=>$name,reference=>$ref->{article_id}})->[0];
	}
	if ($article->{article_id}) {
	    push @zpath, $name;
	}
	else {
	    #$this->warn("Article '$name' not found.");
	    last;
	}
    }
    if (! $article->{article_id} && $ref->{article_id}) {
	$article = $ref;
	$ref = undef;
    }

    if (@zpath > 0) {
	# looks like the path corresponds to a Zine node;
	# we'll register this path element with Input
	my $zpath = "/".join("/",@zpath);
	my $in = new ExSite::Input;
	$in->new_path("Zine",$zpath);
    }
    
    $this->{article} = $article;
    if ($article->{reference} == $ref->{article_id}) {
	$this->{reference} = $article->{reference} ? $ref : undef;
	$this->{article_id} = $article->{article_id};
	return $article;
    }
    else {
	return $this->set($article);
    }
}

sub set ($$) {
    my ($this,$art) = @_;
    $this->{article_id} = $art->{article_id};
    $this->{article} = $art;
    if ($this->{article}{reference}) {
	$this->{reference} = $this->fetch($this->{article}{reference});
    }
    else {
	$this->{reference} = undef;
    }
    return $art;
}

# save/restore - for temporary changing of article context
# Articles are placed on a stack, so we can save/restore any number of times.

sub save {
    my $this = shift;
    push @{$this->{save}{article}}, $this->{article};
    push @{$this->{save}{article_id}}, $this->{article_id};
    push @{$this->{save}{reference}}, $this->{reference};
    return;
}

sub restore {
    my $this = shift;
    $this->{article} = pop @{$this->{save}{article}};
    $this->{article_id} = pop @{$this->{save}{article_id}};
    $this->{reference} = pop @{$this->{save}{reference}};
    return;
}

sub fetch_match {
    my ($this,$match,$sort) = @_;
    if (! $sort) { $sort = "ctime"; }
    return $share{DB}->fetch_match("article",$match,$sort);
}

sub fetch_tree {
    my ($this,$id,$match) = @_;
    return undef if ($id !~ /^\d+$/);

    # check if the requested tree is a subtree of the currently-loaded tree
    if ($this->{tree} && $this->{tree}->exists($id)) {
	return $this->{tree}->subtree($id);
    }

    ### FIXME: fetch the most re-useable tree, eg. fetch the zine

    my @art;
    my $db = $share{DB};
    my $topart = $this->fetch($id);
    my $thread = $topart->{thread} || $id;

    # find direct relations
    @art = $db->get_query("articles-direct relations",$id,$thread,$id,$id);

    # get root article class
    my $root_article;
    if ($id == $this->{article_id}) {
	$root_article = $this->{article};
    }
    else {
	my %ahash = &keywise("article_id",@art);
	$root_article = $ahash{$id};
    }
    my $rootclass = $this->class($root_article);
    my $ignore = $root_article->{parent_id};

    # if rootclass is comment or attachment, we need search no further
    if ($rootclass =~ /article|zine/) {
	# find indirect relations, 
	# eg. articles under a subzine, comments under a subarticle
	push @art, $db->get_query("articles-indirect relations",$id,$id,$id,$id,$id);
	if ($rootclass eq "zine" && $root_article->{parent_id}) {
	    # find direct relations of indirect relations, 
	    # eg. comments under subarticles under a subzine
	    push @art, $db->custom_query("articles-direct relations of indirect relations",$id,$id,$id,$id,$id);
	}
    }
    
    # sort the list of articles
    my $rule = $this->rule("indexsort");

    # NB: we don't worry about dupes, because the Tree will ignore them
    # $ignore gives the parent of the root article in our tree;  the tree
    # will ignore this parent for purposes of constructing the tree.

    # build a tree from the articles
#    my $tree = new ExSite::Tree("article_id","parent_id",
#				$this->sort($rule,@art));
    my $tree = new ExSite::Tree("article_id","parent_id");
    my @alist = $this->sort($rule,@art);
    $tree->addnodes(\@alist,"article_id","parent_id",[$ignore]);
    
    # save it in case it's useful for finding other articles
    $this->{tree} = $tree;

    # we have extracted a complete tree for caching purposes, but if $match
    # is defined, then we really only want a subset of it.
    return $match ?
	$tree->subtree($id,$match) :
	$tree;
}

sub fetch_child_articles {
    my ($this,$mode,$limit,$sort,$opt) = @_;
    return undef if (! $this->{article_id});
    $mode = $mode || "article";
    if (! $opt) { $opt = {}; }
    # default to active articles, unless explicitly requesting otherwise
    if (! exists $opt->{status}) { $opt->{status} = "active"; }
    elsif (! $opt->{status}) { delete $opt->{status}; }
    my $match = { 
	parent_id => $this->{article_id},
	type => $mode,
	%$opt,
    };

    # sort rule
    if (! $sort) { $sort = $this->rule("indexsort",$mode); }

    # check if the requested tree is a subtree of the currently-loaded tree
    if ($this->{tree} && $this->{tree}->exists($this->{article_id})) {
	my @child = $this->{tree}->get_child_data($this->{article_id});
	my @goodchild;
	foreach my $c (@child) { 
	    if (&match_hash($match,$c)) { push @goodchild, $c; }
	}
        my @sorted = $this->sort($sort,@goodchild);
	return $limit ? @sorted[0..$limit-1] : @sorted;
    }
    else {
	my $dbsort = $this->{config}{sort_methods}{$sort} || 
	    ["sortkey",{"ctime"=>"desc"}];
	return $share{DB}->fetch_match("article",$match,$dbsort,$limit);
    }
}

sub fetch_latest {
    my ($this,$type,$id,$number) = @_;
    if (! $type) { $type = "article"; }
    # set $number if it was not passed in  
    if (! defined $number ) {
	$number |= $config{$this->module_name}{"new_".$type."s"} || 4;
    }
    # set the zines to check
    my @aid;
    if ($id) {
	@aid = ($id);
    }
    else {
	my @art = $this->our_zines();
	@aid = map { $_->{article_id} } @art;
    }
    return undef if (scalar @aid == 0);

    if (scalar @aid == 1 && $this->{tree} && $this->{tree}->exists($aid[0])) {
	my @art = $this->sort("reverse-date",$this->{tree}->collapse($aid[0]));
	my @latest;
	foreach my $a (@art) {
	    if ($a->{type} eq $type && $a->{status} eq "active") { 
		push @latest,$a; 
	    }
	    last if (@latest >= $number);
	}
	return @latest;
    }
    else {
	return $share{DB}->custom_query("select * from article where reference in (".join(",",@aid).") and type=\"$type\" and status=\"active\" order by article_id desc limit $number");
    }
}

# return array of zines owned by the current user

sub my_zines {
    my ($this) = @_;
    my $uid = $share{DB}->my_uid;
    return () if (! $uid);
    my %match = (status=>"active", owner=>$uid, reference=>0);
    return $share{DB}->fetch_match("article",\%match);
}

# return array of zines avialable to the current site

sub our_zines {
    my ($this,$sid) = @_;
    if (! $sid) { $sid = $share{Page} ? $share{Page}->id("section") : 0; }
    return () if (! $sid);
    my $dat = $share{Cache}->get("article","our_zines");
    if (! $dat) { 
	my $query = 'select * from article where status="active" and section_id in (0,?) and type not in ("article","comment","attachment")';
	$dat = $share{DB}->custom_query($query,$sid);
	$share{Cache}->save($dat,"article","our_zines");
    }
    return wantarray ? @$dat : $dat;
}

# return all zines in the system

sub all_zines {
    my ($this) = @_;
    my $status = $this->admin_mode ? "" : "status=\"active\" and ";
    my $query = "select * from article where $status type not in (\"article\",\"comment\",\"attachment\") order by sortkey";
    my $dat = $share{Cache}->get("article","all_zines");
    if (! $dat) { 
	$dat = $share{DB}->custom_query($query);
	$share{Cache}->save($dat,"article","all_zines");
    }
    return wantarray ? @$dat : $dat;
}

# sort an array of articles

sub sort {
    my ($this,$sortrule,@article) = @_;
    if ($sortrule eq "sorted") {
	return sort { $a->{sortkey} cmp $b->{sortkey} } @article;
    }
    elsif ($sortrule eq "reverse-date" || $sortrule eq "reverse-flip-flop") {
	return sort { $b->{ctime} cmp $a->{ctime} } @article;
    }
    # elsif ($sortrule eq "date" || $sortrule eq "flip-flop")
    return sort { $a->{ctime} cmp $b->{ctime} } @article;
}

# count descendant articles of a certain type, starting
# from an arbitrary article id.

sub count {
    my ($this,$id,$class) = @_;
    my $count = 0;
    if (! $id) { $id = $this->{article_id}; }
    if (! $class) { $class = "comment"; }
    my $tree = $this->fetch_tree($id);
    my $match = {type=>$class};
    if (! $this->admin_mode) { $match->{status} = "active"; }
    my $count = $tree->count($match,$id);
    if ($class eq $this->class) { $count--; }  # don't count ourselves
    return $count;
}

### this is not the same path as in show_path

sub path {
    my $this = shift;
    my @path;
    my $art = $this->{article};
    push @path, ($art->{name} || $art->{article_id});
    while ($art->{reference}) {
	$art = $this->load($art->{reference});
	unshift @path, ($art->{name} || $art->{article_id});
	# we continue until there are no more references;
	# but we could probably stop as soon as we reach a zine.
    }
    unshift @path, undef;
    return wantarray ? @path : join "/", @path;
}

=pod

=head1 Attributes

=cut

sub size {
    my $this = shift;
    return length $this->{article}{body};
}

# returns the article type;

sub type {
    my $this = shift;
    return $this->{article}{type} || "article";
}

# returns the reference article type;  determines general formatting rules

sub reftype {
    my $this = shift;
    return $this->{reference}{article_id} ?
	($this->{reference}{type} || "article") :
	($this->{article}{type} || "article");
}

# returns the zine type;  determines general formatting rules

sub zinetype {
    my $this = shift;
    return $this->{article}{type} if ($this->class eq "zine");
    return $this->{reference}{type} if ($this->class($this->{reference}) eq "zine");
    my $a = $this->{reference};
    while ($a->{reference}) {
	$a = $this->fetch($a->{reference});
	return $a->{type} if ($this->class($a) eq "zine");
    }
    return undef;
}

# returns the article type class.

sub class {
    my $this = shift;
    my $art = shift || $this->{article};
    my $type = $art->{type};# || "article";
    if ($type =~ /^(article|comment|attachment)$/) {
	return $type;
    }
    else {
	return "zine";
    }
}

sub refclass {
    my $this = shift;
    return (keys %{$this->{reference}} > 0) ? 
	$this->class($this->{reference}) :
	$this->class($this->{article});
}

# return the appropriate reference for a new article under the current one

sub newref {
    my ($this,$newclass) = @_;
    my $class = $this->class();
    if ($class eq "zine") {
	return $this->{article_id};
    }
    elsif ($class eq "article") {
	if ($newclass eq "comment") {
	    return $this->{article_id};
	}
	elsif ($newclass =~ /article|attachment/) {
	    return $this->{article}{reference};
	}
    }
    elsif ($class eq "comment") {
	return $this->{article}{reference};
    }
    # weird nesting being attempted;  this is probably an error...
    return undef;
}

### article tests

sub is_zine {
    my ($this) = @_;
    return ($this->type() !~ /^(article|comment|attachment)$/);
}

sub is_article {
    my ($this) = @_;
    return ($this->type() eq "article");
}

sub is_comment {
    my ($this) = @_;
    return ($this->type() eq "comment");
}

sub is_attachment {
    my ($this) = @_;
    return ($this->type() eq "attachment");
}

# is_new - returns 1 if the article is (or contains) newly-added content

sub is_new {
    my ($this,$tree) = @_;

    # handler for local definition of "new", eg. since last login
    my $stat = $this->run_handler("zine_is_new",$tree);
    return $stat if (defined $stat);

    $this->save;
    my $article = $this->{article};
    if ($this->is_zine) {
	# zines are new if any contents are younger than a certain age
	if ($this->confrule("allow","article",$this->type)) {
	    my @art = $this->fetch_latest("article",$this->{article_id},1);
	    $article = $art[0];
	}
	elsif ($this->confrule("allow","comment",$this->type)) {
	    my @art = $this->fetch_latest("comment",$this->{article_id},1);
	    $article = $art[0];
	}
    }
    my $is_new = ($this->age() <= $config{Zine}{new_age});
    $this->restore;
    return $is_new;
}

# rule: find and return an inheritable rule
#
# Inheritable rules are formatting and permissions rules that affect
# sub-articles.  Articles may define (some) of their own rules;  if they
# do not define a rule, we look at the reference article and then zine
# for defined rules.  If still not found, we look in the conf files.
#
# $class can be specified if looking up a formatting rule for a
# different class of article than the current article; eg. the rule
# for formatting comments under an article.  Otherwise the class will
# be assumed to be the same as for the current article (eg. formatting
# articles under an article).
#
# $rule is typically one of:
# indexlimit: maximum number of articles to index
# indexmin: minimum number of articles to index
# indexformat: format of each article in an index
# indextype: layout of the whole index
# indexsort: order of the articles in the index
# indexage: maximum age (in days) of articles in the index
# attachments: number of accepted attachments when posting
# status: default status of newly posted articles
# comment_policy: defines who may comment on the article

sub rule { 
    my ($this,$rule,$class) = @_;
    if (! $class) { $class = $this->class; }

    ### check for settings in the data

    # the article itself overides all definitions
    if ($this->{article}{$rule}) {
	return $this->{article}{$rule};
    }

    my $zine = ($this->refclass eq "zine") ? 
	$this->{reference} : $this->my_zine();

    # the reference article defines defaults for all its sub-articles
    # IF the class is allowed
    my $ztype = $this->zinetype();
    if ($this->confrule("allow",$class,$ztype,$zine->{article_id})) {
	if ($this->{reference}{$rule}) {
	    return $this->{reference}{$rule};
	}
    }

    # the zine defines defaults for all of its contents
    if ($zine->{$rule}) {
	return $zine->{$rule};
    }

    # no rule found; check for configuration settings

    return $this->confrule($rule,$class,$ztype,$zine->{article_id});
}

sub get { 
    my ($this,$attribute,$class) = @_;

    # look in the current article
    
    if ($this->{article}{$attribute}) {
	return $this->{article}{$attribute};
    }

    # For indexes, look through the reference articles for 
    # inheritable attributes.  Attributes are inheritable if the "allow" 
    # rule indicates that we accept articles of $class in the zine.
    # That means:
    # If the zine accepts articles but not comments, then inheritable 
    # indexing rules apply to articles, but not comments.  (Comments 
    # will be formatted according to the general configuration rules.)
    # If the zine accepts comments but not articles (eg. a forum), then 
    # indexing rules apply to comments, but not articles.  If the zine
    # accepts both, then the inheritable rules will apply to both.

    if ($class eq "article" &&
	$this->confrule("allow","article",$this->zinetype) &&
	$attribute =~ /comment_policy|index(age|limit|type|format)/) {
	my $val = $this->getref($attribute);
	return $val if ($val);
    }

    if ($class eq "comment" && 
	$this->confrule("allow","comment",$this->zinetype) &&
	$attribute =~ /comment_policy|index(age|limit|type|format)/) {
	my $val = $this->getref($attribute);
	return $val if ($val);
    }

    # check the configuration file for default attributes

    my $zinetype = $this->zinetype;
    if (! $class) { $class = $this->class; }
    # zine-specific configurations
    if ($config{$this->module_name}{$zinetype}{$class}{$attribute}) {
	return $config{$this->module_name}{$zinetype}{$class}{$attribute};
    }
    # general configurations
    if ($config{$this->module_name}{$class}{$attribute}) {
	return $config{$this->module_name}{$class}{$attribute};
    }

    return undef;
}

sub getref {
    my $this = shift;
    $_ = shift;
    if ($this->{article} && ! $this->{reference}) {
	$this->{reference} =
	    $this->fetch($this->{article}{reference});
    }
    return $this->{reference}{$_} ? $this->{reference}{$_} : undef;
}

sub show_data {
    my $this = shift;
    my $col = shift;
    my $data = $this->{article}{$col};

    # kill/neuter any hidden CMS tags
    if ($col ne "picture") {
	if (! $config{$this->module_name}{allow_cms_tags}) {
	    $data =~ s/<!--/<!-- /g;
	}
	if (! $config{$this->module_name}{allow_cms_links}) {
	    $data =~ s/\{\{/\{&\#00;\{/g;
	    $data =~ s/\}\}/\}&\#00;\}/g;
	    $data =~ s/\[\[/\[&\#00;\[/g;
	    $data =~ s/\]\]/\]&\#00;\]/g;
	}
    }
    return $share{DB}->show_data_noauth("article",$col,$data,
					$this->{article_id});
}

sub label {
    my ($this,$col,$type) = @_;
    if (! $type) { $type = $this->type; }
    my $label = ($type eq "comment") ?
	$this->{config}{comment_label}{$col->{column}} || 
	$col->{label} || 
	$col->{column} :
	$col->{label} || $col->{column};
    return $msg{$label};
}

sub title {
    my $this = shift;
    my $title = $this->get("title");
    if (! $title) { 
	$title = $this->class eq "comment" ? 
	    $msg{"(no subject)"} : 
	    $msg{"(no title)"};
    }
    return $title;
}

sub name {
    my $this = shift;
    my $name = $this->get("name") || $this->{article_id};
    return $name;
}

sub reftitle {
    my $this = shift;
    return $this->getref("title");
}

sub author {
    my $this = shift;
    my $stat = $this->run_handler("ZineAuthor");
    return $stat if ($stat);
    my $author = $this->get("author");
    if ($author) {
	if ($this->{article}{type} eq "comment" && 
	    $config{$this->module_name}{mark_unauthenticated_comments} && 
	    ! $this->{article}{owner}) {
	    my $mark = $config{$this->module_name}{unauthenticated_comment_mark};
	    return $mark ? "$author $mark" : $author;
	}
	else {
	    return $author;
	}
    }
    elsif ($this->type eq "comment") {
	if ($this->{article}{owner}) {
	    # comment is not anonymous, but user blanked their name
	    if ($this->{config}{withhold_name}) {
		# blank names allowed
		return $msg{"(name withheld)"};
	    }
	    else {
		# blank names not allowed
		my %owner = $share{DB}->fetch("member",$this->{article}{owner});
		return $owner{name} || $owner{login};
	    }
	}
	else {
	    return $config{$this->module_name}{anonymous_name} || $msg{"Anonymous"};
	}
    }
    return "";
}

sub author_email {
    my $this = shift;
    if ($this->{article}{owner}) {
	my %owner = $share{DB}->fetch("member",$this->{article}{owner});
	return $owner{email};
    }
    return undef;
}

sub author_login {
    my $this = shift;
    if ($this->{article}{owner}) {
	my %owner = $share{DB}->fetch("member",$this->{article}{owner});
	return $owner{$config{auth}{user_login_column}};
    }
    return undef;
}

# basic timestamp formatting

sub timestamp {
    my ($this,$dtype) = @_;
    if (! $dtype) { $dtype = "ctime"; }  # alternative is mtime
    return $this->show_data($dtype);
}

# article date

sub date {
    my ($this,$type) = @_;
    my ($timestamp, $timetype, $label);
    if (! $type) { $type = $this->{config}{datetype}; }
    if ($type eq "simple") {
	$label = "";
	$timetype = "ctime";
    }
    elsif ($type =~ /creation|ctime/) {
	$label = $msg{"posted on "};
	$timetype = "ctime";
    }
    else { 
	$timetype = ($this->{article}{mtime} ne $this->{article}{ctime}) ? 
	    "mtime" : "ctime";
	$label = $timetype eq "mtime" ? $msg{"updated on "} : $msg{"posted on "};
    }
    $timestamp = $this->timestamp($timetype);
    return $label.$timestamp;
}

# return age of article in days (approximate - does not account for leap years)

sub age {
    my $this = shift;
    my $time = new ExSite::Time; # (now)
    # time diff is in seconds; convert to days
    my $age = ($time->diff($this->{article}{ctime},"sql_timestamp")) / 86400;
    return - int($age);
}

# indexlimit: the maximum number of items that will be displayed in an index
# if indexlimit == 0, the index size is unlimited
# if indexlimit is undefined, it defaults to some number

sub indexlimit {
    my ($this,$class) = @_;
    # check input for override
    my $input = $this->{input}{indexlimit} || $this->{opt}{indexlimit};
    return $input if ($input);
    # custom article setting
    if (! $class) { $class = $this->class; }
    my $limit = $this->rule("indexlimit",$class);
    return $limit if ($limit);
    return 10;
}

# indexage: the maximum age of items that will be displayed in an index
# if indexage == 0, the index age is unlimited
# if indexage is undefined, it defaults to some number

sub indexage {
    my ($this,$class) = @_;
   # check input for override
    my $input = $this->{input}{indexage} || $this->{opt}{indexage};
    return $input if ($input);
    # custom article setting
    if (! $class) { $class = $this->class; }
    my $age = $this->rule("indexage",$class);
    return $age if ($age);
    return 30;
}

# indextype: the rules for laying out the index
#
# basic - each article is wrapped in a div
# deep - like basic, but the full depth of the tree is shown
# list - each article is a bullet item in a list
# numbered - each article is a numbered item in a list
# table - each article is a row in a table
# gallery - each article is a cell in a table
# threaded - each article is nested under its parent
# semithreaded - table for top-level; deep for sublevels  ###TODO
# paged - next/previous links to page through the articles
# wiki - use special wiki rules

sub indextype {
    my ($this,$class) = @_;
    # check input for override
    my $input = $this->{input}{indextype} || $this->{opt}{indextype};
    return $input if ($input);
    # custom article setting
    if (! $class) { $class = $this->class; }
    my $type = $this->rule("indextype",$class);
    return $type if ($type);
    return "basic";
}

# indexformat: the rules for formatting each article in the index
#
# full - show the whole article
# summary - show an article summary/abstract
# basic - show linked title + author + date
# title - show linked title

sub indexformat {
    my ($this,$class) = @_;
    # check input for override
    my $input = $this->{input}{indexformat} || $this->{opt}{indexformat};
    return $input if ($input);
    # custom article setting
    if (! $class) { $class = $this->class; }
    my $fmt = $this->rule("indexformat",$class);
    return $fmt if ($fmt);
    return "basic";
}

# indexsort: the rules for sorting the index
#
# sorted - by sortkey
# date - by date, most recent last
# reverse-date - by date, most recent first
# flip-flop - by date on top-level; by reverse-date on sub-levels
# reverse-flip-flop - reverse-date on top-level; by date on sub-levels

sub indexsort {
    my ($this,$class) = @_;
    # check input for override
    my $input = $this->{input}{indexsort} || $this->{opt}{indexsort};
    return $input if ($input);
    # custom article setting
    if (! $class) { $class = $this->class; }
    my $sort = $this->rule("indexsort",$class);
    return $sort if ($sort);
    return "reverse-date";
}

# comment_policy: whether or not comments are accepted
# DEPRECATED - use policy() instead

sub comment_policy {
    my ($this,$class) = @_;
    # custom article setting
    my $com = $this->get("comment_policy",$class);
    return $com if ($com);
    # configuration setting
    my $ref = $this->reftype;
    ### FIXME: should test for site membership, not just access level
    my $access = $share{DB}->authorize;
    if ($access) {
	$com = $config{$this->module_name}{$ref}{comment_policy}{member};
    }
    else {
	$com = $config{$this->module_name}{$ref}{comment_policy}{public};
    }
    return $com if ($com);
    # comments are forbidden by default, unless this is a forum or comment
    if ($access && $this->{article}{type} =~ /forum|comment/) {
	return "allowed";
    }
    return "forbidden";  
}

# find relations

sub my_zine {
    my $this = shift;
    if ($this->class() eq "zine") {
	return $this->{article};
    }
    elsif ($this->class($this->{reference}) eq "zine") {
	return $this->{reference};
    }
    my $refid = $this->{reference}{reference};
    return $this->fetch($this->{reference}{reference});
}

sub my_article {
    my $this = shift;
    my $class = $this->class();
    if ($class eq "zine") {
	return undef;
    }
    elsif ($class eq "article") {
	return $this->{article};
    }
    elsif ($this->class($this->{reference}) eq "article") {
	return $this->{reference};
    }
    else {
	return undef;
    }
}

sub my_comment {
    my $this = shift;
    my $class = $this->class;
    if ($class eq "comment") {
	return $this->{article};
    }
    elsif ($class eq "attachment") {
	my $parent = $this->fetch($this->{article}{parent_id});
	if ($parent->{type} eq "comment") {
	    return $parent;
	}
    }
    return undef;
}

sub description {
    my ($this,$data) = @_;
    if (! $data) { $data = $this->{article}; }
    my $class = $this->class($data);
    if (! $class) {
	return undef;
    }
    elsif ($class =~ /zine|article/) {
	return $data->{title};
    }
    else {
	return ucfirst $class . " \"" . $data->{title} . "\"";
    }
}

# show_path : shows a cookie-crumb menu of the parent articles

sub show_path {
    my ($this,%opt) = @_;
    my ($url,@plink);
    my $ml = &get_obj("ML");
    if ($this->class ne "zine" || $this->{admin}) {
	my $data = $this->{article};
	while ($data) {
	    my $url = $this->link(zine=>"show",aid=>$data->{article_id},
				  zadm=>undef,archive=>undef,goto=>undef);
	    my $label = $this->description($data);
	    unshift @plink, $ml->a($label,{href=>$url});
	    # next line is a kludge to avoid structure zines (eg. books that
	    # bundle other zines).
	    last if ($this->class($data) eq "zine");
	    $data = $data->{parent_id} ? $this->fetch($data->{parent_id}) : 0;
	}
	if ($config{$this->module_name}{show_top} || $this->{admin}) {
	    $url = $this->link(zine=>undef,aid=>undef,
			       zadm=>undef,archive=>undef,goto=>undef);
	    unshift @plink, $ml->a($msg{Top},{href=>$url});
	}
    }
    if (@plink > 0) {
	return $ml->p(join(" &gt; ",@plink));
    }
    return undef;
}

#==========================================================================
# delete the article and its children

sub delete {
    my ($this) = @_;
    my $db = $share{DB};
    my $data = $this->{article};
    my $ml = &get_obj("ML");
    
    my $out;
    if ($this->{admin} && $this->owns) {
	# deletions are restricted to the control panel and owner
	$db->trash_r("article",$this->{article_id},1);
	$out .= $ml->p( ucfirst($data->{type})." ".$data->{article_id}.
			" was moved to the trash.");
    }
    else {
	$out .= $ml->p("Sorry, you do not have permission to delete this article.");
    }
    my $url;
    if ($data->{parent_id}) {
	$url = $this->link(zine=>"show",aid=>$data->{parent_id});
    }
    else {
	$url = $this->link(zine=>undef,aid=>undef);
    }
    $out .= $ml->p($ml->a($msg{Continue},{ href=>$url }));
    return $out;
}

sub show_data {
    my $this = shift;
    my $col = shift;
    return $share{DB}->show_data_noauth("article",$col,
				 $this->{article}{$col},$this->{article_id});
}

sub clean_text { 
    my ($this,$text,$allow_basic_html) = @_;

    # Escape all CMS tags so they show clearly;  this allows us to 
    # write about the CMS system for example.

    # NB: this is normally used when INSERTING data into the database.
    # When displaying, we allow DB->clean_text() to clean the data, 
    # which will not mask all CMS tags.  That means you can force
    # CMS tags into an article by inserting them directly (not via
    # this module's doform() method).

    # escape double-bracket CMS tags not inside HTML tags
    $text =~ s/(?!<[^<>]*)\[\[(?![^<>]*>)/\&\#91;\&\#91;/g;
    $text =~ s/(?!<[^<>]*)\]\](?![^<>]*>)/\&\#93;\&\#93;/g;
    $text =~ s/(?!<[^<>]*)\{\{(?![^<>]*>)/\&\#123;\&\#123;/g;
    $text =~ s/(?!<[^<>]*)\}\}(?![^<>]*>)/\&\#125;\&\#125;/g;

    # escape comment-style CMS tags
    $text =~ s/<!--([\$\&].*?)-->/&lt;!--$1--&gt;/g;
    $text =~ s/<!--(content\(.*?\))-->/&lt;!--$1--&gt;/g;

    # now proceed with standard text clean-up
    return $share{DB}->clean_text($text,$allow_basic_html);
}

#------------------------------------------------------------------------
# article security

# admin_mode : returns TRUE if 
#   - in an administrator control panel
#   - if the owner is previewing the page

sub admin_mode {
    my $this = shift;
    # admin mode can be explicitly turned on by the admin control panel
    return 1 if ($this->{admin});
    if ($share{Page} && $share{DB}) {
	# admin mode always off for public or when publishing
	return 0 if ($share{DB}->level == 0);
	# admin mode is never enabled for regular users
	return 0 if ($share{DB}->level == 1);
	# admin mode is on if the viewer is the owner
	return 1 if ($this->owns());
    }
    return 0;
}

# owns : tests whether the viewer has permission to modify/update the article

sub owns {
    my $this = shift;
    my $db = $share{DB};
    return 1 if ( $db->is_manager() ); 
    my $uid = $db->my_uid;
    return 0 if ( ! $uid );
    return ($uid == $this->{article}{owner});
}

# privacy : is this a public or members-only article

sub privacy { 
    my $this = shift;
    my $privacy = $this->{reference}{privacy} || "public";
    if ($privacy eq "public") {
	# public articles in private zines are private
	$privacy = $this->{article}{privacy} || "public";
    }
    if ($privacy eq "public" && $this->class eq "comment") {
	# comments to articles in private zines are private
	my $zine = $this->fetch($this->{reference}{reference});
	$privacy = $zine->{privacy} || "public";
    }
    return $privacy;
}

# approve : test whether the user has permission to view the article

sub approve { 
    my $this = shift;
    my $db = $share{DB};

    # managers/admins see all
    return 1 if ($db->is_manager);
    
    # article privacy check
    return 0 if ($this->privacy =~ /members/ && ! $db->is_member);

    ### reference article/Zine privacy check?

    # article status check (must be active or unset [default is active])
    return 1 if (! $this->{article}{status} || 
		 $this->{article}{status} eq "active");

    return 0;
}

# moderator : test whether the user has permission to moderate comments

sub moderator {
    my $this = shift;
    my $db = $share{DB};
    return 1 if ( $db->is_manager() ); 
    my $article;
    if ($this->{article}{type} eq "comment") {
	$article = $this->{reference};
    }
    else {
	$article = $this->{article};
    }
    my $uid = $db->my_uid;
    return 0 if ( ! $uid ); 
    return ($uid == $article->{owner});
}

# allow_comments : test whether comments are accepted on the current article

sub allow_comments {
    my ($this) = @_;
    my $dummy_comment = { type=>"comment",
			  parent_id=>$this->{article_id},
			  reference=>$this->newref("comment") };
    return $this->allow_post($dummy_comment) ? 0 : 1;
}

sub allow_articles {
    my ($this) = @_;
    my $dummy_article = { type=>"article",
			  parent_id=>$this->{article_id},
			  reference=>$this->newref("article") };
    return $this->allow_post($dummy_article) ? 0 : 1;
}

# allow_post : test whether user is allowed to post;  returns an error
# string if DISallowed, undef if allowed.  Article data must be provided.

sub allow_post {
    my ($this,$data) = @_;
    my $ml = &get_obj("ML");
    
    if (scalar keys %$data == 0 && $this->{admin}) {
	# admin is creating a new zine
	return undef;
    }
    elsif ($data->{article_id} > 0) {
	# we appear to be editing an existing article

	if (! $this->admin_mode) {
	    my $uid = $share{DB}->my_uid();
	    if ($uid && $data->{owner} == $uid) {
		# we own the article
		if ($data->{type} eq "comment") {
		    # posters may edit their own comments within a certain time limit
		    my $t = new ExSite::Time;
		    my $t1 = $t->write("unix_timestamp");
		    $t->set($data->{ctime},"sql_timestamp");
		    my $t2 = $t->write("unix_timestamp");
		    if ($t1 - $t2 > $config{$this->module_name}{edit_my_comment_timeout}) {
			return $ml->p($msg{"This comment is now frozen."});
		    }
		    # else, edit comment is allowed
		    return undef;
		}
		# else it's an article, and we can edit our own articles
		return undef;
	    }
	    else {
		# we don't own the article
		return $ml->p(&substitute($msg{"You don't have permission to edit this [[article_type]]."},{article_type=>$data->{type}}));
	    }
	}
    }
    else {
	# We are inserting a new article.

	# check the posting policy
	my $policy = $this->policy($data);
	if ($policy =~ /member/ && 
	    ($share{DB}->level < 1)) {
	    return $ml->p($msg{"Sorry, only members may post here."});
	}
	elsif ($policy =~ /owner/ && 
	       ! ($share{Page} && $this->owns()) &&
	       ! ($this->admin_mode())) {
	    return $ml->p($msg{"Permission denied."});
	}
	elsif ($policy =~ /admin/ && 
	       ! $this->admin_mode()) {
	    return $ml->p($msg{"Sorry, only administrators may post here."});
	}
	elsif ($policy =~ /forbidden/) {
	    my $type = $data->{type} ? $data->{type}."s " : " ";
	    return $ml->p($msg{"Sorry, posting ${type}here is forbidden."});
	}
	elsif ($policy !~ /public|member|owner|admin|forbidden/) {
	    # unknown policy
	    return $ml->p("Sorry, posts are not accepted, because no posting policy has been defined.");
	}

	# check article status
	if ($this->{article}{status} ne "active") {
	    my $type = ucfirst($data->{type});
	    return $ml->p("Sorry, ${type}s are not accepted here.");
	}

	# just in case, check the parent status
	my $parent = $this->fetch($data->{parent_id});
	my $zine = $this->my_zine();
	if ($parent->{status} ne "active") {
	    return $ml->p(&substitute("Sorry, this [[ptype]] is [[pstatus]], and is not accepting [[type]]s.",{ptype=>$parent->{type},pstatus=>$parent->{status},type=>$this->{article}{type}}));
	}
    }
    return undef;
}

# policy - returns the policy for accepting submitted articles
#
# return values are:
# public - anyone may submit
# members - only members may submit
# owner - only the owner may submit
# admin - submissions only accepted from the administrator control panel
# forbidden - no submissions are accepted

sub policy {
    my ($this,$data) = @_;

    # determine what we are trying to post
    my ($type,$class,$zine,$zinetype,$refclass,$parent,$pclass,$rule);
    $this->save;
    $this->set($data);
    $type = $this->type;
    $class = $this->class;
    $zinetype = $this->zinetype;
    $zine = $this->my_zine;
    $parent = $data->{parent_id} ? $this->fetch($data->{parent_id}) : {};
    $pclass = $this->class($parent);
    $this->restore;

    # is this class even permitted to exist here?
    if ($pclass eq $class) {
	# only allowed if these classes can nest
	if (! $this->confrule("nest",$class,$zinetype,$zine->{article_id})) {
	    return "forbidden";
	}
    }
    if ($pclass eq "zine") {
	# check the allow rule
	if (! $this->confrule("allow",$class,$zinetype,$zine->{article_id})) {
	    return "forbidden";
	}
    }

    # check class hierarchy
    if ($class eq "zine") {
	return $pclass =~ /article|comment/ ? "forbidden" : "admin";
    }
    elsif ($class eq "article" && $pclass =~ /comment|attachment/) {
	return "forbidden";
    }
    elsif ($class eq "comment" && $pclass eq "attachment") {
	return "forbidden";
    }

    # class is permitted, now check policy settings for whom may add it

    if ($class eq "comment") {
	# Comments may be affected by the legacy rule "comment_policy",
	# which is still used in the article data.
	return 
	    $this->rule("comment_policy",$class,$zinetype) || 
	    $this->confrule("policy",$class,$zinetype,$zine->{article_id});
    }
    return $this->confrule("policy",$class,$zinetype,$zine->{article_id}) 
}

# A trusted article is one that is comes from the owner or administrator.
# Untrusted articles come from members and the public.

sub is_trusted {
    my $this = shift;
    return 1 if ($this->{article}{owner} == $this->{reference}{owner});
    return 0 if ($this->type eq "comment");
    return 1 if ($this->rule("policy","article") =~ /admin|owner/);
    return 0;
}

#------------------------------------------------------------------------
# link - optionally reference articles using pathinfo

sub link {
    my ($this,%opt) = @_;
    # remove keys that should not persist
    if (! exists $opt{_auth}) {
	$opt{_auth} = undef;
    }
    if (! exists $opt{ac}) {
	$opt{ac} = undef;
    }
    if (! exists $opt{cc}) {
	$opt{cc} = undef;
    }
    return (! $this->{admin} && $this->{config}{link_style} eq "path") ?
	$this->link_by_path(%opt) :
	$this->SUPER::link(%opt);
}

sub link_by_path {
    my ($this,%opt) = @_;

    my $in = new ExSite::Input;
    my $q = $in->query();

    # %opt2 will contain the new query options in the link
    my %opt2 = (%$q,%opt);

    my $cmd = $opt2{zine} || "show";
    if ($cmd eq "show" && $opt{aid}) {
	# this appears to be a simple link to view the article
	# so we will use path-linking
	$this->save;
	$this->load($opt{aid});
	my $path = $this->path();  # path to article
	if ($path) {
	    $opt{__path} = { "Zine"=>$path };
	    $opt{aid} = undef;          # remove article ID from query
	    $opt{zine} = undef;         # remove zine cmd from query
	}
	$this->restore;
    }
    else {
	# this is an administrative link, not a view link
	$in->path("Zine",undef);
    }
    return $this->SUPER::link(%opt);
}

# generate a link to an article, by name

sub link_to_name {
    my ($this,$name,$anchor) = @_;
    my $ml = &get_obj("ML");
    if (! $anchor) { $anchor = $name; }
    $name = &html_to_plaintext($name);
    $name =~ s/\s/_/g;
    my $art = $this->fetch($name);
    if ($art) {
	my $url = $this->link(aid=>$art->{article_id},zine=>"show");
	return $ml->a($anchor,{href=>$url});
    }
    else {
	return $anchor;
    }
}

# alink - shortcut link notation for cross-linking articles
# body text of the form ((tag|anchor)) is converted to:
# <a href=tag-url>anchor</a>.  Anchor defaults to tag.

sub alink {
    my ($this,$name,$anchor) = @_;
    my $ml = &get_obj("ML");
    if (! $anchor) {
	$anchor = $name;
	$anchor =~ s/_/ /g;
    }
    my $url = $this->link(goto=>$name);
    return $ml->a($anchor,{href=>$url});
}

=pod

=head1 Publishing

=cut

sub publish {
    my $this = shift;
    my $ml = &get_obj("ML");
    my $db = $share{DB};
    my ($out,@zines);
    $this->info("Publishing Zine");
    if ($this->{input}{aid}) {
	$this->load($this->{input}{aid});
	if ($this->owns()) {
	    if ($this->{article}{type} !~ /article|comment|attachment/) {
		if ($this->{input}{rss}) {
		    # rss only
		    return $this->publish_rss($this->{article});
		}
		else {
		    return $this->publish_zine($this->{article});
		}
	    }
	    else {
		return $this->publish_article($this->{article});
	    }
	}
    }
    else {
	$out = $ml->h2("Publishing Zines");
	# admins publish all; others only publish their own zines
	@zines = $db->is_admin ?
	    $db->fetch_match("article",{status=>"active",reference=>0}) :
	    $db->fetch_match("article",{status=>"active",reference=>0,
					owner=>$db->my_uid});
	foreach my $z (@zines) { $out .= $this->publish_zine($z); }
    }
    return $out;
}

sub publish_zine {
    my ($this,$zine) = @_;
    my $ml = &get_obj("ML");
    if ($zine->{privacy} =~ /members/) {
	return $ml->p("$zine->{title} is private, and cannot be published.");
    }
    my $db = $share{DB};
    my $out = "<h4>Publishing $zine->{title}</h4>\n";
    $out .= $this->publish_article($zine);
    my @articles = $db->fetch_match("article",{status=>"active",
					    reference=>$zine->{article_id}} );
    foreach my $art (@articles) { 
	$out .= $this->publish_article($art);
    }
    $out .= $this->publish_rss($zine);
    return $out;
}

sub publish_rss {
    my ($this,$zine) = @_;
    return undef if ($zine->{privacy} =~ /members/);
    my $db = $share{DB};
    my $ml = &get_obj("ML");

    # can only publish RSS feeds if there is a service page to direct
    # feed links to.

    return if (! $zine->{section_id});
    my $p = &ExSite::Module::service_page($this->module_name,
					  $zine->{section_id});
    if (! $p) {
	return $this->warn("Skipping RSS publishing (no service page).");
    }

    # do not publish comment feeds
    $this->set($zine);
    return undef if (! $this->rule("allow","article"));

    # make sure we don't publish RSS feeds with the wrong domain name
    $p->set_context("parent");
    my $s = new ExSite::Section(section=>$p->get("section"));
    my $surl = $s->get_url;
    if ($surl !~ /^$config{server}{server}/) {
	return $this->error("Cannot publish RSS from a different domain name");
    }

    # spoof the service page to get the correct links
    $p->update_env();

    my $out = "Publishing RSS feed for $zine->{title}...".$ml->br;
    my $rss = $this->rss;

    # restore environment (just in case)
    $ENV{SCRIPT_NAME} = $share{ENV}{SCRIPT_NAME};
    $ENV{QUERY_STRING} = $share{ENV}{QUERY_STRING};
    $ENV{PATH_INFO} = $share{ENV}{PATH_INFO};

    my $file = $this->diskpath."/index.rss";
    my $mkdir = $this->mkdir($this->diskpath);
    if (! $mkdir) {
	return $this->show_diagnostics("error","html");
    }
    $out .= $mkdir;
    $file =~ /^(((\/[\w-]+)+)\/([\w-]+\.[\w]+))$/;
    my $secure_filename = $1;
    if ($secure_filename) {
	$out .= "Writing $secure_filename... ";
	if (open OUTFILE, ">$secure_filename") {
	    print OUTFILE $rss;
	    close OUTFILE;
	    $out .= "succeeded.", $ml->br;
	}
	else {
	    $this->error("Zine::publish: publish $secure_filename failed: $!");
	    $out .= $ml->span("FAILED: $!",{class=>"error"}) . $ml->br;
	}
    }
    return $out;
}

sub publish_article {
    my ($this,$art) = @_;
#    return if ($art->{type} eq "comment");
    my $out;
    my $db = $share{DB};
    if ($art->{picture}) {
	$this->set($art);
	my $zpath = $this->diskpath;
	my $mkdir = $this->mkdir($zpath);
	if (! $mkdir) {
	    return $this->show_diagnostics("error","html");
	}
	$out .= $mkdir;
	my $img = new ExSite::Image($art->{picture});
	my $imgfile = &clean_filename($img->{filename});
	my $picpath = "$zpath/$art->{article_id}_$imgfile";
	$out .= $this->publish_file("picture",$picpath,$img);
	if ($art->{thumb}) {
	    $img->setup($art->{thumb});
	    $picpath = "$zpath/$art->{article_id}T_$imgfile";
	    $out .= $this->publish_file("picture",$picpath,$img);
	}
    }
    my @articles = $db->fetch_match("article",{status=>"active",
					       parent_id=>$art->{article_id}} );
    foreach my $article (@articles) { 
	$out .= $this->publish_article($article);
    }
    return $out;
}

sub publish_file {
    my ($this,$col,$file,$img) = @_;
    my $out;
    my $ml = &get_obj("ML");
    $file =~ /^(((\/[\w-]+)+)\/([\w-]+\.[\w]+))$/;
    my $secure_filename = $1;
    $out .= "Writing $secure_filename... ";
    if (open OUTFILE, ">$secure_filename") {
	print OUTFILE $img->get;
	close OUTFILE;
	$out .= "succeeded.".$ml->br;
    }
    else {
	$this->error("Zine::publish: publish $secure_filename failed: $!");
	$out .= $ml->span("FAILED: $!",{class=>"error"}) . $ml->br;
    }
    return $out;
}

sub mkdir {
    my ($this,$dir) = @_;
    if (! $this->{dircache}) { $this->{dircache} = {}; }
    return $this->{dircache}{$dir} if (exists $this->{dircache}{$dir});
    my $out = " ";  # dummy return value
    my $ml = &get_obj("ML");
    ### FIXME: doesn't allow dirs with "." in them, eg. /var/www/foo.com/html
    $dir =~ /(\/[\w-\/]+)/;
    my $secure_dir = $1;
    if (! $secure_dir) { 
	$this->error("mkdir: invalid directory: $dir");
	return undef;
    }
    my @path = split(/\//,$secure_dir);
    shift @path; # first one is a dummy
    my $path;
    foreach my $dir (@path) {
	$path .= "/$dir";
	next if (-e "$path");
	$out .= "Creating Zine directory $dir... ";
	if (! mkdir "$path") {
	    $this->error("mkdir $dir: failed: $!");
	    #$this->error("mkdir $path: failed: $!");
	    $this->{dircache}{$dir} = undef;
	    return undef;
	}
	else {
	    $this->info("created site directory $dir");
	    $out .= "succeeded.";
	    # add blank index file, to prevent snooping
	    my $file = "$secure_dir/index.html";
	    if (open OUTFILE, ">$file") {
		print OUTFILE "<html></html>\n";
		close OUTFILE;
	    }
	    $this->{dircache}{$dir} = " ";
	}
    }
    return $ml->p($out);
}

# publish methods, above, require special priveleges that only the publish
# program run by admins possesses.  If this does not describe us, we have 
# to call these methods indirectly as follows.

sub publish_unprivileged {
    my ($this,$article,%opt) = @_;
    my $out;
    my $ml = &get_obj("ML");
    if ($article->{picture}) {
	my $mod = $this->module_name;
	# if we are publishing rss, but not in a zine, switch to the zine
	my $aid = ($opt{rss} && $this->class($article) ne "zine") ?
	    $article->{reference} : $article->{article_id};
	my $uri = new ExSite::URI();
	$uri->setup("$config{server}{server}$config{server}{CGIpath}/$config{prog}{publish}");
	$uri->query(module=>$mod,aid=>$aid,%opt);
	my $u = $share{DB}->fetch_owner();
	my $uname = $u->{$config{auth}{user_login_column}};
	#my $authtoken = $share{DB}->make_authtoken($uname,1,$publish_url);
	my $authtoken = $uri->authtoken($uname,1);
	#$out .= $publish_url . $ml->br . $authtoken . $ml->br;
	$out .= &run_publisher("module",$mod,0,{%opt,aid=>$aid,_auth=>$authtoken});
    }
    return $out;
}

# diskpath to article's published files

sub diskpath {
    my $this = shift;
    return $config{server}{HTMLroot}.$this->htmlpath;
}

sub htmlpath {
    my $this = shift;
    my $zpath = "$config{server}{HTMLpath}/_Modules/Zine";
    if ($this->{article}{reference}) {
	$zpath .= "/$this->{article}{reference}";
    }
    else {
	$zpath .= "/$this->{article}{article_id}";
    }
    return $zpath;
}

# search_index adds links to the system search index
# $search = an ExSite::Search object, which can be used to update the index
# $site = an ExSite::Section object, to find site-specific zines/articles
# $page = the service page that will display all results

sub search_index {
    my ($this,$search,$site,$page) = @_;
    my $out;
    $page->update_env();
    my $url = $page->get_url_dynamic();
    $this->save;
    my $ml = &get_obj("ML");
    my $br = $ml->br();
    # loop over every zine we have access to
    foreach my $zine ($this->our_zines()) {
	$this->set($zine);
	$out .= "==> Indexing '".$this->title."'...$br";
	my $id = $this->{article_id};
	my $tree = $this->fetch_tree($id,{reference=>$id},1,1);
	my @art = $tree->collapse($id);
	# loop over every article in the zine
	foreach my $a (@art) {
	    next if ($a->{type} eq "article" &&
		     ! $this->{config}{search}{article});
	    next if ($a->{type} eq "comment" &&
		     ! $this->{config}{search}{comment});
	    $this->set($a);
	    my $aid = $this->{article_id};
#	    my $aurl = $url =~ /\?/ ? 
#		$url."&aid=${aid}&zine=show" :
#		$url."?aid=${aid}&zine=show";
	    my $aurl = $this->link(aid=>$aid,zine=>"show");
		
	    $search->add_url($aurl,
			     $site->id(),
			     $this->title(),
			     $this->abstract(),
			     $this->privacy(),
			     "Zine"
			     );
	    # strip the article down to its bare text
	    #my $text = &html_to_plaintext($this->show_article());
	    #$search->index($aurl,1,$text);
	    my $headtext = &html_to_plaintext($this->header());
	    my $headweight;
	    if ($a->{type} eq "article") {
		$headweight = 5;
	    }
	    else {
		# comment; rank top comments higher than replies
		$headweight = ($headtext =~ /^Re:/) ? 2 : 1;
	    }
	    $search->index($aurl,$headweight,$headtext);
	    my $abouttext = &html_to_plaintext($this->about());
	    $search->index($aurl,2,$abouttext);
	    my $bodytext = &html_to_plaintext($this->body());
	    $search->index($aurl,1,$bodytext);
	}
    }
    $this->restore;
    return $out;
}

1;
