#!/usr/bin/perl

package Modules::Zine::Digest;

#==========================================================================
# Digest : tools for sending email digests of Zine activity
#==========================================================================

use strict;
use ExSite::Config;
use ExSite::Module qw(&service_page);
use ExSite::Mail;
use ExSite::Misc;
use ExSite::Time;
use ExSite::URI;
use Modules::Zine::Article;

use vars qw(@ISA);
@ISA = qw(Modules::Zine::Article);

# setup - initialize object

sub setup {
    my ($this,$article,$opt) = @_;

    # $article is what we will build a digest of;  
    # typically it is a zine.  It can be undefined if we provide 
    # a section instead.

    if ($article) { $this->load($article); }

    # $opt can specify a section or a service page;  we need a service
    # page in order to provide links back to real HTML views

    my $section = $opt->{section} || $this->{article}{section_id};
    $this->{page} = &service_page("Zine", $section);

    # daily or weekly digest?
    $this->{type} = $opt->{type} =~ /^(daily|weekly)$/ ? 
	$opt->{type} : "daily";

    return;
}

sub get_start_time {
    my ($this) = @_;
    # get the start time of articles to digest
    my $t = new ExSite::Time;
    $this->{today} = $t->write("date");
    # go back 1 day/week
    my %tunit = ( daily=>"day", weekly=>"week" );
    $t->add(-1, $tunit{$this->{type}});
    return $t->write("sql_timestamp");
}

sub get_zine_ids {
    my $this = shift;
    if ($this->{article}) {
	# one zine
	return ($this->{article_id});
    }
    elsif ($this->{section}) {
	my $sid;
	my $s = $this->{section};
	if (ref $s =~ /Section/) { $sid = $s->id(); }
	elsif (ref $s eq "HASH") { $sid = $s->{section_id}; }
	else { $sid = $s; }
	# get zines
	my @zid;
	if ($sid) {
	    my @zine = $this->our_zines($sid);
	    return map { $_->{article_id} } @zine;
	}
    }
    return undef;
}

sub get_articles {
    my $this = shift;
    my $refids = join(",", $this->get_zine_ids());
    my $timestamp = $this->get_start_time;

    # This query will find articles/comments posted directly to a zine.
    # Eg. it will find comments in a forum, and articles in a blog, but
    # not comments in a blog.

    my @a = $share{DB}->custom_query("select * from article where status=\"active\" and reference in ($refids) and mtime > \"$timestamp\" order by article_id");
    return @a;
}

sub get_addresses {
    my ($this) = @_;

    ### NB: you MUST have a handler to generate the email list, or
    ### this package does nothing!

    my $addresses = $this->run_handler("ZineDigestSubscriptions");
    return @$addresses;
}

sub make_digest {
    my ($this) = @_;
    return undef if (! $this->{page});
    my (@articles, @comments, @atitles, @ctitles, %atitles, %ctitles);
    foreach my $msg ($this->get_articles()) {
	$this->set($msg);
	my $art = $this->show_as_text();
	if ($this->type eq "article") { 
	    push @articles,$art; 
	    my $title = $msg->{title};
	    $title =~ s/^Re: ?//;
	    if (! exists $atitles{$title}) {
		push @atitles, $title;
		$atitles{$title} = 1;
	    }
	}
	elsif ($this->type eq "comment") { 
	    push @comments, $art; 
	    my $title = $msg->{title};
	    $title =~ s/^Re: ?//;
	    if (! exists $ctitles{$title}) {
		push @ctitles, $title;
		$ctitles{$title} = 1;
	    }
	}
    }

    return undef if (scalar @comments + scalar @articles == 0);
    
    my $digest = &substitute($msg{"Do not reply to this message via email.
This is your [[type]] digest from [[site]] for [[day]].
You are receiving it because you have subscribed to it from our website.
To unsubscribe, log in to our website and click on the subscriptions menu, 
or contact the webmaster at [[admin]].\n
[[url]]\n\n"}, {
		type=>$this->{type},
		site=>$config{site}{name},
		day=>$this->{today},
		admin=>$config{site}{administrator},
		url=>$config{server}{server}});

    if (@articles > 0) {
	$digest .= $msg{"Article Topics"}.":\n";
	my $nart = 0;
	foreach my $title (@atitles) {
	    $nart++;
	    $digest .= "   $nart. $title\n";
	}
	$digest .= "\n";
    }

    if (@comments > 0) {
	$digest .= $msg{"Comment Topics"}.":\n";
	my $nart = 0;
	foreach my $title (@ctitles) {
	    $nart++;
	    $digest .= "   $nart. $title\n";
	}
	$digest .= "\n";
    }

    if (@articles > 0) {
	$digest .= "====================================================================\n\n";
	$digest .= $msg{ARTICLES}."\n\n";
	$digest .= join("\n--------------------------------------------------------------------\n\n",@articles);
    }
    
    if (@comments > 0) {
	$digest .= "====================================================================\n\n";
	$digest .= $msg{COMMENTS}."\n\n";
	$digest .= join("\n--------------------------------------------------------------------\n\n",@comments);
    }
	$digest .= "====================================================================\n\n";    
	$digest .= $this->{config}{digest}{footer};
    return $digest;
}

sub send_digest {
    my $this = shift;

    # get message body
    my $message = $this->make_digest();
    return undef if (! $message);

    # get recipients
    my @addresses = $this->get_addresses();
    return undef if (@addresses == 0);

    # get sender
    my $from = $this->{config}{notification_from} || $config{site}{administrator};

    # send!
    foreach my $to (@addresses) {
	&SendMail($to,
		  $from,
		  "$config{site}{name} $this->{type} digest",
		  $message);
    }

    return scalar @addresses;
}

# display a complete article in plaintext format
#
# Can optionally pass a page object, which is a page that contains
# the Zine plug-in;  this is used for making links to the HTML version 
# of the article.  If not provided, no such links will be included.

sub show_as_text {
    my ($this) = @_;
    return undef if (! $this->approve);
    # build the article structure up from component elements

    my $title = $this->title;
    my $author = $this->author;
    my $date = $this->date;
    my $body = $this->html_to_plaintext($this->{article}{body});
    my $picture = $this->picture ? 1 : 0;
    my $attachment = $this->attachment ? 1 : 0;
    my $ref = $this->{reference}{title};

    my $out = &substitute($msg{"Posted in: [[ref]]
Title:     [[title]]
Author:    [[author]]
Date:      [[date]]

[[body]]\n"},{ref=>$ref, title=>$title, author=>$author, date=>$date, body=>$body}) ;

    if ($this->{page}) {
	$out .= "\n(";
        my $message;
	if ($picture || $attachment) {
	    $message .= "This [[type]] includes";
	    if ($picture) {
		$message .= " a picture";
	    }
	    if ($picture && $attachment) {
		$message .= " and";
	    }
	    if ($attachment) {
		$message .= " attachments";
	    }
	    $message .= "; ";
	}
	my $uri = $this->{page}->get_uri_dynamic();
	$uri->plaintext;
	$uri->query(aid=>$this->{article_id});
	my $url = $uri->write_full;
	$message .= "Click to view: [[url]]";
        $out .= &substitute($msg{$message},{type=>$this->{article}{type},url=>$url});
        $out .= " )\n";
    }
    return $out;
}

# slightly different rules from ExSite::Misc::html_to_plaintext
sub html_to_plaintext {
    my $this = shift;
    $_ = shift;

    # escape conversion

    my %htmlesc = (
		   quot => '"',
		   amp => '&',
		   lt => '<',
		   gt => '>',
		   trade => '(TM)',
		   copy => '(C)',
		   reg => '(R)',
		   nbsp => ' ',
		   );

    s/<br>/\n/ig;                         # convert <br> to linebreak
    s/<p( .+?)?>/\n\n/ig;                 # convert <p> to double-linebreak
    s/<.+?>/ /g;                          # remove all other tags
    s/ +/ /g;                             # collapse multiple spaces
    s/\&\#(\d+);/chr($1)/ge;              # convert escape characters
    s/\&(\w+);/$htmlesc{$1}/ge;

    return $_;
}

1;
