#!/usr/bin/perl

package Modules::Zine::IndexSemithreaded;

=pod

=head1 Modules::Zine::IndexSemithreaded

This index uses a table format for the top-level items, and a threaded
or list format for the individual threads.

=cut

use strict;
use ExSite::Config;
use ExSite::Tree;
use Modules::Zine::IndexThreaded;

use vars qw(@ISA);
@ISA = qw(Modules::Zine::IndexThreaded);

sub index {
    my ($this,%opt) = @_;
    my $tree = $this->fetch_index(%opt);
    return undef if (! $tree);
    if ($this->class() eq "zine") {
	# semi-threaded indexes show thread heads at the top level
	return $this->index_semithreaded(\%opt,$tree);
    }
    else {
	# show fully threaded inside a thread
	my $ml = &get_obj("ML");
	return $ml->ul($this->index_threaded(\%opt,$tree),{class=>"ZineThread"});
    }
}

sub index_semithreaded {
    my ($this,$opt,$tree) = @_;
    my $stat = $this->handler("ZineIndexSemiThreaded",$opt,$tree);
    return $stat if ($stat);

    my $out;
    my $ml = &get_obj("ML");
    my $mode = $opt->{mode};
    my $indexage = $opt->{indexage} || $this->indexage($mode);
    $this->{archived_threads} = 0;
    $this->{active_threads} = 0;

    # prune uninteresting articles from the thread tree
    my @articles;
    foreach my $art (reverse $tree->get_child_data($this->{article_id})) {
        # ignore attachments
        next if ($art->{type} eq "attachment");
	# drop invalid type
	if ($art->{type} ne $mode) {
	    next;
	}
        # prune old threads
        if ($indexage > 0 && $this->age_of_thread($tree,$art) > $indexage) {
	    $this->{archived_threads}++;
            next;
        }
        if ($art->{status} eq "archived") {
	    $this->{archived_threads}++;
            next;
        }
	$this->{active_threads}++;
        push @articles,$art;
    }
    $this->{active_threads} = scalar @articles;

    my $count_type = $opt->{mode} eq "article" ? "Articles" : "Replies";
    my $rpt = new ExSite::ReportBuilder(
			headers=>[$msg{Thread},$msg{Author},$msg{"Last Post"},$msg{$count_type}],
			class=>"ZineIndex",
			);

    $this->save;
    foreach my $art (@articles) {
        $this->set($art);
	# count replies
	my @clist = $tree->collapse($this->{article_id});
        my $count = 0;
        foreach my $c (@clist) { 
	    if ($c->{type} eq $opt->{mode} &&
		$c->{status} eq "active") { 
		$count++; 
	    } 
	}
        my $url = $this->link(zine=>"show",aid=>$this->{article_id},cc=>$count);
        my $title = $this->title;
        my $author = $this->author;
	$this->save;
	$this->set($this->youngest_of_thread($tree,$art));
	my $date = $this->about("simple");
	$this->restore;
	# new posts flag
	my %link = ( href => $url );
	if ($this->{config}{new_age}) {
	    if ($this->age_of_thread($tree,$art) <= $this->{config}{new_age}) {
		$link{class} = "ZineNew";
	    }
	}
        $rpt->push($ml->a($title,\%link),$author,$date,$count);
    }
    $this->restore;
    return (@articles > 0) ? $rpt->make : undef;
}

sub index_threaded {
    my ($this,$opt,$tree) = @_;
    my $mode = $opt->{mode};
    my $indexlimit = $opt->{indexlimit} || $this->indexlimit($mode);
    my $indexage = $opt->{indexage} || $this->indexage($mode);
    my $indexformat = $opt->{indexformat} || $this->indexformat($mode);
    $this->{archived_threads} = 0;
    $this->{active_threads} = 0;
    $this->{thread_depth} = 0;
    my @toparticles = $tree->get_child_data($this->{article_id});
    return $this->index_thread($indexformat,$indexlimit,$indexage,
			       $tree,@toparticles);
}

1;

