#!/usr/bin/perl

package Modules::Zine::IndexWiki;

=pod

=head1 Modules::Zine::WikiIndex

This is a plug-in indexing engine to display article indexed by a keyword.

This index style is used by Wikis, which also permit some degree of
user editing/contribution to the database.  ExSite wikis allow users
to contibute their own articles under a keyword, rather than edit
someone else's.  All articles under a keyword are shown, so the
aggregate of the articles may reflect multiple points of view.

The method of fetching index articles is unique, but the display of
the articles is otherwise normal, defaulting to the Basic style.

=cut

use strict;
use ExSite::Config;
use ExSite::Misc;
use Modules::Zine::Index;

use vars qw(@ISA);
@ISA = qw(Modules::Zine::Index);

sub index {
    my ($this,%opt) = @_;
    my $in = new ExSite::Input;
    my $q = $in->query;
    $opt{goto} = lc $q->{goto};
    my $ml = &get_obj("ML");

    # get all the articles in the index
    my @articles = $this->fetch_index(%opt);

    my $url = $this->link(goto=>undef,zine=>undef);
    my $form = new ExSite::FormBuilder(method=>"get",action=>$url);
    $form->input(type=>"hidden",name=>"zine",value=>"show");
    $form->input(type=>"hidden",name=>"aid",value=>$this->{input}{aid});
    $form->input(type=>"hidden",name=>"_id",value=>$this->{input}{_id});
    $form->input(type=>"text",prompt=>"",name=>"goto",size=>10,required=>1,value=>$this->{input}{goto});
    $form->set("buttons"," ");
    $form->template($ml->p(
			   "[[goto:input]]".
			   $ml->input(undef,{type=>"submit",value=>$msg{"go!"}})
			   ));
    my $out = $form->make();
    $out .= $this->index_wiki(\%opt,@articles);
    return $out;
}

sub fetch_index {
    my ($this,%opt) = @_;
    my $mode = $opt{mode} || "article";
    if ($mode ne "article") {
	# wiki indexes only apply to articles; otherwise revert to Basic
	return Modules::Zine::Index::fetch_index(%opt);
    }

    my @wart;
    my $goto = $opt{term} || $opt{goto};
    if ($goto) {
	my $wiki_id = $this->type eq "wiki" ?
	    $this->{article_id} : $this->{reference}{article_id};
	$goto =~ s/\s/_/g;

	@wart = $this->fetch_match({reference=>$wiki_id,
				    title=>"%$goto%",
				    status=>"active"},
				   "title");
	if (@wart == 0 && $goto =~ /s$/) {
	    # hmm, no hits;  maybe we should drop the "s"
	    my $singular_goto = $goto;
	    $singular_goto =~ s/s$//;
	    @wart = $this->fetch_match({reference=>$wiki_id,
					title=>"%$singular_goto%",
					status=>"active"},
				       "title");
	}
    }
    return wantarray ? @wart : \@wart;
}

sub index_wiki {
    my ($this,$opt,@article) = @_;
    my ($out,$body,%refs,$start);
    $this->{nart} = 0;
    my $ml = &get_obj("ML");

    my $wiki_id = $this->type eq "wiki" ?
	$this->{article_id} : $this->{reference}{article_id};
    $this->save;

    if (@article == 0) {
	if ($opt->{goto}) {
	    $out .= $ml->p(&substitute($msg{"No articles found for \"[[goto]]\".  Use the new article link to create one."},{goto=>$opt->{goto}}),{class=>"ZineWarning"});
	}
	@article = $this->get_start();
    }

    foreach my $wart (@article) {
	$this->set($wart);
	$out .= $this->show_item("full");
	#$out .= $this->show_article();
	$this->{nart}++;
	my $st_body = $this->{article}{body};
	$body =~ s//$this->link_to_name($1,$3)/egs;
	while ($st_body =~ /\(\((.+?)(\|(.+?))?\)\)/) {
	    my $item = $1;
	    $refs{$item} = 1;
	    $st_body =~ s/\(\($item(\|(.+?))?\)\)//g;
	}
    }
    
    $this->restore;

    if (keys %refs > 0) { 
	my $index;
	foreach my $ref (sort keys %refs) {
	    my $url = $this->link(goto=>$ref);
	    $index .= $ml->li($ml->a($ref,{href=>$url}));
	}
	$out .= $ml->h3($msg{"Related articles:"}).
	    $ml->ul($index);
    }

    # TODO: should we also show the articles that link _to_ us?

    return $out;
}

# get top article of wiki, if no article was requested

sub get_start {
    my $this = shift;
    my $wiki_id = $this->type eq "wiki" ?
	$this->{article_id} : $this->{reference}{article_id};
    my @wart = $this->fetch_match({reference=>$wiki_id,
				   name=>"_start",
				   status=>"active"},
				  "title");
    return wantarray ? @wart : \@wart;
}

#sub show_full {
#    my ($this,$tree) = @_;
#    if (! $this->approve) {
#	my $type = $this->type;
#	return $this->error($msg{"Sorry, you don't have permission to view this $type."});
#    }
#    my $ml = &get_obj("ML");
#    my $out = $this->show_article;
#    my $comment_count = $this->count($this->{article_id},"comment",$tree);
#    if ($comment_count) {
#	my $url = $this->link(zine=>"show",aid=>$this->{article_id});
#	$out .= "(";
#	my $comment_type;
#	if ($comment_count > 1) { 
#	    $comment_type = ($this->class() eq "article") ? 
#		"comments" : "replies";
#	}
#	else {
#	    $comment_type = ($this->class() eq "article") ? 
#		"comment" : "reply";
#	}
#	$out .= "$comment_count $msg{$comment_type}; ";
#	$out .= "(" . $ml->a($msg{"Read more..."},{href=>$url}) . ")";
#    }
#    return $out;
#}

# link to a wiki article by name
#
# Unlike normal name-based linking, in which we try to find an exact
# match in the name field, with wikis we do a substring match on the
# title.

sub link_to_name {
    my ($this,$name,$anchor) = @_;
    my $ml = &get_obj("ML");
    if (! $anchor) { $anchor = $name; }
    $name = &html_to_plaintext($name);
    #$name =~ s/\s/_/g;
    my $wiki_id = $this->type eq "wiki" ?
	$this->{article_id} : $this->{reference}{article_id};
    my $url = $this->link(zine=>"show",aid=>$wiki_id,goto=>$name);
    return $ml->a($anchor,{href=>$url});
}

sub archive_count {
    my $this = shift;
    return 0;
}

sub article_count {
    my $this = shift;
    # always return a non-zero value so we get the search form in the
    # event of a failed article lookup
    return $this->{nart} || 1;
}

1;

